@@ -14,17 +14,23 @@
 #include "nsStringFwd.h"
 
 class nsIContent;
-class nsIDocument;
 class nsIFrame;
 struct JSContext;
 
 namespace mozilla {
 
+enum class PseudoStyleType : uint8_t;
 class ComputedTimingFunction;
 class EffectSet;
 
+namespace dom {
+class Document;
+}
+
 class AnimationUtils {
  public:
+  typedef dom::Document Document;
+
   static dom::Nullable<double> TimeDurationToDouble(
       const dom::Nullable<TimeDuration>& aTime) {
     dom::Nullable<double> result;
@@ -34,8 +40,8 @@ class AnimationUtils {
       // needs to have it's Time Reduction Logic refactored, so it's currently
       // only clamping for RFP mode. RFP mode gives a much lower time precision,
       // so we accept the security leak here for now
-      result.SetValue(nsRFPService::ReduceTimePrecisionAsMSecs(
-          aTime.Value().ToMilliseconds(), 0, TimerPrecisionType::RFPOnly));
+      result.SetValue(nsRFPService::ReduceTimePrecisionAsMSecsRFPOnly(
+          aTime.Value().ToMilliseconds(), 0));
     }
 
     return result;
@@ -58,26 +64,26 @@ class AnimationUtils {
   /**
    * Get the document from the JS context to use when parsing CSS properties.
    */
-  static nsIDocument* GetCurrentRealmDocument(JSContext* aCx);
+  static Document* GetCurrentRealmDocument(JSContext* aCx);
 
   /**
    * Get the document from the global object, or nullptr if the document has
    * no window, to use when constructing DOM object without entering the
    * target window's compartment (see KeyframeEffect constructor).
    */
-  static nsIDocument* GetDocumentFromGlobal(JSObject* aGlobalObject);
+  static Document* GetDocumentFromGlobal(JSObject* aGlobalObject);
 
   /**
-   * Checks if offscreen animation throttling is enabled.
+   * Returns true if the given frame has an animated scale.
    */
-  static bool IsOffscreenThrottlingEnabled();
+  static bool FrameHasAnimatedScale(const nsIFrame* aFrame);
 
   /**
-   * Returns true if the given EffectSet contains a current effect that animates
-   * scale. |aFrame| is used for calculation of scale values.
+   * Returns true if the given (pseudo-)element has any transitions that are
+   * current (playing or waiting to play) or in effect (e.g. filling forwards).
    */
-  static bool EffectSetContainsAnimatedScale(EffectSet& aEffects,
-                                             const nsIFrame* aFrame);
+  static bool HasCurrentTransitions(const dom::Element* aElement,
+                                    PseudoStyleType aPseudoType);
 };
 
 }  // namespace mozilla
