@@ -20,26 +20,50 @@
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/ServoBindingTypes.h"
 #include "mozilla/ServoUtils.h"
-#include "mozilla/DeclarationBlockInlines.h"
+#include "mozilla/ShadowParts.h"
+#include "mozilla/DeclarationBlock.h"
 #include "nsContentUtils.h"
 #include "nsReadableUtils.h"
 #include "nsHTMLCSSStyleSheet.h"
-#include "nsCSSParser.h"
 #include "nsStyledElement.h"
 #include "nsIURI.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
+#include "ReferrerInfo.h"
 #include <algorithm>
 
-#ifdef LoadImage
-// Undefine LoadImage to prevent naming conflict with Windows.
-#undef LoadImage
-#endif
-
 using namespace mozilla;
 
 #define MISC_STR_PTR(_cont) \
   reinterpret_cast<void*>((_cont)->mStringBits & NS_ATTRVALUE_POINTERVALUE_MASK)
 
+/* static */
+MiscContainer* nsAttrValue::AllocMiscContainer() {
+  MOZ_ASSERT(NS_IsMainThread());
+  MiscContainer* cont = nullptr;
+  std::swap(cont, sMiscContainerCache);
+
+  if (cont) {
+    return new (cont) MiscContainer;
+  }
+
+  return new MiscContainer;
+}
+
+/* static */
+void nsAttrValue::DeallocMiscContainer(MiscContainer* aCont) {
+  MOZ_ASSERT(NS_IsMainThread());
+  if (!aCont) {
+    return;
+  }
+
+  if (!sMiscContainerCache) {
+    aCont->~MiscContainer();
+    sMiscContainerCache = aCont;
+  } else {
+    delete aCont;
+  }
+}
+
 bool MiscContainer::GetString(nsAString& aString) const {
   void* ptr = MISC_STR_PTR(this);
 
@@ -122,6 +146,7 @@ void MiscContainer::Evict() {
 }
 
 nsTArray<const nsAttrValue::EnumTable*>* nsAttrValue::sEnumTableArray = nullptr;
+MiscContainer* nsAttrValue::sMiscContainerCache = nullptr;
 
 nsAttrValue::nsAttrValue() : mBits(0) {}
 
@@ -136,7 +161,7 @@ nsAttrValue::nsAttrValue(nsAtom* aValue) : mBits(0) { SetTo(aValue); }
 nsAttrValue::nsAttrValue(already_AddRefed<DeclarationBlock> aValue,
                          const nsAString* aSerialized)
     : mBits(0) {
-  SetTo(Move(aValue), aSerialized);
+  SetTo(std::move(aValue), aSerialized);
 }
 
 nsAttrValue::nsAttrValue(const nsIntMargin& aValue) : mBits(0) {
@@ -146,16 +171,21 @@ nsAttrValue::nsAttrValue(const nsIntMargin& aValue) : mBits(0) {
 nsAttrValue::~nsAttrValue() { ResetIfSet(); }
 
 /* static */
-nsresult nsAttrValue::Init() {
-  NS_ASSERTION(!sEnumTableArray, "nsAttrValue already initialized");
+void nsAttrValue::Init() {
+  MOZ_ASSERT(!sEnumTableArray, "nsAttrValue already initialized");
   sEnumTableArray = new nsTArray<const EnumTable*>;
-  return NS_OK;
 }
 
 /* static */
 void nsAttrValue::Shutdown() {
+  MOZ_ASSERT(NS_IsMainThread());
   delete sEnumTableArray;
   sEnumTableArray = nullptr;
+  // The MiscContainer pointed to by sMiscContainerCache has already
+  // be destructed so `delete sMiscContainerCache` is
+  // dangerous. Invoke `operator delete` to free the memory.
+  ::operator delete(sMiscContainerCache);
+  sMiscContainerCache = nullptr;
 }
 
 void nsAttrValue::Reset() {
@@ -175,7 +205,7 @@ void nsAttrValue::Reset() {
         break;
       }
 
-      delete ClearMiscContainer();
+      DeallocMiscContainer(ClearMiscContainer());
 
       break;
     }
@@ -227,7 +257,7 @@ void nsAttrValue::SetTo(const nsAttrValue& aOther) {
 
   MiscContainer* otherCont = aOther.GetMiscContainer();
   if (otherCont->IsRefCounted()) {
-    delete ClearMiscContainer();
+    DeallocMiscContainer(ClearMiscContainer());
     NS_ADDREF(otherCont);
     SetPtrValueAndType(otherCont, eOtherBase);
     return;
@@ -244,13 +274,14 @@ void nsAttrValue::SetTo(const nsAttrValue& aOther) {
       break;
     }
     case ePercent: {
-      cont->mValue.mPercent = otherCont->mValue.mPercent;
+      cont->mDoubleValue = otherCont->mDoubleValue;
       break;
     }
     case eColor: {
       cont->mValue.mColor = otherCont->mValue.mColor;
       break;
     }
+    case eShadowParts:
     case eCSSDeclaration: {
       MOZ_CRASH("These should be refcounted!");
     }
@@ -258,16 +289,14 @@ void nsAttrValue::SetTo(const nsAttrValue& aOther) {
       NS_ADDREF(cont->mValue.mURL = otherCont->mValue.mURL);
       break;
     }
-    case eImage: {
-      NS_ADDREF(cont->mValue.mImage = otherCont->mValue.mImage);
-      break;
-    }
     case eAtomArray: {
-      if (!EnsureEmptyAtomArray() ||
-          !GetAtomArrayValue()->AppendElements(*otherCont->mValue.mAtomArray)) {
+      if (!EnsureEmptyAtomArray()) {
         Reset();
         return;
       }
+      // XXX(Bug 1631371) Check if this should use a fallible operation as it
+      // pretended earlier.
+      GetAtomArrayValue()->AppendElements(*otherCont->mValue.mAtomArray);
       break;
     }
     case eDoubleValue: {
@@ -275,18 +304,19 @@ void nsAttrValue::SetTo(const nsAttrValue& aOther) {
       break;
     }
     case eIntMarginValue: {
-      if (otherCont->mValue.mIntMargin)
+      if (otherCont->mValue.mIntMargin) {
         cont->mValue.mIntMargin =
             new nsIntMargin(*otherCont->mValue.mIntMargin);
+      }
       break;
     }
     default: {
       if (IsSVGType(otherCont->mType)) {
         // All SVG types are just pointers to classes and will therefore have
         // the same size so it doesn't really matter which one we assign
-        cont->mValue.mSVGAngle = otherCont->mValue.mSVGAngle;
+        cont->mValue.mSVGLength = otherCont->mValue.mSVGLength;
       } else {
-        NS_NOTREACHED("unknown type stored in MiscContainer");
+        MOZ_ASSERT_UNREACHABLE("unknown type stored in MiscContainer");
       }
       break;
     }
@@ -351,7 +381,7 @@ void nsAttrValue::SetTo(already_AddRefed<DeclarationBlock> aValue,
   MOZ_ASSERT(cont->mValue.mRefCount == 1);
 }
 
-void nsAttrValue::SetTo(css::URLValue* aValue, const nsAString* aSerialized) {
+void nsAttrValue::SetTo(nsIURI* aValue, const nsAString* aSerialized) {
   MiscContainer* cont = EnsureEmptyMiscContainer();
   NS_ADDREF(cont->mValue.mURL = aValue);
   cont->mType = eURL;
@@ -375,17 +405,17 @@ void nsAttrValue::SetToSerialized(const nsAttrValue& aOther) {
   }
 }
 
-void nsAttrValue::SetTo(const nsSVGAngle& aValue,
+void nsAttrValue::SetTo(const SVGAnimatedOrient& aValue,
                         const nsAString* aSerialized) {
-  SetSVGType(eSVGAngle, &aValue, aSerialized);
+  SetSVGType(eSVGOrient, &aValue, aSerialized);
 }
 
-void nsAttrValue::SetTo(const nsSVGIntegerPair& aValue,
+void nsAttrValue::SetTo(const SVGAnimatedIntegerPair& aValue,
                         const nsAString* aSerialized) {
   SetSVGType(eSVGIntegerPair, &aValue, aSerialized);
 }
 
-void nsAttrValue::SetTo(const nsSVGLength2& aValue,
+void nsAttrValue::SetTo(const SVGAnimatedLength& aValue,
                         const nsAString* aSerialized) {
   SetSVGType(eSVGLength, &aValue, aSerialized);
 }
@@ -410,7 +440,7 @@ void nsAttrValue::SetTo(const SVGNumberList& aValue,
   SetSVGType(eSVGNumberList, &aValue, aSerialized);
 }
 
-void nsAttrValue::SetTo(const nsSVGNumberPair& aValue,
+void nsAttrValue::SetTo(const SVGAnimatedNumberPair& aValue,
                         const nsAString* aSerialized) {
   SetSVGType(eSVGNumberPair, &aValue, aSerialized);
 }
@@ -460,7 +490,7 @@ void nsAttrValue::SetTo(const SVGTransformList& aValue,
   SetSVGType(eSVGTransformList, &aValue, aSerialized);
 }
 
-void nsAttrValue::SetTo(const nsSVGViewBox& aValue,
+void nsAttrValue::SetTo(const SVGAnimatedViewBox& aValue,
                         const nsAString* aSerialized) {
   SetSVGType(eSVGViewBox, &aValue, aSerialized);
 }
@@ -506,7 +536,7 @@ void nsAttrValue::ToString(nsAString& aResult) const {
     }
 #ifdef DEBUG
     case eColor: {
-      NS_NOTREACHED("color attribute without string data");
+      MOZ_ASSERT_UNREACHABLE("color attribute without string data");
       aResult.Truncate();
       break;
     }
@@ -516,9 +546,13 @@ void nsAttrValue::ToString(nsAString& aResult) const {
       break;
     }
     case ePercent: {
-      nsAutoString intStr;
-      intStr.AppendInt(cont ? cont->mValue.mPercent : GetIntInternal());
-      aResult = intStr + NS_LITERAL_STRING("%");
+      nsAutoString str;
+      if (cont) {
+        str.AppendFloat(cont->mDoubleValue);
+      } else {
+        str.AppendInt(GetIntInternal());
+      }
+      aResult = str + NS_LITERAL_STRING("%");
 
       break;
     }
@@ -541,14 +575,14 @@ void nsAttrValue::ToString(nsAString& aResult) const {
       aResult.AppendFloat(GetDoubleValue());
       break;
     }
-    case eSVGAngle: {
-      SVGAttrValueWrapper::ToString(GetMiscContainer()->mValue.mSVGAngle,
-                                    aResult);
+    case eSVGIntegerPair: {
+      SVGAttrValueWrapper::ToString(
+          GetMiscContainer()->mValue.mSVGAnimatedIntegerPair, aResult);
       break;
     }
-    case eSVGIntegerPair: {
-      SVGAttrValueWrapper::ToString(GetMiscContainer()->mValue.mSVGIntegerPair,
-                                    aResult);
+    case eSVGOrient: {
+      SVGAttrValueWrapper::ToString(
+          GetMiscContainer()->mValue.mSVGAnimatedOrient, aResult);
       break;
     }
     case eSVGLength: {
@@ -567,8 +601,8 @@ void nsAttrValue::ToString(nsAString& aResult) const {
       break;
     }
     case eSVGNumberPair: {
-      SVGAttrValueWrapper::ToString(GetMiscContainer()->mValue.mSVGNumberPair,
-                                    aResult);
+      SVGAttrValueWrapper::ToString(
+          GetMiscContainer()->mValue.mSVGAnimatedNumberPair, aResult);
       break;
     }
     case eSVGPathData: {
@@ -583,7 +617,7 @@ void nsAttrValue::ToString(nsAString& aResult) const {
     }
     case eSVGPreserveAspectRatio: {
       SVGAttrValueWrapper::ToString(
-          GetMiscContainer()->mValue.mSVGPreserveAspectRatio, aResult);
+          GetMiscContainer()->mValue.mSVGAnimatedPreserveAspectRatio, aResult);
       break;
     }
     case eSVGStringList: {
@@ -597,8 +631,8 @@ void nsAttrValue::ToString(nsAString& aResult) const {
       break;
     }
     case eSVGViewBox: {
-      SVGAttrValueWrapper::ToString(GetMiscContainer()->mValue.mSVGViewBox,
-                                    aResult);
+      SVGAttrValueWrapper::ToString(
+          GetMiscContainer()->mValue.mSVGAnimatedViewBox, aResult);
       break;
     }
     default: {
@@ -627,7 +661,7 @@ already_AddRefed<nsAtom> nsAttrValue::GetAsAtom() const {
 }
 
 const nsCheapString nsAttrValue::GetStringValue() const {
-  NS_PRECONDITION(Type() == eString, "wrong type");
+  MOZ_ASSERT(Type() == eString, "wrong type");
 
   return nsCheapString(static_cast<nsStringBuffer*>(GetPtr()));
 }
@@ -644,7 +678,7 @@ bool nsAttrValue::GetColorValue(nscolor& aColor) const {
 }
 
 void nsAttrValue::GetEnumString(nsAString& aResult, bool aRealTag) const {
-  NS_PRECONDITION(Type() == eEnum, "wrong type");
+  MOZ_ASSERT(Type() == eEnum, "wrong type");
 
   uint32_t allEnumBits = (BaseType() == eIntegerBase)
                              ? static_cast<uint32_t>(GetIntInternal())
@@ -665,7 +699,7 @@ void nsAttrValue::GetEnumString(nsAString& aResult, bool aRealTag) const {
     table++;
   }
 
-  NS_NOTREACHED("couldn't find value in EnumTable");
+  MOZ_ASSERT_UNREACHABLE("couldn't find value in EnumTable");
 }
 
 uint32_t nsAttrValue::GetAtomCount() const {
@@ -683,8 +717,8 @@ uint32_t nsAttrValue::GetAtomCount() const {
 }
 
 nsAtom* nsAttrValue::AtomAt(int32_t aIndex) const {
-  NS_PRECONDITION(aIndex >= 0, "Index must not be negative");
-  NS_PRECONDITION(GetAtomCount() > uint32_t(aIndex), "aIndex out of range");
+  MOZ_ASSERT(aIndex >= 0, "Index must not be negative");
+  MOZ_ASSERT(GetAtomCount() > uint32_t(aIndex), "aIndex out of range");
 
   if (BaseType() == eAtomBase) {
     return GetAtomValue();
@@ -732,7 +766,7 @@ uint32_t nsAttrValue::HashValue() const {
       return cont->mValue.mEnumValue;
     }
     case ePercent: {
-      return cont->mValue.mPercent;
+      return cont->mDoubleValue;
     }
     case eColor: {
       return cont->mValue.mColor;
@@ -740,10 +774,7 @@ uint32_t nsAttrValue::HashValue() const {
     case eCSSDeclaration: {
       return NS_PTR_TO_INT32(cont->mValue.mCSSDeclaration);
     }
-    // Intentionally identical, so that loading the image does not change the
-    // hash code.
-    case eURL:
-    case eImage: {
+    case eURL: {
       nsString str;
       ToString(str);
       return HashString(str);
@@ -768,9 +799,9 @@ uint32_t nsAttrValue::HashValue() const {
     default: {
       if (IsSVGType(cont->mType)) {
         // All SVG types are just pointers to classes so we can treat them alike
-        return NS_PTR_TO_INT32(cont->mValue.mSVGAngle);
+        return NS_PTR_TO_INT32(cont->mValue.mSVGLength);
       }
-      NS_NOTREACHED("unknown type stored in MiscContainer");
+      MOZ_ASSERT_UNREACHABLE("unknown type stored in MiscContainer");
       return 0;
     }
   }
@@ -820,7 +851,7 @@ bool nsAttrValue::Equals(const nsAttrValue& aOther) const {
       break;
     }
     case ePercent: {
-      if (thisCont->mValue.mPercent == otherCont->mValue.mPercent) {
+      if (thisCont->mDoubleValue == otherCont->mDoubleValue) {
         needsStringComparison = true;
       }
       break;
@@ -838,9 +869,6 @@ bool nsAttrValue::Equals(const nsAttrValue& aOther) const {
     case eURL: {
       return thisCont->mValue.mURL == otherCont->mValue.mURL;
     }
-    case eImage: {
-      return thisCont->mValue.mImage == otherCont->mValue.mImage;
-    }
     case eAtomArray: {
       // For classlists we could be insensitive to order, however
       // classlists are never mapped attributes so they are never compared.
@@ -868,7 +896,7 @@ bool nsAttrValue::Equals(const nsAttrValue& aOther) const {
         MOZ_ASSERT(false, "Comparing nsAttrValues that point to SVG data");
         return false;
       }
-      NS_NOTREACHED("unknown type stored in MiscContainer");
+      MOZ_ASSERT_UNREACHABLE("unknown type stored in MiscContainer");
       return false;
     }
   }
@@ -922,7 +950,8 @@ bool nsAttrValue::Equals(const nsAString& aValue,
              : nsContentUtils::EqualsIgnoreASCIICase(val, aValue);
 }
 
-bool nsAttrValue::Equals(nsAtom* aValue, nsCaseTreatment aCaseSensitive) const {
+bool nsAttrValue::Equals(const nsAtom* aValue,
+                         nsCaseTreatment aCaseSensitive) const {
   if (aCaseSensitive != eCaseMatters) {
     // Need a better way to handle this!
     nsAutoString value;
@@ -952,6 +981,99 @@ bool nsAttrValue::Equals(nsAtom* aValue, nsCaseTreatment aCaseSensitive) const {
   return aValue->Equals(val);
 }
 
+struct HasPrefixFn {
+  static bool Check(const char16_t* aAttrValue, size_t aAttrLen,
+                    const nsAString& aSearchValue,
+                    nsCaseTreatment aCaseSensitive) {
+    if (aCaseSensitive == eCaseMatters) {
+      if (aSearchValue.Length() > aAttrLen) {
+        return false;
+      }
+      return !memcmp(aAttrValue, aSearchValue.BeginReading(),
+                     aSearchValue.Length() * sizeof(char16_t));
+    }
+    return StringBeginsWith(nsDependentString(aAttrValue, aAttrLen),
+                            aSearchValue,
+                            nsASCIICaseInsensitiveStringComparator);
+  }
+};
+
+struct HasSuffixFn {
+  static bool Check(const char16_t* aAttrValue, size_t aAttrLen,
+                    const nsAString& aSearchValue,
+                    nsCaseTreatment aCaseSensitive) {
+    if (aCaseSensitive == eCaseMatters) {
+      if (aSearchValue.Length() > aAttrLen) {
+        return false;
+      }
+      return !memcmp(aAttrValue + aAttrLen - aSearchValue.Length(),
+                     aSearchValue.BeginReading(),
+                     aSearchValue.Length() * sizeof(char16_t));
+    }
+    return StringEndsWith(nsDependentString(aAttrValue, aAttrLen), aSearchValue,
+                          nsASCIICaseInsensitiveStringComparator);
+  }
+};
+
+struct HasSubstringFn {
+  static bool Check(const char16_t* aAttrValue, size_t aAttrLen,
+                    const nsAString& aSearchValue,
+                    nsCaseTreatment aCaseSensitive) {
+    if (aCaseSensitive == eCaseMatters) {
+      if (aSearchValue.IsEmpty()) {
+        return true;
+      }
+      const char16_t* end = aAttrValue + aAttrLen;
+      return std::search(aAttrValue, end, aSearchValue.BeginReading(),
+                         aSearchValue.EndReading()) != end;
+    }
+    return FindInReadable(aSearchValue, nsDependentString(aAttrValue, aAttrLen),
+                          nsASCIICaseInsensitiveStringComparator);
+  }
+};
+
+template <typename F>
+bool nsAttrValue::SubstringCheck(const nsAString& aValue,
+                                 nsCaseTreatment aCaseSensitive) const {
+  switch (BaseType()) {
+    case eStringBase: {
+      auto str = static_cast<nsStringBuffer*>(GetPtr());
+      if (str) {
+        return F::Check(static_cast<char16_t*>(str->Data()),
+                        str->StorageSize() / sizeof(char16_t) - 1, aValue,
+                        aCaseSensitive);
+      }
+      return aValue.IsEmpty();
+    }
+    case eAtomBase: {
+      auto atom = static_cast<nsAtom*>(GetPtr());
+      return F::Check(atom->GetUTF16String(), atom->GetLength(), aValue,
+                      aCaseSensitive);
+    }
+    default:
+      break;
+  }
+
+  nsAutoString val;
+  ToString(val);
+  return F::Check(val.BeginReading(), val.Length(), aValue, aCaseSensitive);
+}
+
+bool nsAttrValue::HasPrefix(const nsAString& aValue,
+                            nsCaseTreatment aCaseSensitive) const {
+  return SubstringCheck<HasPrefixFn>(aValue, aCaseSensitive);
+}
+
+bool nsAttrValue::HasSuffix(const nsAString& aValue,
+                            nsCaseTreatment aCaseSensitive) const {
+  return SubstringCheck<HasSuffixFn>(aValue, aCaseSensitive);
+}
+
+bool nsAttrValue::HasSubstring(const nsAString& aValue,
+                               nsCaseTreatment aCaseSensitive) const {
+  return SubstringCheck<HasSubstringFn>(aValue, aCaseSensitive);
+}
+
 bool nsAttrValue::EqualsAsStrings(const nsAttrValue& aOther) const {
   if (Type() == aOther.Type()) {
     return Equals(aOther);
@@ -984,15 +1106,13 @@ bool nsAttrValue::Contains(nsAtom* aValue,
   switch (BaseType()) {
     case eAtomBase: {
       nsAtom* atom = GetAtomValue();
-
       if (aCaseSensitive == eCaseMatters) {
         return aValue == atom;
       }
 
       // For performance reasons, don't do a full on unicode case insensitive
       // string comparison. This is only used for quirks mode anyway.
-      return nsContentUtils::EqualsIgnoreASCIICase(
-          nsDependentAtomString(aValue), nsDependentAtomString(atom));
+      return nsContentUtils::EqualsIgnoreASCIICase(aValue, atom);
     }
     default: {
       if (Type() == eAtomArray) {
@@ -1001,16 +1121,11 @@ bool nsAttrValue::Contains(nsAtom* aValue,
           return array->Contains(aValue);
         }
 
-        nsDependentAtomString val1(aValue);
-
-        for (RefPtr<nsAtom>*cur = array->Elements(),
-            *end = cur + array->Length();
-             cur != end; ++cur) {
+        for (RefPtr<nsAtom>& cur : *array) {
           // For performance reasons, don't do a full on unicode case
           // insensitive string comparison. This is only used for quirks mode
           // anyway.
-          if (nsContentUtils::EqualsIgnoreASCIICase(
-                  val1, nsDependentAtomString(*cur))) {
+          if (nsContentUtils::EqualsIgnoreASCIICase(aValue, cur)) {
             return true;
           }
         }
@@ -1105,10 +1220,9 @@ void nsAttrValue::ParseAtomArray(const nsAString& aValue) {
 
   AtomArray* array = GetAtomArrayValue();
 
-  if (!array->AppendElement(Move(classAtom))) {
-    Reset();
-    return;
-  }
+  // XXX(Bug 1631371) Check if this should use a fallible operation as it
+  // pretended earlier.
+  array->AppendElement(std::move(classAtom));
 
   // parse the rest of the classnames
   while (iter != end) {
@@ -1120,10 +1234,9 @@ void nsAttrValue::ParseAtomArray(const nsAString& aValue) {
 
     classAtom = NS_AtomizeMainThread(Substring(start, iter));
 
-    if (!array->AppendElement(Move(classAtom))) {
-      Reset();
-      return;
-    }
+    // XXX(Bug 1631371) Check if this should use a fallible operation as it
+    // pretended earlier.
+    array->AppendElement(std::move(classAtom));
 
     // skip whitespace
     while (iter != end && nsContentUtils::IsHTMLWhitespace(*iter)) {
@@ -1137,7 +1250,7 @@ void nsAttrValue::ParseAtomArray(const nsAString& aValue) {
 void nsAttrValue::ParseStringOrAtom(const nsAString& aValue) {
   uint32_t len = aValue.Length();
   // Don't bother with atoms if it's an empty string since
-  // we can store those efficently anyway.
+  // we can store those efficiently anyway.
   if (len && len <= NS_ATTRVALUE_MAX_STRINGLENGTH_ATOM) {
     ParseAtom(aValue);
   } else {
@@ -1145,6 +1258,17 @@ void nsAttrValue::ParseStringOrAtom(const nsAString& aValue) {
   }
 }
 
+void nsAttrValue::ParsePartMapping(const nsAString& aValue) {
+  ResetIfSet();
+  MiscContainer* cont = EnsureEmptyMiscContainer();
+
+  cont->mType = eShadowParts;
+  cont->mValue.mShadowParts = new ShadowParts(ShadowParts::Parse(aValue));
+  NS_ADDREF(cont);
+  SetMiscAtomOrString(&aValue);
+  MOZ_ASSERT(cont->mValue.mRefCount == 1);
+}
+
 void nsAttrValue::SetIntValueAndType(int32_t aValue, ValueType aType,
                                      const nsAString* aStringValue) {
   if (aStringValue || aValue > NS_ATTRVALUE_INTEGERTYPE_MAXVALUE ||
@@ -1156,7 +1280,7 @@ void nsAttrValue::SetIntValueAndType(int32_t aValue, ValueType aType,
         break;
       }
       case ePercent: {
-        cont->mValue.mPercent = aValue;
+        cont->mDoubleValue = aValue;
         break;
       }
       case eEnum: {
@@ -1164,7 +1288,7 @@ void nsAttrValue::SetIntValueAndType(int32_t aValue, ValueType aType,
         break;
       }
       default: {
-        NS_NOTREACHED("unknown integer type");
+        MOZ_ASSERT_UNREACHABLE("unknown integer type");
         break;
       }
     }
@@ -1176,6 +1300,15 @@ void nsAttrValue::SetIntValueAndType(int32_t aValue, ValueType aType,
   }
 }
 
+void nsAttrValue::SetDoubleValueAndType(double aValue, ValueType aType,
+                                        const nsAString* aStringValue) {
+  MOZ_ASSERT(aType == eDoubleValue || aType == ePercent, "Unexpected type");
+  MiscContainer* cont = EnsureEmptyMiscContainer();
+  cont->mDoubleValue = aValue;
+  cont->mType = aType;
+  SetMiscAtomOrString(aStringValue);
+}
+
 int16_t nsAttrValue::GetEnumTableIndex(const EnumTable* aTable) {
   int16_t index = sEnumTableArray->IndexOf(aTable);
   if (index < 0) {
@@ -1226,8 +1359,8 @@ bool nsAttrValue::ParseEnumValue(const nsAString& aValue,
   }
 
   if (aDefaultValue) {
-    NS_PRECONDITION(aTable <= aDefaultValue && aDefaultValue < tableEntry,
-                    "aDefaultValue not inside aTable?");
+    MOZ_ASSERT(aTable <= aDefaultValue && aDefaultValue < tableEntry,
+               "aDefaultValue not inside aTable?");
     SetIntValueAndType(EnumTableEntryToValue(aTable, aDefaultValue), eEnum,
                        &aValue);
     return true;
@@ -1236,37 +1369,132 @@ bool nsAttrValue::ParseEnumValue(const nsAString& aValue,
   return false;
 }
 
-bool nsAttrValue::ParseSpecialIntValue(const nsAString& aString) {
+bool nsAttrValue::DoParseHTMLDimension(const nsAString& aInput,
+                                       bool aEnsureNonzero) {
   ResetIfSet();
 
-  nsAutoString tmp(aString);
-  nsContentUtils::ParseHTMLIntegerResultFlags result;
-  int32_t originalVal = nsContentUtils::ParseHTMLInteger(aString, &result);
+  // We don't use nsContentUtils::ParseHTMLInteger here because we
+  // need a bunch of behavioral differences from it.  We _could_ try to
+  // use it, but it would not be a great fit.
 
-  if (result & nsContentUtils::eParseHTMLInteger_Error) {
+  // https://html.spec.whatwg.org/multipage/#rules-for-parsing-dimension-values
+
+  // Steps 1 and 2.
+  const char16_t* position = aInput.BeginReading();
+  const char16_t* end = aInput.EndReading();
+
+  // We will need to keep track of whether this was a canonical representation
+  // or not.  It's non-canonical if it has leading whitespace, leading '+',
+  // leading '0' characters, or trailing garbage.
+  bool canonical = true;
+
+  // Step 3.
+  while (position != end && nsContentUtils::IsHTMLWhitespace(*position)) {
+    canonical = false;  // Leading whitespace
+    ++position;
+  }
+
+  // Step 4.
+  if (position == end || *position < char16_t('0') ||
+      *position > char16_t('9')) {
     return false;
   }
 
-  bool isPercent = result & nsContentUtils::eParseHTMLInteger_IsPercent;
-  int32_t val = std::max(originalVal, 0);
-  bool nonStrict =
-      val != originalVal ||
-      (result & nsContentUtils::eParseHTMLInteger_NonStandard) ||
-      (result & nsContentUtils::eParseHTMLInteger_DidNotConsumeAllInput);
+  // Step 5.
+  CheckedInt32 value = 0;
 
-  // % (percent)
-  if (isPercent || tmp.RFindChar('%') >= 0) {
-    isPercent = true;
+  // Collect up leading '0' first to avoid extra branching in the main
+  // loop to set 'canonical' properly.
+  while (position != end && *position == char16_t('0')) {
+    canonical = false;  // Leading '0'
+    ++position;
   }
 
-  SetIntValueAndType(val, isPercent ? ePercent : eInteger,
-                     nonStrict ? &aString : nullptr);
+  // Now collect up other digits.
+  while (position != end && *position >= char16_t('0') &&
+         *position <= char16_t('9')) {
+    value = value * 10 + (*position - char16_t('0'));
+    if (!value.isValid()) {
+      // The spec assumes we can deal with arbitrary-size integers here, but we
+      // really can't.  If someone sets something too big, just bail out and
+      // ignore it.
+      return false;
+    }
+    ++position;
+  }
+
+  // Step 6 is implemented implicitly via the various "position != end" guards
+  // from this point on.
+
+  Maybe<double> doubleValue;
+  // Step 7.  The return in step 7.2 is handled by just falling through to the
+  // code below this block when we reach end of input or a non-digit, because
+  // the while loop will terminate at that point.
+  if (position != end && *position == char16_t('.')) {
+    canonical = false;  // Let's not rely on double serialization reproducing
+                        // the string we started with.
+    // Step 7.1.
+    ++position;
+    // If we have a '.' _not_ followed by digits, this is not as efficient as it
+    // could be, because we will store as a double while we could have stored as
+    // an int.  But that seems like a pretty rare case.
+    doubleValue.emplace(value.value());
+    // Step 7.3.
+    double divisor = 1.0f;
+    // Step 7.4.
+    while (position != end && *position >= char16_t('0') &&
+           *position <= char16_t('9')) {
+      // Step 7.4.1.
+      divisor = divisor * 10.0f;
+      // Step 7.4.2.
+      doubleValue.ref() += (*position - char16_t('0')) / divisor;
+      // Step 7.4.3.
+      ++position;
+      // Step 7.4.4 and 7.4.5 are captured in the while loop condition and the
+      // "position != end" checks below.
+    }
+  }
+
+  if (aEnsureNonzero && value.value() == 0 &&
+      (!doubleValue || *doubleValue == 0.0f)) {
+    // Not valid.  Just drop it.
+    return false;
+  }
+
+  // Step 8 and the spec's early return from step 7.2.
+  ValueType type;
+  if (position != end && *position == char16_t('%')) {
+    type = ePercent;
+    ++position;
+  } else if (doubleValue) {
+    type = eDoubleValue;
+  } else {
+    type = eInteger;
+  }
+
+  if (position != end) {
+    canonical = false;
+  }
+
+  if (doubleValue) {
+    MOZ_ASSERT(!canonical, "We set it false above!");
+    SetDoubleValueAndType(*doubleValue, type, &aInput);
+  } else {
+    SetIntValueAndType(value.value(), type, canonical ? nullptr : &aInput);
+  }
+
+#ifdef DEBUG
+  nsAutoString str;
+  ToString(str);
+  MOZ_ASSERT(str == aInput, "We messed up our 'canonical' boolean!");
+#endif
+
   return true;
 }
 
 bool nsAttrValue::ParseIntWithBounds(const nsAString& aString, int32_t aMin,
                                      int32_t aMax) {
-  NS_PRECONDITION(aMin < aMax, "bad boundaries");
+  MOZ_ASSERT(aMin < aMax, "bad boundaries");
 
   ResetIfSet();
 
@@ -1280,7 +1508,6 @@ bool nsAttrValue::ParseIntWithBounds(const nsAString& aString, int32_t aMin,
   val = std::min(val, aMax);
   bool nonStrict =
       (val != originalVal) ||
-      (result & nsContentUtils::eParseHTMLInteger_IsPercent) ||
       (result & nsContentUtils::eParseHTMLInteger_NonStandard) ||
       (result & nsContentUtils::eParseHTMLInteger_DidNotConsumeAllInput);
 
@@ -1306,8 +1533,7 @@ void nsAttrValue::ParseIntWithFallback(const nsAString& aString,
     nonStrict = true;
   }
 
-  if ((result & nsContentUtils::eParseHTMLInteger_IsPercent) ||
-      (result & nsContentUtils::eParseHTMLInteger_NonStandard) ||
+  if ((result & nsContentUtils::eParseHTMLInteger_NonStandard) ||
       (result & nsContentUtils::eParseHTMLInteger_DidNotConsumeAllInput)) {
     nonStrict = true;
   }
@@ -1323,7 +1549,6 @@ void nsAttrValue::ParseClampedNonNegativeInt(const nsAString& aString,
   nsContentUtils::ParseHTMLIntegerResultFlags result;
   int32_t val = nsContentUtils::ParseHTMLInteger(aString, &result);
   bool nonStrict =
-      (result & nsContentUtils::eParseHTMLInteger_IsPercent) ||
       (result & nsContentUtils::eParseHTMLInteger_NonStandard) ||
       (result & nsContentUtils::eParseHTMLInteger_DidNotConsumeAllInput);
 
@@ -1358,7 +1583,6 @@ bool nsAttrValue::ParseNonNegativeIntValue(const nsAString& aString) {
   }
 
   bool nonStrict =
-      (result & nsContentUtils::eParseHTMLInteger_IsPercent) ||
       (result & nsContentUtils::eParseHTMLInteger_NonStandard) ||
       (result & nsContentUtils::eParseHTMLInteger_DidNotConsumeAllInput);
 
@@ -1377,7 +1601,6 @@ bool nsAttrValue::ParsePositiveIntValue(const nsAString& aString) {
   }
 
   bool nonStrict =
-      (result & nsContentUtils::eParseHTMLInteger_IsPercent) ||
       (result & nsContentUtils::eParseHTMLInteger_NonStandard) ||
       (result & nsContentUtils::eParseHTMLInteger_DidNotConsumeAllInput);
 
@@ -1474,39 +1697,19 @@ bool nsAttrValue::ParseIntMarginValue(const nsAString& aString) {
   return true;
 }
 
-void nsAttrValue::LoadImage(nsIDocument* aDocument) {
-  NS_ASSERTION(Type() == eURL, "wrong type");
-
-  MiscContainer* cont = GetMiscContainer();
-  mozilla::css::URLValue* url = cont->mValue.mURL;
-
-  NS_ASSERTION(!url->IsStringEmpty(),
-               "How did we end up with an empty string for eURL");
-
-  mozilla::css::ImageValue* image =
-      mozilla::css::ImageValue::CreateFromURLValue(
-          url, aDocument, mozilla::CORSMode::CORS_NONE);
-
-  NS_ADDREF(image);
-  cont->mValue.mImage = image;
-  NS_RELEASE(url);
-  cont->mType = eImage;
-}
-
 bool nsAttrValue::ParseStyleAttribute(const nsAString& aString,
                                       nsIPrincipal* aMaybeScriptedPrincipal,
                                       nsStyledElement* aElement) {
-  nsIDocument* ownerDoc = aElement->OwnerDoc();
+  dom::Document* ownerDoc = aElement->OwnerDoc();
   nsHTMLCSSStyleSheet* sheet = ownerDoc->GetInlineStyleSheet();
-  nsCOMPtr<nsIURI> baseURI = aElement->GetBaseURIForStyleAttr();
+  nsIURI* baseURI = aElement->GetBaseURIForStyleAttr();
   nsIURI* docURI = ownerDoc->GetDocumentURI();
 
   NS_ASSERTION(aElement->NodePrincipal() == ownerDoc->NodePrincipal(),
                "This is unexpected");
 
-  nsCOMPtr<nsIPrincipal> principal =
-      (aMaybeScriptedPrincipal ? aMaybeScriptedPrincipal
-                               : aElement->NodePrincipal());
+  nsIPrincipal* principal = aMaybeScriptedPrincipal ? aMaybeScriptedPrincipal
+                                                    : aElement->NodePrincipal();
 
   // If the (immutable) document URI does not match the element's base URI
   // (the common case is that they do match) do not cache the rule.  This is
@@ -1515,8 +1718,8 @@ bool nsAttrValue::ParseStyleAttribute(const nsAString& aString,
   // Similarly, if the triggering principal does not match the node principal,
   // do not cache the rule, since the principal will be encoded in any parsed
   // URLs in the rule.
-  bool cachingAllowed =
-      (sheet && baseURI == docURI && principal == aElement->NodePrincipal());
+  const bool cachingAllowed =
+      sheet && baseURI == docURI && principal == aElement->NodePrincipal();
   if (cachingAllowed) {
     MiscContainer* cont = sheet->LookupStyleAttr(aString);
     if (cont) {
@@ -1527,20 +1730,12 @@ bool nsAttrValue::ParseStyleAttribute(const nsAString& aString,
     }
   }
 
-  RefPtr<DeclarationBlock> decl;
-  if (ownerDoc->GetStyleBackendType() == StyleBackendType::Servo) {
-    RefPtr<URLExtraData> data = new URLExtraData(baseURI, docURI, principal);
-    decl = ServoDeclarationBlock::FromCssText(
-        aString, data, ownerDoc->GetCompatibilityMode(), ownerDoc->CSSLoader());
-  } else {
-#ifdef MOZ_OLD_STYLE
-    css::Loader* cssLoader = ownerDoc->CSSLoader();
-    nsCSSParser cssParser(cssLoader);
-    decl = cssParser.ParseStyleAttribute(aString, docURI, baseURI, principal);
-#else
-    MOZ_CRASH("old style system disabled");
-#endif
-  }
+  nsCOMPtr<nsIReferrerInfo> referrerInfo =
+      dom::ReferrerInfo::CreateForInternalCSSResources(ownerDoc);
+  RefPtr<URLExtraData> data =
+      new URLExtraData(baseURI, referrerInfo, principal);
+  RefPtr<DeclarationBlock> decl = DeclarationBlock::FromCssText(
+      aString, data, ownerDoc->GetCompatibilityMode(), ownerDoc->CSSLoader());
   if (!decl) {
     return false;
   }
@@ -1627,7 +1822,7 @@ void nsAttrValue::SetSVGType(ValueType aType, const void* aValue,
   // will do. We'll lose type-safety but the signature of the calling
   // function should ensure we don't get anything unexpected, and once we
   // stick aValue in a union we lose type information anyway.
-  cont->mValue.mSVGAngle = static_cast<const nsSVGAngle*>(aValue);
+  cont->mValue.mSVGLength = static_cast<const SVGAnimatedLength*>(aValue);
   cont->mType = aType;
   SetMiscAtomOrString(aSerialized);
 }
@@ -1640,7 +1835,7 @@ MiscContainer* nsAttrValue::ClearMiscContainer() {
       // This MiscContainer is shared, we need a new one.
       NS_RELEASE(cont);
 
-      cont = new MiscContainer;
+      cont = AllocMiscContainer();
       SetPtrValueAndType(cont, eOtherBase);
     } else {
       switch (cont->mType) {
@@ -1651,12 +1846,14 @@ MiscContainer* nsAttrValue::ClearMiscContainer() {
           NS_RELEASE(cont->mValue.mCSSDeclaration);
           break;
         }
-        case eURL: {
-          NS_RELEASE(cont->mValue.mURL);
+        case eShadowParts: {
+          MOZ_ASSERT(cont->mValue.mRefCount == 1);
+          cont->Release();
+          delete cont->mValue.mShadowParts;
           break;
         }
-        case eImage: {
-          NS_RELEASE(cont->mValue.mImage);
+        case eURL: {
+          NS_RELEASE(cont->mValue.mURL);
           break;
         }
         case eAtomArray: {
@@ -1667,7 +1864,9 @@ MiscContainer* nsAttrValue::ClearMiscContainer() {
           delete cont->mValue.mIntMargin;
           break;
         }
-        default: { break; }
+        default: {
+          break;
+        }
       }
     }
     ResetMiscAtomOrString();
@@ -1685,7 +1884,7 @@ MiscContainer* nsAttrValue::EnsureEmptyMiscContainer() {
     ResetMiscAtomOrString();
     cont = GetMiscContainer();
   } else {
-    cont = new MiscContainer;
+    cont = AllocMiscContainer();
     SetPtrValueAndType(cont, eOtherBase);
   }
 
@@ -1764,7 +1963,7 @@ size_t nsAttrValue::SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const {
       if (Type() == eCSSDeclaration && container->mValue.mCSSDeclaration) {
         // TODO: mCSSDeclaration might be owned by another object which
         //       would make us count them twice, bug 677493.
-        // Bug 1281964: For ServoDeclarationBlock if we do measure we'll
+        // Bug 1281964: For DeclarationBlock if we do measure we'll
         // need a way to call the Servo heap_size_of function.
         // n += container->mCSSDeclaration->SizeOfIncludingThis(aMallocSizeOf);
       } else if (Type() == eAtomArray && container->mValue.mAtomArray) {
