@@ -13,12 +13,9 @@
 #include "nsXMLContentSerializer.h"
 
 #include "nsGkAtoms.h"
-#include "nsIDOMProcessingInstruction.h"
-#include "nsIDOMComment.h"
-#include "nsIDOMDocumentType.h"
 #include "nsIContent.h"
 #include "nsIContentInlines.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsIDocumentEncoder.h"
 #include "nsElementTable.h"
 #include "nsNameSpaceManager.h"
@@ -29,7 +26,11 @@
 #include "nsCRT.h"
 #include "nsContentUtils.h"
 #include "nsAttrName.h"
+#include "mozilla/dom/Comment.h"
+#include "mozilla/dom/CustomElementRegistry.h"
+#include "mozilla/dom/DocumentType.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/dom/ProcessingInstruction.h"
 #include "mozilla/intl/LineBreaker.h"
 #include "nsParserConstants.h"
 #include "mozilla/Encoding.h"
@@ -66,7 +67,7 @@ nsXMLContentSerializer::nsXMLContentSerializer()
       mBodyOnly(false),
       mInBody(0) {}
 
-nsXMLContentSerializer::~nsXMLContentSerializer() {}
+nsXMLContentSerializer::~nsXMLContentSerializer() = default;
 
 NS_IMPL_ISUPPORTS(nsXMLContentSerializer, nsIContentSerializer)
 
@@ -74,7 +75,8 @@ NS_IMETHODIMP
 nsXMLContentSerializer::Init(uint32_t aFlags, uint32_t aWrapColumn,
                              const Encoding* aEncoding, bool aIsCopying,
                              bool aRewriteEncodingDeclaration,
-                             bool* aNeedsPreformatScanning) {
+                             bool* aNeedsPreformatScanning,
+                             nsAString& aOutput) {
   *aNeedsPreformatScanning = false;
   mPrefixIndex = 0;
   mColPos = 0;
@@ -119,6 +121,7 @@ nsXMLContentSerializer::Init(uint32_t aFlags, uint32_t aWrapColumn,
     mMaxColumn = aWrapColumn;
   }
 
+  mOutput = &aOutput;
   mPreLevel = 0;
   mIsIndentationAddedOnCurrentLine = false;
   return NS_OK;
@@ -162,16 +165,16 @@ nsresult nsXMLContentSerializer::AppendTextData(nsIContent* aNode,
                      NS_ERROR_OUT_OF_MEMORY);
     }
   } else {
+    nsAutoString utf16;
+    if (!CopyASCIItoUTF16(MakeSpan(frag->Get1b() + aStartOffset, length), utf16,
+                          mozilla::fallible_t())) {
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
     if (aTranslateEntities) {
-      NS_ENSURE_TRUE(
-          AppendAndTranslateEntities(
-              NS_ConvertASCIItoUTF16(frag->Get1b() + aStartOffset, length),
-              aStr),
-          NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendAndTranslateEntities(utf16, aStr),
+                     NS_ERROR_OUT_OF_MEMORY);
     } else {
-      NS_ENSURE_TRUE(aStr.Append(NS_ConvertASCIItoUTF16(
-                                     frag->Get1b() + aStartOffset, length),
-                                 mozilla::fallible),
+      NS_ENSURE_TRUE(aStr.Append(utf16, mozilla::fallible),
                      NS_ERROR_OUT_OF_MEMORY);
     }
   }
@@ -181,8 +184,9 @@ nsresult nsXMLContentSerializer::AppendTextData(nsIContent* aNode,
 
 NS_IMETHODIMP
 nsXMLContentSerializer::AppendText(nsIContent* aText, int32_t aStartOffset,
-                                   int32_t aEndOffset, nsAString& aStr) {
+                                   int32_t aEndOffset) {
   NS_ENSURE_ARG(aText);
+  NS_ENSURE_STATE(mOutput);
 
   nsAutoString data;
   nsresult rv;
@@ -191,14 +195,17 @@ nsXMLContentSerializer::AppendText(nsIContent* aText, int32_t aStartOffset,
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
   if (mDoRaw || PreLevel() > 0) {
-    NS_ENSURE_TRUE(AppendToStringConvertLF(data, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToStringConvertLF(data, *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
   } else if (mDoFormat) {
-    NS_ENSURE_TRUE(AppendToStringFormatedWrapped(data, aStr),
+    NS_ENSURE_TRUE(AppendToStringFormatedWrapped(data, *mOutput),
                    NS_ERROR_OUT_OF_MEMORY);
   } else if (mDoWrap) {
-    NS_ENSURE_TRUE(AppendToStringWrapped(data, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToStringWrapped(data, *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
   } else {
-    NS_ENSURE_TRUE(AppendToStringConvertLF(data, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToStringConvertLF(data, *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
   }
 
   return NS_OK;
@@ -207,53 +214,52 @@ nsXMLContentSerializer::AppendText(nsIContent* aText, int32_t aStartOffset,
 NS_IMETHODIMP
 nsXMLContentSerializer::AppendCDATASection(nsIContent* aCDATASection,
                                            int32_t aStartOffset,
-                                           int32_t aEndOffset,
-                                           nsAString& aStr) {
+                                           int32_t aEndOffset) {
   NS_ENSURE_ARG(aCDATASection);
+  NS_ENSURE_STATE(mOutput);
+
   nsresult rv;
 
   NS_NAMED_LITERAL_STRING(cdata, "<![CDATA[");
 
   if (mDoRaw || PreLevel() > 0) {
-    NS_ENSURE_TRUE(AppendToString(cdata, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(cdata, *mOutput), NS_ERROR_OUT_OF_MEMORY);
   } else if (mDoFormat) {
-    NS_ENSURE_TRUE(AppendToStringFormatedWrapped(cdata, aStr),
+    NS_ENSURE_TRUE(AppendToStringFormatedWrapped(cdata, *mOutput),
                    NS_ERROR_OUT_OF_MEMORY);
   } else if (mDoWrap) {
-    NS_ENSURE_TRUE(AppendToStringWrapped(cdata, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToStringWrapped(cdata, *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
   } else {
-    NS_ENSURE_TRUE(AppendToString(cdata, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(cdata, *mOutput), NS_ERROR_OUT_OF_MEMORY);
   }
 
   nsAutoString data;
   rv = AppendTextData(aCDATASection, aStartOffset, aEndOffset, data, false);
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-  NS_ENSURE_TRUE(AppendToStringConvertLF(data, aStr), NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(AppendToStringConvertLF(data, *mOutput),
+                 NS_ERROR_OUT_OF_MEMORY);
 
-  NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING("]]>"), aStr),
+  NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING("]]>"), *mOutput),
                  NS_ERROR_OUT_OF_MEMORY);
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXMLContentSerializer::AppendProcessingInstruction(nsIContent* aPI,
+nsXMLContentSerializer::AppendProcessingInstruction(ProcessingInstruction* aPI,
                                                     int32_t aStartOffset,
-                                                    int32_t aEndOffset,
-                                                    nsAString& aStr) {
-  nsCOMPtr<nsIDOMProcessingInstruction> pi = do_QueryInterface(aPI);
-  NS_ENSURE_ARG(pi);
-  nsresult rv;
+                                                    int32_t aEndOffset) {
+  NS_ENSURE_STATE(mOutput);
+
   nsAutoString target, data, start;
 
-  NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(aStr), NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(*mOutput), NS_ERROR_OUT_OF_MEMORY);
 
-  rv = pi->GetTarget(target);
-  if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+  aPI->GetTarget(target);
 
-  rv = pi->GetData(data);
-  if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+  aPI->GetData(data);
 
   NS_ENSURE_TRUE(start.AppendLiteral("<?", mozilla::fallible),
                  NS_ERROR_OUT_OF_MEMORY);
@@ -261,24 +267,27 @@ nsXMLContentSerializer::AppendProcessingInstruction(nsIContent* aPI,
                  NS_ERROR_OUT_OF_MEMORY);
 
   if (mDoRaw || PreLevel() > 0) {
-    NS_ENSURE_TRUE(AppendToString(start, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(start, *mOutput), NS_ERROR_OUT_OF_MEMORY);
   } else if (mDoFormat) {
     if (mAddSpace) {
-      NS_ENSURE_TRUE(AppendNewLineToString(aStr), NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendNewLineToString(*mOutput), NS_ERROR_OUT_OF_MEMORY);
     }
-    NS_ENSURE_TRUE(AppendToStringFormatedWrapped(start, aStr),
+    NS_ENSURE_TRUE(AppendToStringFormatedWrapped(start, *mOutput),
                    NS_ERROR_OUT_OF_MEMORY);
   } else if (mDoWrap) {
-    NS_ENSURE_TRUE(AppendToStringWrapped(start, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToStringWrapped(start, *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
   } else {
-    NS_ENSURE_TRUE(AppendToString(start, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(start, *mOutput), NS_ERROR_OUT_OF_MEMORY);
   }
 
   if (!data.IsEmpty()) {
-    NS_ENSURE_TRUE(AppendToString(char16_t(' '), aStr), NS_ERROR_OUT_OF_MEMORY);
-    NS_ENSURE_TRUE(AppendToStringConvertLF(data, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(char16_t(' '), *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToStringConvertLF(data, *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
   }
-  NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING("?>"), aStr),
+  NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING("?>"), *mOutput),
                  NS_ERROR_OUT_OF_MEMORY);
 
   MaybeFlagNewlineForRootNode(aPI);
@@ -287,16 +296,12 @@ nsXMLContentSerializer::AppendProcessingInstruction(nsIContent* aPI,
 }
 
 NS_IMETHODIMP
-nsXMLContentSerializer::AppendComment(nsIContent* aComment,
-                                      int32_t aStartOffset, int32_t aEndOffset,
-                                      nsAString& aStr) {
-  nsCOMPtr<nsIDOMComment> comment = do_QueryInterface(aComment);
-  NS_ENSURE_ARG(comment);
-  nsresult rv;
-  nsAutoString data;
+nsXMLContentSerializer::AppendComment(Comment* aComment, int32_t aStartOffset,
+                                      int32_t aEndOffset) {
+  NS_ENSURE_STATE(mOutput);
 
-  rv = comment->GetData(data);
-  if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+  nsAutoString data;
+  aComment->GetData(data);
 
   int32_t dataLength = data.Length();
   if (aStartOffset || (aEndOffset != -1 && aEndOffset < dataLength)) {
@@ -311,29 +316,32 @@ nsXMLContentSerializer::AppendComment(nsIContent* aComment,
     data.Assign(frag);
   }
 
-  NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(aStr), NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(*mOutput), NS_ERROR_OUT_OF_MEMORY);
 
   NS_NAMED_LITERAL_STRING(startComment, "<!--");
 
   if (mDoRaw || PreLevel() > 0) {
-    NS_ENSURE_TRUE(AppendToString(startComment, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(startComment, *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
   } else if (mDoFormat) {
     if (mAddSpace) {
-      NS_ENSURE_TRUE(AppendNewLineToString(aStr), NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendNewLineToString(*mOutput), NS_ERROR_OUT_OF_MEMORY);
     }
-    NS_ENSURE_TRUE(AppendToStringFormatedWrapped(startComment, aStr),
+    NS_ENSURE_TRUE(AppendToStringFormatedWrapped(startComment, *mOutput),
                    NS_ERROR_OUT_OF_MEMORY);
   } else if (mDoWrap) {
-    NS_ENSURE_TRUE(AppendToStringWrapped(startComment, aStr),
+    NS_ENSURE_TRUE(AppendToStringWrapped(startComment, *mOutput),
                    NS_ERROR_OUT_OF_MEMORY);
   } else {
-    NS_ENSURE_TRUE(AppendToString(startComment, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(startComment, *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
   }
 
   // Even if mDoformat, we don't format the content because it
   // could have been preformated by the author
-  NS_ENSURE_TRUE(AppendToStringConvertLF(data, aStr), NS_ERROR_OUT_OF_MEMORY);
-  NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING("-->"), aStr),
+  NS_ENSURE_TRUE(AppendToStringConvertLF(data, *mOutput),
+                 NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING("-->"), *mOutput),
                  NS_ERROR_OUT_OF_MEMORY);
 
   MaybeFlagNewlineForRootNode(aComment);
@@ -342,49 +350,45 @@ nsXMLContentSerializer::AppendComment(nsIContent* aComment,
 }
 
 NS_IMETHODIMP
-nsXMLContentSerializer::AppendDoctype(nsIContent* aDocType, nsAString& aStr) {
-  nsCOMPtr<nsIDOMDocumentType> docType = do_QueryInterface(aDocType);
-  NS_ENSURE_ARG(docType);
-  nsresult rv;
+nsXMLContentSerializer::AppendDoctype(DocumentType* aDocType) {
+  NS_ENSURE_STATE(mOutput);
+
   nsAutoString name, publicId, systemId;
+  aDocType->GetName(name);
+  aDocType->GetPublicId(publicId);
+  aDocType->GetSystemId(systemId);
 
-  rv = docType->GetName(name);
-  if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-  rv = docType->GetPublicId(publicId);
-  if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-  rv = docType->GetSystemId(systemId);
-  if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(*mOutput), NS_ERROR_OUT_OF_MEMORY);
 
-  NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(aStr), NS_ERROR_OUT_OF_MEMORY);
-
-  NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING("<!DOCTYPE "), aStr),
+  NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING("<!DOCTYPE "), *mOutput),
                  NS_ERROR_OUT_OF_MEMORY);
-  NS_ENSURE_TRUE(AppendToString(name, aStr), NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(AppendToString(name, *mOutput), NS_ERROR_OUT_OF_MEMORY);
 
   char16_t quote;
   if (!publicId.IsEmpty()) {
-    NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING(" PUBLIC "), aStr),
+    NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING(" PUBLIC "), *mOutput),
                    NS_ERROR_OUT_OF_MEMORY);
     if (publicId.FindChar(char16_t('"')) == -1) {
       quote = char16_t('"');
     } else {
       quote = char16_t('\'');
     }
-    NS_ENSURE_TRUE(AppendToString(quote, aStr), NS_ERROR_OUT_OF_MEMORY);
-    NS_ENSURE_TRUE(AppendToString(publicId, aStr), NS_ERROR_OUT_OF_MEMORY);
-    NS_ENSURE_TRUE(AppendToString(quote, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(quote, *mOutput), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(publicId, *mOutput), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(quote, *mOutput), NS_ERROR_OUT_OF_MEMORY);
 
     if (!systemId.IsEmpty()) {
-      NS_ENSURE_TRUE(AppendToString(char16_t(' '), aStr),
+      NS_ENSURE_TRUE(AppendToString(char16_t(' '), *mOutput),
                      NS_ERROR_OUT_OF_MEMORY);
       if (systemId.FindChar(char16_t('"')) == -1) {
         quote = char16_t('"');
       } else {
         quote = char16_t('\'');
       }
-      NS_ENSURE_TRUE(AppendToString(quote, aStr), NS_ERROR_OUT_OF_MEMORY);
-      NS_ENSURE_TRUE(AppendToString(systemId, aStr), NS_ERROR_OUT_OF_MEMORY);
-      NS_ENSURE_TRUE(AppendToString(quote, aStr), NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendToString(quote, *mOutput), NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendToString(systemId, *mOutput),
+                     NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendToString(quote, *mOutput), NS_ERROR_OUT_OF_MEMORY);
     }
   } else if (!systemId.IsEmpty()) {
     if (systemId.FindChar(char16_t('"')) == -1) {
@@ -392,14 +396,15 @@ nsXMLContentSerializer::AppendDoctype(nsIContent* aDocType, nsAString& aStr) {
     } else {
       quote = char16_t('\'');
     }
-    NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING(" SYSTEM "), aStr),
+    NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING(" SYSTEM "), *mOutput),
                    NS_ERROR_OUT_OF_MEMORY);
-    NS_ENSURE_TRUE(AppendToString(quote, aStr), NS_ERROR_OUT_OF_MEMORY);
-    NS_ENSURE_TRUE(AppendToString(systemId, aStr), NS_ERROR_OUT_OF_MEMORY);
-    NS_ENSURE_TRUE(AppendToString(quote, aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(quote, *mOutput), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(systemId, *mOutput), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(quote, *mOutput), NS_ERROR_OUT_OF_MEMORY);
   }
 
-  NS_ENSURE_TRUE(AppendToString(kGreaterThan, aStr), NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(AppendToString(kGreaterThan, *mOutput),
+                 NS_ERROR_OUT_OF_MEMORY);
   MaybeFlagNewlineForRootNode(aDocType);
 
   return NS_OK;
@@ -579,9 +584,7 @@ bool nsXMLContentSerializer::ConfirmPrefix(nsAString& aPrefix,
 
 void nsXMLContentSerializer::GenerateNewPrefix(nsAString& aPrefix) {
   aPrefix.Assign('a');
-  char buf[128];
-  SprintfLiteral(buf, "%d", mPrefixIndex++);
-  AppendASCIItoUTF16(buf, aPrefix);
+  aPrefix.AppendInt(mPrefixIndex++);
 }
 
 bool nsXMLContentSerializer::SerializeAttr(const nsAString& aPrefix,
@@ -764,6 +767,8 @@ bool nsXMLContentSerializer::SerializeAttributes(
   xmlnsStr.AssignLiteral(kXMLNS);
   uint32_t index, count;
 
+  MaybeSerializeIsValue(aElement, aStr);
+
   // If we had to add a new namespace declaration, serialize
   // and push it on the namespace stack
   if (aAddNSAttr) {
@@ -837,13 +842,13 @@ bool nsXMLContentSerializer::SerializeAttributes(
 
 NS_IMETHODIMP
 nsXMLContentSerializer::AppendElementStart(Element* aElement,
-                                           Element* aOriginalElement,
-                                           nsAString& aStr) {
+                                           Element* aOriginalElement) {
   NS_ENSURE_ARG(aElement);
+  NS_ENSURE_STATE(mOutput);
 
   bool forceFormat = false;
   nsresult rv = NS_OK;
-  if (!CheckElementStart(aElement, forceFormat, aStr, rv)) {
+  if (!CheckElementStart(aElement, forceFormat, *mOutput, rv)) {
     // When we go to AppendElementEnd for this element, we're going to
     // MaybeLeaveFromPreContent().  So make sure to MaybeEnterInPreContent()
     // now, so our PreLevel() doesn't get confused.
@@ -867,22 +872,25 @@ nsXMLContentSerializer::AppendElementStart(Element* aElement,
 
   if ((mDoFormat || forceFormat) && !mDoRaw && !PreLevel()) {
     if (mColPos && lineBreakBeforeOpen) {
-      NS_ENSURE_TRUE(AppendNewLineToString(aStr), NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendNewLineToString(*mOutput), NS_ERROR_OUT_OF_MEMORY);
     } else {
-      NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(aStr), NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(*mOutput),
+                     NS_ERROR_OUT_OF_MEMORY);
     }
     if (!mColPos) {
-      NS_ENSURE_TRUE(AppendIndentation(aStr), NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendIndentation(*mOutput), NS_ERROR_OUT_OF_MEMORY);
     } else if (mAddSpace) {
-      NS_ENSURE_TRUE(AppendToString(char16_t(' '), aStr),
+      NS_ENSURE_TRUE(AppendToString(char16_t(' '), *mOutput),
                      NS_ERROR_OUT_OF_MEMORY);
       mAddSpace = false;
     }
   } else if (mAddSpace) {
-    NS_ENSURE_TRUE(AppendToString(char16_t(' '), aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(char16_t(' '), *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
     mAddSpace = false;
   } else {
-    NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(MaybeAddNewlineForRootNode(*mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
   }
 
   // Always reset to avoid false newlines in case MaybeAddNewlineForRootNode
@@ -894,13 +902,14 @@ nsXMLContentSerializer::AppendElementStart(Element* aElement,
       ConfirmPrefix(tagPrefix, tagNamespaceURI, aOriginalElement, false);
 
   // Serialize the qualified name of the element
-  NS_ENSURE_TRUE(AppendToString(kLessThan, aStr), NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(AppendToString(kLessThan, *mOutput), NS_ERROR_OUT_OF_MEMORY);
   if (!tagPrefix.IsEmpty()) {
-    NS_ENSURE_TRUE(AppendToString(tagPrefix, aStr), NS_ERROR_OUT_OF_MEMORY);
-    NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING(":"), aStr),
+    NS_ENSURE_TRUE(AppendToString(tagPrefix, *mOutput), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING(":"), *mOutput),
                    NS_ERROR_OUT_OF_MEMORY);
   }
-  NS_ENSURE_TRUE(AppendToString(tagLocalName, aStr), NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(AppendToString(tagLocalName, *mOutput),
+                 NS_ERROR_OUT_OF_MEMORY);
 
   MaybeEnterInPreContent(aElement);
 
@@ -910,18 +919,18 @@ nsXMLContentSerializer::AppendElementStart(Element* aElement,
 
   NS_ENSURE_TRUE(
       SerializeAttributes(aElement, aOriginalElement, tagPrefix,
-                          tagNamespaceURI, name, aStr, skipAttr, addNSAttr),
+                          tagNamespaceURI, name, *mOutput, skipAttr, addNSAttr),
       NS_ERROR_OUT_OF_MEMORY);
 
-  NS_ENSURE_TRUE(AppendEndOfElementStart(aElement, aOriginalElement, aStr),
+  NS_ENSURE_TRUE(AppendEndOfElementStart(aElement, aOriginalElement, *mOutput),
                  NS_ERROR_OUT_OF_MEMORY);
 
   if ((mDoFormat || forceFormat) && !mDoRaw && !PreLevel() &&
       LineBreakAfterOpen(aElement->GetNameSpaceID(), name)) {
-    NS_ENSURE_TRUE(AppendNewLineToString(aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendNewLineToString(*mOutput), NS_ERROR_OUT_OF_MEMORY);
   }
 
-  NS_ENSURE_TRUE(AfterElementStart(aElement, aOriginalElement, aStr),
+  NS_ENSURE_TRUE(AfterElementStart(aElement, aOriginalElement, *mOutput),
                  NS_ERROR_OUT_OF_MEMORY);
 
   return NS_OK;
@@ -972,13 +981,16 @@ bool nsXMLContentSerializer::AppendEndOfElementStart(Element* aElement,
 }
 
 NS_IMETHODIMP
-nsXMLContentSerializer::AppendElementEnd(Element* aElement, nsAString& aStr) {
+nsXMLContentSerializer::AppendElementEnd(Element* aElement,
+                                         Element* aOriginalElement) {
   NS_ENSURE_ARG(aElement);
+  NS_ENSURE_STATE(mOutput);
 
   nsIContent* content = aElement;
 
   bool forceFormat = false, outputElementEnd;
-  outputElementEnd = CheckElementEnd(aElement, forceFormat, aStr);
+  outputElementEnd =
+      CheckElementEnd(aElement, aOriginalElement, forceFormat, *mOutput);
 
   nsAtom* name = content->NodeInfo()->NameAtom();
 
@@ -991,7 +1003,7 @@ nsXMLContentSerializer::AppendElementEnd(Element* aElement, nsAString& aStr) {
     PopNameSpaceDeclsFor(aElement);
     MaybeLeaveFromPreContent(content);
     MaybeFlagNewlineForRootNode(aElement);
-    AfterElementEnd(content, aStr);
+    AfterElementEnd(content, *mOutput);
     return NS_OK;
   }
 
@@ -1013,28 +1025,31 @@ nsXMLContentSerializer::AppendElementEnd(Element* aElement, nsAString& aStr) {
         LineBreakBeforeClose(content->GetNameSpaceID(), name);
 
     if (mColPos && lineBreakBeforeClose) {
-      NS_ENSURE_TRUE(AppendNewLineToString(aStr), NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendNewLineToString(*mOutput), NS_ERROR_OUT_OF_MEMORY);
     }
     if (!mColPos) {
-      NS_ENSURE_TRUE(AppendIndentation(aStr), NS_ERROR_OUT_OF_MEMORY);
+      NS_ENSURE_TRUE(AppendIndentation(*mOutput), NS_ERROR_OUT_OF_MEMORY);
     } else if (mAddSpace) {
-      NS_ENSURE_TRUE(AppendToString(char16_t(' '), aStr),
+      NS_ENSURE_TRUE(AppendToString(char16_t(' '), *mOutput),
                      NS_ERROR_OUT_OF_MEMORY);
       mAddSpace = false;
     }
   } else if (mAddSpace) {
-    NS_ENSURE_TRUE(AppendToString(char16_t(' '), aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(char16_t(' '), *mOutput),
+                   NS_ERROR_OUT_OF_MEMORY);
     mAddSpace = false;
   }
 
-  NS_ENSURE_TRUE(AppendToString(kEndTag, aStr), NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(AppendToString(kEndTag, *mOutput), NS_ERROR_OUT_OF_MEMORY);
   if (!tagPrefix.IsEmpty()) {
-    NS_ENSURE_TRUE(AppendToString(tagPrefix, aStr), NS_ERROR_OUT_OF_MEMORY);
-    NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING(":"), aStr),
+    NS_ENSURE_TRUE(AppendToString(tagPrefix, *mOutput), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendToString(NS_LITERAL_STRING(":"), *mOutput),
                    NS_ERROR_OUT_OF_MEMORY);
   }
-  NS_ENSURE_TRUE(AppendToString(tagLocalName, aStr), NS_ERROR_OUT_OF_MEMORY);
-  NS_ENSURE_TRUE(AppendToString(kGreaterThan, aStr), NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(AppendToString(tagLocalName, *mOutput),
+                 NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(AppendToString(kGreaterThan, *mOutput),
+                 NS_ERROR_OUT_OF_MEMORY);
 
   // Keep what follows in sync with the cleanup in the !outputElementEnd case.
   PopNameSpaceDeclsFor(aElement);
@@ -1043,20 +1058,38 @@ nsXMLContentSerializer::AppendElementEnd(Element* aElement, nsAString& aStr) {
 
   if ((mDoFormat || forceFormat) && !mDoRaw && !PreLevel() &&
       LineBreakAfterClose(content->GetNameSpaceID(), name)) {
-    NS_ENSURE_TRUE(AppendNewLineToString(aStr), NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(AppendNewLineToString(*mOutput), NS_ERROR_OUT_OF_MEMORY);
   } else {
     MaybeFlagNewlineForRootNode(aElement);
   }
 
-  AfterElementEnd(content, aStr);
+  AfterElementEnd(content, *mOutput);
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXMLContentSerializer::AppendDocumentStart(nsIDocument* aDocument,
-                                            nsAString& aStr) {
+nsXMLContentSerializer::Finish() {
+  NS_ENSURE_STATE(mOutput);
+
+  mOutput = nullptr;
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXMLContentSerializer::GetOutputLength(uint32_t& aLength) const {
+  NS_ENSURE_STATE(mOutput);
+
+  aLength = mOutput->Length();
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXMLContentSerializer::AppendDocumentStart(Document* aDocument) {
   NS_ENSURE_ARG_POINTER(aDocument);
+  NS_ENSURE_STATE(mOutput);
 
   nsAutoString version, encoding, standalone;
   aDocument->GetXMLDeclaration(version, encoding, standalone);
@@ -1066,14 +1099,14 @@ nsXMLContentSerializer::AppendDocumentStart(nsIDocument* aDocument,
 
   NS_NAMED_LITERAL_STRING(endQuote, "\"");
 
-  aStr += NS_LITERAL_STRING("<?xml version=\"") + version + endQuote;
+  *mOutput += NS_LITERAL_STRING("<?xml version=\"") + version + endQuote;
 
   if (!mCharset.IsEmpty()) {
-    aStr += NS_LITERAL_STRING(" encoding=\"") +
-            NS_ConvertASCIItoUTF16(mCharset) + endQuote;
+    *mOutput += NS_LITERAL_STRING(" encoding=\"") +
+                NS_ConvertASCIItoUTF16(mCharset) + endQuote;
   }
-    // Otherwise just don't output an encoding attr.  Not that we expect
-    // mCharset to ever be empty.
+  // Otherwise just don't output an encoding attr.  Not that we expect
+  // mCharset to ever be empty.
 #ifdef DEBUG
   else {
     NS_WARNING("Empty mCharset?  How come?");
@@ -1081,10 +1114,10 @@ nsXMLContentSerializer::AppendDocumentStart(nsIDocument* aDocument,
 #endif
 
   if (!standalone.IsEmpty()) {
-    aStr += NS_LITERAL_STRING(" standalone=\"") + standalone + endQuote;
+    *mOutput += NS_LITERAL_STRING(" standalone=\"") + standalone + endQuote;
   }
 
-  NS_ENSURE_TRUE(aStr.AppendLiteral("?>", mozilla::fallible),
+  NS_ENSURE_TRUE(mOutput->AppendLiteral("?>", mozilla::fallible),
                  NS_ERROR_OUT_OF_MEMORY);
   mAddNewlineForRootNode = true;
 
@@ -1100,15 +1133,12 @@ bool nsXMLContentSerializer::CheckElementStart(Element*, bool& aForceFormat,
 }
 
 bool nsXMLContentSerializer::CheckElementEnd(Element* aElement,
+                                             Element* aOriginalElement,
                                              bool& aForceFormat,
                                              nsAString& aStr) {
   // We don't output a separate end tag for empty element
   aForceFormat = false;
-
-  // XXXbz this is a bit messed up, but by now we don't have our fixed-up
-  // version of aElement anymore.  Let's hope fixup never changes the localName
-  // or namespace...
-  return ElementNeedsSeparateEndTag(aElement, aElement);
+  return ElementNeedsSeparateEndTag(aElement, aOriginalElement);
 }
 
 bool nsXMLContentSerializer::AppendToString(const char16_t aChar,
@@ -1129,8 +1159,6 @@ bool nsXMLContentSerializer::AppendToString(const nsAString& aStr,
   return aOutputStr.Append(aStr, mozilla::fallible);
 }
 
-static const uint16_t kGTVal = 62;
-
 #define _ 0
 
 // This table indexes into kEntityStrings[].
@@ -1174,6 +1202,19 @@ static const char* const kEntityStrings[] = {
 
 bool nsXMLContentSerializer::AppendAndTranslateEntities(const nsAString& aStr,
                                                         nsAString& aOutputStr) {
+  if (mInAttribute) {
+    return AppendAndTranslateEntities<kGTVal>(aStr, aOutputStr, kAttrEntities,
+                                              kEntityStrings);
+  }
+
+  return AppendAndTranslateEntities<kGTVal>(aStr, aOutputStr, kEntities,
+                                            kEntityStrings);
+}
+
+/* static */
+bool nsXMLContentSerializer::AppendAndTranslateEntities(
+    const nsAString& aStr, nsAString& aOutputStr, const uint8_t aEntityTable[],
+    uint16_t aMaxTableIndex, const char* const aStringTable[]) {
   nsReadingIterator<char16_t> done_reading;
   aStr.EndReading(done_reading);
 
@@ -1181,8 +1222,6 @@ bool nsXMLContentSerializer::AppendAndTranslateEntities(const nsAString& aStr,
   uint32_t advanceLength = 0;
   nsReadingIterator<char16_t> iter;
 
-  const uint8_t* entityTable = mInAttribute ? kAttrEntities : kEntities;
-
   for (aStr.BeginReading(iter); iter != done_reading;
        iter.advance(int32_t(advanceLength))) {
     uint32_t fragmentLength = done_reading - iter;
@@ -1196,8 +1235,8 @@ bool nsXMLContentSerializer::AppendAndTranslateEntities(const nsAString& aStr,
     // needs to be replaced
     for (; c < fragmentEnd; c++, advanceLength++) {
       char16_t val = *c;
-      if ((val <= kGTVal) && entityTable[val]) {
-        entityText = kEntityStrings[entityTable[val]];
+      if ((val <= aMaxTableIndex) && aEntityTable[val]) {
+        entityText = aStringTable[aEntityTable[val]];
         break;
       }
     }
@@ -1206,8 +1245,9 @@ bool nsXMLContentSerializer::AppendAndTranslateEntities(const nsAString& aStr,
         aOutputStr.Append(fragmentStart, advanceLength, mozilla::fallible),
         false);
     if (entityText) {
-      NS_ENSURE_TRUE(
-          AppendASCIItoUTF16(entityText, aOutputStr, mozilla::fallible), false);
+      NS_ENSURE_TRUE(AppendASCIItoUTF16(mozilla::MakeStringSpan(entityText),
+                                        aOutputStr, mozilla::fallible),
+                     false);
       advanceLength++;
     }
   }
@@ -1226,7 +1266,7 @@ bool nsXMLContentSerializer::MaybeAddNewlineForRootNode(nsAString& aStr) {
 void nsXMLContentSerializer::MaybeFlagNewlineForRootNode(nsINode* aNode) {
   nsINode* parent = aNode->GetParentNode();
   if (parent) {
-    mAddNewlineForRootNode = parent->IsNodeOfType(nsINode::eDOCUMENT);
+    mAddNewlineForRootNode = parent->IsDocument();
   }
 }
 
@@ -1359,7 +1399,7 @@ bool nsXMLContentSerializer::AppendFormatedWrapped_WhitespaceSequence(
       case ' ':
       case '\t':
         sawBlankOrTab = true;
-        MOZ_FALLTHROUGH;
+        [[fallthrough]];
       case '\n':
         ++aPos;
         // do not increase mColPos,
@@ -1731,3 +1771,19 @@ bool nsXMLContentSerializer::ShouldMaintainPreLevel() const {
   // Only attempt to maintain the pre level for consumers who care about it.
   return !mDoRaw || (mFlags & nsIDocumentEncoder::OutputNoFormattingInPre);
 }
+
+bool nsXMLContentSerializer::MaybeSerializeIsValue(Element* aElement,
+                                                   nsAString& aStr) {
+  CustomElementData* ceData = aElement->GetCustomElementData();
+  if (ceData) {
+    nsAtom* isAttr = ceData->GetIs(aElement);
+    if (isAttr && !aElement->HasAttr(kNameSpaceID_None, nsGkAtoms::is)) {
+      NS_ENSURE_TRUE(aStr.AppendLiteral(" is=\"", mozilla::fallible), false);
+      NS_ENSURE_TRUE(
+          aStr.Append(nsDependentAtomString(isAttr), mozilla::fallible), false);
+      NS_ENSURE_TRUE(aStr.AppendLiteral("\"", mozilla::fallible), false);
+    }
+  }
+
+  return true;
+}
