@@ -187,8 +187,8 @@ class EventStates {
 // Content is URL's target (ref).
 #define NS_EVENT_STATE_URLTARGET NS_DEFINE_EVENT_STATE_MACRO(8)
 // Content is the full screen element, or a frame containing the
-// current full-screen element.
-#define NS_EVENT_STATE_FULL_SCREEN NS_DEFINE_EVENT_STATE_MACRO(9)
+// current fullscreen element.
+#define NS_EVENT_STATE_FULLSCREEN NS_DEFINE_EVENT_STATE_MACRO(9)
 // Content is valid (and can be invalid).
 #define NS_EVENT_STATE_VALID NS_DEFINE_EVENT_STATE_MACRO(10)
 // Content is invalid.
@@ -216,7 +216,8 @@ class EventStates {
 #define NS_EVENT_STATE_REQUIRED NS_DEFINE_EVENT_STATE_MACRO(21)
 // Content is optional (and can be required).
 #define NS_EVENT_STATE_OPTIONAL NS_DEFINE_EVENT_STATE_MACRO(22)
-// Free bit                          NS_DEFINE_EVENT_STATE_MACRO(23)
+// Element is either a defined custom element or uncustomized element.
+#define NS_EVENT_STATE_DEFINED NS_DEFINE_EVENT_STATE_MACRO(23)
 // Link has been visited.
 #define NS_EVENT_STATE_VISITED NS_DEFINE_EVENT_STATE_MACRO(24)
 // Link hasn't been visited.
@@ -227,11 +228,12 @@ class EventStates {
 #define NS_EVENT_STATE_INRANGE NS_DEFINE_EVENT_STATE_MACRO(27)
 // Content value is out-of-range.
 #define NS_EVENT_STATE_OUTOFRANGE NS_DEFINE_EVENT_STATE_MACRO(28)
-// These two are temporary (see bug 302188)
 // Content is read-only.
-#define NS_EVENT_STATE_MOZ_READONLY NS_DEFINE_EVENT_STATE_MACRO(29)
+// TODO(emilio): This is always the inverse of READWRITE. With some style system
+// work we could remove one of the two bits.
+#define NS_EVENT_STATE_READONLY NS_DEFINE_EVENT_STATE_MACRO(29)
 // Content is editable.
-#define NS_EVENT_STATE_MOZ_READWRITE NS_DEFINE_EVENT_STATE_MACRO(30)
+#define NS_EVENT_STATE_READWRITE NS_DEFINE_EVENT_STATE_MACRO(30)
 // Content is the default one (meaning depends of the context).
 #define NS_EVENT_STATE_DEFAULT NS_DEFINE_EVENT_STATE_MACRO(31)
 // Content is a submit control and the form isn't valid.
@@ -286,9 +288,12 @@ class EventStates {
 #define NS_EVENT_STATE_AUTOFILL NS_DEFINE_EVENT_STATE_MACRO(50)
 // Element is filled with preview data by Autofill feature.
 #define NS_EVENT_STATE_AUTOFILL_PREVIEW NS_DEFINE_EVENT_STATE_MACRO(51)
-
-// Event state that is used for values that need to be parsed but do nothing.
-#define NS_EVENT_STATE_IGNORE NS_DEFINE_EVENT_STATE_MACRO(63)
+// Element matches the :focus-visible pseudo-class.
+//
+// TODO(emilio): We should eventually unify this and FOCUSRING.
+#define NS_EVENT_STATE_FOCUS_VISIBLE NS_DEFINE_EVENT_STATE_MACRO(52)
+// Modal <dialog> element
+#define NS_EVENT_STATE_MODAL_DIALOG NS_DEFINE_EVENT_STATE_MACRO(53)
 
 /**
  * NOTE: do not go over 63 without updating EventStates::InternalType!
@@ -318,12 +323,13 @@ class EventStates {
 // EventStateManager, or by other code).  As opposed to those in
 // INTRINSIC_STATES, which are are computed by the element itself
 // and returned from Element::IntrinsicState.
-#define EXTERNALLY_MANAGED_STATES                                      \
-  (MANUALLY_MANAGED_STATES | DIR_ATTR_STATES | DISABLED_STATES |       \
-   REQUIRED_STATES | NS_EVENT_STATE_ACTIVE | NS_EVENT_STATE_DRAGOVER | \
-   NS_EVENT_STATE_FOCUS | NS_EVENT_STATE_FOCUSRING |                   \
-   NS_EVENT_STATE_FOCUS_WITHIN | NS_EVENT_STATE_FULL_SCREEN |          \
-   NS_EVENT_STATE_HOVER | NS_EVENT_STATE_URLTARGET)
+#define EXTERNALLY_MANAGED_STATES                                              \
+  (MANUALLY_MANAGED_STATES | DIR_ATTR_STATES | DISABLED_STATES |               \
+   REQUIRED_STATES | NS_EVENT_STATE_ACTIVE | NS_EVENT_STATE_DEFINED |          \
+   NS_EVENT_STATE_DRAGOVER | NS_EVENT_STATE_FOCUS | NS_EVENT_STATE_FOCUSRING | \
+   NS_EVENT_STATE_FOCUS_WITHIN | NS_EVENT_STATE_FULLSCREEN |                   \
+   NS_EVENT_STATE_HOVER | NS_EVENT_STATE_URLTARGET |                           \
+   NS_EVENT_STATE_FOCUS_VISIBLE | NS_EVENT_STATE_MODAL_DIALOG)
 
 #define INTRINSIC_STATES (~EXTERNALLY_MANAGED_STATES)
 
