@@ -6,19 +6,18 @@
 #include "mozilla/dom/PGamepadEventChannelChild.h"
 
 #ifndef mozilla_dom_GamepadEventChannelChild_h_
-#define mozilla_dom_GamepadEventChannelChild_h_
+#  define mozilla_dom_GamepadEventChannelChild_h_
 
 namespace mozilla {
 namespace dom {
 
 class GamepadEventChannelChild final : public PGamepadEventChannelChild {
  public:
-  GamepadEventChannelChild() {}
-  ~GamepadEventChannelChild() {}
-  virtual mozilla::ipc::IPCResult RecvGamepadUpdate(
-      const GamepadChangeEvent& aGamepadEvent) override;
-  virtual mozilla::ipc::IPCResult RecvReplyGamepadVibrateHaptic(
-      const uint32_t& aPromiseID) override;
+  GamepadEventChannelChild() = default;
+  ~GamepadEventChannelChild() = default;
+  mozilla::ipc::IPCResult RecvGamepadUpdate(
+      const GamepadChangeEvent& aGamepadEvent);
+  mozilla::ipc::IPCResult RecvReplyGamepadPromise(const uint32_t& aPromiseID);
   void AddPromise(const uint32_t& aID, dom::Promise* aPromise);
 
  private:
