@@ -14,16 +14,16 @@ namespace mozilla {
 namespace dom {
 
 HTMLDataElement::HTMLDataElement(
-    already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo)
-    : nsGenericHTMLElement(aNodeInfo) {}
+    already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo)
+    : nsGenericHTMLElement(std::move(aNodeInfo)) {}
 
-HTMLDataElement::~HTMLDataElement() {}
+HTMLDataElement::~HTMLDataElement() = default;
 
 NS_IMPL_ELEMENT_CLONE(HTMLDataElement)
 
 JSObject* HTMLDataElement::WrapNode(JSContext* aCx,
                                     JS::Handle<JSObject*> aGivenProto) {
-  return HTMLDataElementBinding::Wrap(aCx, this, aGivenProto);
+  return HTMLDataElement_Binding::Wrap(aCx, this, aGivenProto);
 }
 
 }  // namespace dom
