@@ -9,13 +9,13 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/EventForwards.h"
 #include "nsMappedAttributeElement.h"
-#include "nsIDOMElement.h"
 #include "nsNameSpaceManager.h"  // for kNameSpaceID_None
 #include "nsIFormControl.h"
 #include "nsGkAtoms.h"
 #include "nsContentCreatorFunctions.h"
 #include "mozilla/ErrorResult.h"
 #include "mozilla/dom/BindingDeclarations.h"
+#include "mozilla/dom/Element.h"
 #include "mozilla/dom/DOMRect.h"
 #include "mozilla/dom/ValidityState.h"
 #include "mozilla/dom/Element.h"
@@ -25,7 +25,6 @@ class nsIFormControlFrame;
 class nsIFrame;
 class nsILayoutHistoryState;
 class nsIURI;
-class nsPresState;
 struct nsSize;
 
 namespace mozilla {
@@ -35,7 +34,9 @@ class EventChainVisitor;
 class EventListenerManager;
 class EventStates;
 class TextEditor;
+class PresState;
 namespace dom {
+class ElementInternals;
 class HTMLFormElement;
 class HTMLMenuElement;
 }  // namespace dom
@@ -46,26 +47,26 @@ typedef nsMappedAttributeElement nsGenericHTMLElementBase;
 /**
  * A common superclass for HTML elements
  */
-class nsGenericHTMLElement : public nsGenericHTMLElementBase,
-                             public nsIDOMElement {
+class nsGenericHTMLElement : public nsGenericHTMLElementBase {
  public:
   using Element::Focus;
   using Element::SetTabIndex;
   explicit nsGenericHTMLElement(
-      already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo)
-      : nsGenericHTMLElementBase(aNodeInfo) {
+      already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo)
+      : nsGenericHTMLElementBase(std::move(aNodeInfo)) {
     NS_ASSERTION(mNodeInfo->NamespaceID() == kNameSpaceID_XHTML,
                  "Unexpected namespace");
     AddStatesSilently(NS_EVENT_STATE_LTR);
     SetFlags(NODE_HAS_DIRECTION_LTR);
   }
 
-  NS_DECL_ISUPPORTS_INHERITED
+  NS_INLINE_DECL_REFCOUNTING_INHERITED(nsGenericHTMLElement,
+                                       nsGenericHTMLElementBase)
 
-  NS_IMPL_FROMCONTENT(nsGenericHTMLElement, kNameSpaceID_XHTML)
+  NS_IMPL_FROMNODE(nsGenericHTMLElement, kNameSpaceID_XHTML)
 
   // From Element
-  nsresult CopyInnerTo(mozilla::dom::Element* aDest, bool aPreallocateChildren);
+  nsresult CopyInnerTo(mozilla::dom::Element* aDest);
 
   void GetTitle(mozilla::dom::DOMString& aTitle) {
     GetHTMLAttr(nsGkAtoms::title, aTitle);
@@ -129,7 +130,7 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
   }
   bool IsContentEditable() {
     for (nsIContent* node = this; node; node = node->GetParent()) {
-      nsGenericHTMLElement* element = FromContent(node);
+      nsGenericHTMLElement* element = FromNode(node);
       if (element) {
         ContentEditableTristate value = element->GetContentEditableValue();
         if (value != eInherit) {
@@ -140,6 +141,18 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
     return false;
   }
 
+  void SetNonce(const nsAString& aNonce) {
+    SetProperty(nsGkAtoms::nonce, new nsString(aNonce),
+                nsINode::DeleteProperty<nsString>);
+  }
+  void RemoveNonce() { RemoveProperty(nsGkAtoms::nonce); }
+  void GetNonce(nsAString& aNonce) const {
+    nsString* cspNonce = static_cast<nsString*>(GetProperty(nsGkAtoms::nonce));
+    if (cspNonce) {
+      aNonce = *cspNonce;
+    }
+  }
+
   /**
    * Returns the count of descendants (inclusive of this node) in
    * the uncomposed document that are explicitly set as editable.
@@ -160,6 +173,13 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
                     mozilla::ErrorResult& aError);
   void SetInnerText(const nsAString& aValue);
 
+  void GetInputMode(nsAString& aValue) {
+    GetEnumAttr(nsGkAtoms::inputmode, nullptr, aValue);
+  }
+  void SetInputMode(const nsAString& aValue, ErrorResult& aRv) {
+    SetHTMLAttr(nsGkAtoms::inputmode, aValue, aRv);
+  }
+
   /**
    * Determine whether an attribute is an event (onclick, etc.)
    * @param aName the attribute
@@ -227,8 +247,12 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
     return IsNodeInternal(aFirst, aArgs...);
   }
 
+  // https://html.spec.whatwg.org/multipage/custom-elements.html#dom-attachinternals
+  already_AddRefed<mozilla::dom::ElementInternals> AttachInternals(
+      ErrorResult& aRv);
+
  protected:
-  virtual ~nsGenericHTMLElement() {}
+  virtual ~nsGenericHTMLElement() = default;
 
  public:
   /**
@@ -239,15 +263,10 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
   MOZ_CAN_RUN_SCRIPT
   nsSize GetWidthHeightForImage(RefPtr<imgRequestProxy>& aImageRequest);
 
-  virtual nsIDOMNode* AsDOMNode() override { return this; }
-
  public:
   // Implementation for nsIContent
-  virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
-                              nsIContent* aBindingParent,
-                              bool aCompileEventHandlers) override;
-  virtual void UnbindFromTree(bool aDeep = true,
-                              bool aNullParent = true) override;
+  virtual nsresult BindToTree(BindContext&, nsINode& aParent) override;
+  virtual void UnbindFromTree(bool aNullParent = true) override;
 
   virtual bool IsFocusableInternal(int32_t* aTabIndex,
                                    bool aWithMouse) override {
@@ -270,13 +289,13 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    */
   bool CheckHandleEventForAnchorsPreconditions(
       mozilla::EventChainVisitor& aVisitor);
-  nsresult GetEventTargetParentForAnchors(
-      mozilla::EventChainPreVisitor& aVisitor);
+  void GetEventTargetParentForAnchors(mozilla::EventChainPreVisitor& aVisitor);
+  MOZ_CAN_RUN_SCRIPT
   nsresult PostHandleEventForAnchors(mozilla::EventChainPostVisitor& aVisitor);
   bool IsHTMLLink(nsIURI** aURI) const;
 
   // HTML element methods
-  void Compact() { mAttrsAndChildren.Compact(); }
+  void Compact() { mAttrs.Compact(); }
 
   virtual void UpdateEditableState(bool aNotify) override;
 
@@ -418,9 +437,8 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapCommonAttributesInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapCommonAttributesInto(const nsMappedAttributes* aAttributes,
+                                      mozilla::MappedDeclarations&);
   /**
    * Same as MapCommonAttributesInto except that it does not handle hidden.
    *
@@ -429,8 +447,7 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @see GetAttributeMappingFunction
    */
   static void MapCommonAttributesIntoExceptHidden(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+      const nsMappedAttributes* aAttributes, mozilla::MappedDeclarations&);
 
   static const MappedAttributeEntry sCommonAttributeMap[];
   static const MappedAttributeEntry sImageMarginSizeAttributeMap[];
@@ -447,9 +464,8 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapImageAlignAttributeInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapImageAlignAttributeInto(const nsMappedAttributes* aAttributes,
+                                         mozilla::MappedDeclarations&);
 
   /**
    * Helper to map the align attribute into a style struct for things
@@ -459,9 +475,8 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapDivAlignAttributeInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapDivAlignAttributeInto(const nsMappedAttributes* aAttributes,
+                                       mozilla::MappedDeclarations&);
 
   /**
    * Helper to map the valign attribute into a style struct for things
@@ -471,9 +486,8 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapVAlignAttributeInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapVAlignAttributeInto(const nsMappedAttributes* aAttributes,
+                                     mozilla::MappedDeclarations&);
 
   /**
    * Helper to map the image border attribute into a style struct.
@@ -482,9 +496,8 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapImageBorderAttributeInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapImageBorderAttributeInto(const nsMappedAttributes* aAttributes,
+                                          mozilla::MappedDeclarations&);
   /**
    * Helper to map the image margin attribute into a style struct.
    *
@@ -492,19 +505,18 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapImageMarginAttributeInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapImageMarginAttributeInto(const nsMappedAttributes* aAttributes,
+                                          mozilla::MappedDeclarations&);
+
+  // Whether to map the width and height attributes to aspect-ratio.
+  enum class MapAspectRatio { No, Yes };
+
   /**
    * Helper to map the image position attribute into a style struct.
-   *
-   * @param aAttributes the list of attributes to map
-   * @param aData the returned rule data [INOUT]
-   * @see GetAttributeMappingFunction
    */
-  static void MapImageSizeAttributesInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapImageSizeAttributesInto(const nsMappedAttributes*,
+                                         mozilla::MappedDeclarations&,
+                                         MapAspectRatio = MapAspectRatio::No);
 
   /**
    * Helper to map `width` attribute into a style struct.
@@ -513,9 +525,8 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapWidthAttributeInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapWidthAttributeInto(const nsMappedAttributes* aAttributes,
+                                    mozilla::MappedDeclarations&);
   /**
    * Helper to map `height` attribute into a style struct.
    *
@@ -523,9 +534,8 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapHeightAttributeInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapHeightAttributeInto(const nsMappedAttributes* aAttributes,
+                                     mozilla::MappedDeclarations&);
   /**
    * Helper to map the background attribute
    * into a style struct.
@@ -535,7 +545,7 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @see GetAttributeMappingFunction
    */
   static void MapBackgroundInto(const nsMappedAttributes* aAttributes,
-                                mozilla::GenericSpecifiedValues* aGenericData);
+                                mozilla::MappedDeclarations&);
   /**
    * Helper to map the bgcolor attribute
    * into a style struct.
@@ -545,7 +555,7 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @see GetAttributeMappingFunction
    */
   static void MapBGColorInto(const nsMappedAttributes* aAttributes,
-                             mozilla::GenericSpecifiedValues* aGenericData);
+                             mozilla::MappedDeclarations&);
   /**
    * Helper to map the background attributes (currently background and bgcolor)
    * into a style struct.
@@ -554,9 +564,8 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapBackgroundAttributesInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapBackgroundAttributesInto(const nsMappedAttributes* aAttributes,
+                                          mozilla::MappedDeclarations&);
   /**
    * Helper to map the scrolling attribute on FRAME and IFRAME
    * into a style struct.
@@ -565,9 +574,8 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapScrollingAttributeInto(
-      const nsMappedAttributes* aAttributes,
-      mozilla::GenericSpecifiedValues* aGenericData);
+  static void MapScrollingAttributeInto(const nsMappedAttributes* aAttributes,
+                                        mozilla::MappedDeclarations&);
   /**
    * Get the presentation context for this content node.
    * @return the presentation context
@@ -594,7 +602,7 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    * See if the document being tested has nav-quirks mode enabled.
    * @param doc the document
    */
-  static bool InNavQuirksMode(nsIDocument* aDoc);
+  static bool InNavQuirksMode(Document*);
 
   /**
    * Gets the absolute URI value of an attribute, by resolving any relative
@@ -634,10 +642,7 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
 
   already_AddRefed<nsINodeList> Labels();
 
-  virtual bool IsInteractiveHTMLContent(bool aIgnoreTabindex) const override;
-
-  static bool TouchEventsEnabled(JSContext* /* unused */,
-                                 JSObject* /* unused */);
+  static bool LegacyTouchAPIEnabled(JSContext* aCx, JSObject* aObj);
 
   static inline bool CanHaveName(nsAtom* aTag) {
     return aTag == nsGkAtoms::img || aTag == nsGkAtoms::form ||
@@ -658,26 +663,16 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
     return aElement->IsHTMLElement(nsGkAtoms::img) && aElement->HasName();
   }
 
+  virtual inline void ResultForDialogSubmit(nsAString& aResult) {
+    GetAttr(kNameSpaceID_None, nsGkAtoms::value, aResult);
+  }
+
  protected:
   /**
    * Add/remove this element to the documents name cache
    */
-  void AddToNameTable(nsAtom* aName) {
-    NS_ASSERTION(HasName(), "Node doesn't have name?");
-    nsIDocument* doc = GetUncomposedDoc();
-    if (doc && !IsInAnonymousSubtree()) {
-      doc->AddToNameTable(this, aName);
-    }
-  }
-  void RemoveFromNameTable() {
-    if (HasName() && CanHaveName(NodeInfo()->NameAtom())) {
-      nsIDocument* doc = GetUncomposedDoc();
-      if (doc) {
-        doc->RemoveFromNameTable(
-            this, GetParsedAttr(nsGkAtoms::name)->GetAtomValue());
-      }
-    }
-  }
+  void AddToNameTable(nsAtom* aName);
+  void RemoveFromNameTable();
 
   /**
    * Register or unregister an access key to this element based on the
@@ -725,7 +720,7 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
   nsresult NewURIFromString(const nsAString& aURISpec, nsIURI** aURI);
 
   void GetHTMLAttr(nsAtom* aName, nsAString& aResult) const {
-    GetAttr(kNameSpaceID_None, aName, aResult);
+    GetAttr(aName, aResult);
   }
   void GetHTMLAttr(nsAtom* aName, mozilla::dom::DOMString& aResult) const {
     GetAttr(kNameSpaceID_None, aName, aResult);
@@ -742,15 +737,15 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
   }
   void SetHTMLAttr(nsAtom* aName, const nsAString& aValue,
                    mozilla::ErrorResult& aError) {
-    mozilla::dom::Element::SetAttr(aName, aValue, aError);
+    SetAttr(aName, aValue, aError);
   }
   void SetHTMLAttr(nsAtom* aName, const nsAString& aValue,
                    nsIPrincipal* aTriggeringPrincipal,
                    mozilla::ErrorResult& aError) {
-    mozilla::dom::Element::SetAttr(aName, aValue, aTriggeringPrincipal, aError);
+    SetAttr(aName, aValue, aTriggeringPrincipal, aError);
   }
   void UnsetHTMLAttr(nsAtom* aName, mozilla::ErrorResult& aError) {
-    mozilla::dom::Element::UnsetAttr(aName, aError);
+    UnsetAttr(aName, aError);
   }
   void SetHTMLBoolAttr(nsAtom* aName, bool aValue,
                        mozilla::ErrorResult& aError) {
@@ -819,6 +814,18 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
     SetHTMLAttr(aName, value, aError);
   }
 
+  /**
+   * Gets the unsigned integer-value of an attribute that is stored as a
+   * dimension (i.e. could be an integer or a percentage), returns specified
+   * default value if the attribute isn't set or isn't set to a dimension. Only
+   * works for attributes in null namespace.
+   *
+   * @param aAttr    name of attribute.
+   * @param aDefault default-value to return if attribute isn't set.
+   */
+  uint32_t GetDimensionAttrAsUnsignedInt(nsAtom* aAttr,
+                                         uint32_t aDefault) const;
+
   /**
    * Sets value of attribute to specified double. Only works for attributes
    * in null namespace.
@@ -867,7 +874,7 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
    */
   ContentEditableTristate GetContentEditableValue() const {
     static const Element::AttrValuesArray values[] = {
-        &nsGkAtoms::_false, &nsGkAtoms::_true, &nsGkAtoms::_empty, nullptr};
+        nsGkAtoms::_false, nsGkAtoms::_true, nsGkAtoms::_empty, nullptr};
 
     if (!MayHaveContentEditableAttr()) return eInherit;
 
@@ -880,6 +887,7 @@ class nsGenericHTMLElement : public nsGenericHTMLElementBase,
   // Used by A, AREA, LINK, and STYLE.
   already_AddRefed<nsIURI> GetHrefURIForAnchors() const;
 
+ public:
   /**
    * Returns whether this element is an editable root. There are two types of
    * editable roots:
@@ -938,7 +946,7 @@ ASSERT_NODE_FLAGS_SPACE(ELEMENT_TYPE_SPECIFIC_BITS_OFFSET + 3);
 class nsGenericHTMLFormElement : public nsGenericHTMLElement,
                                  public nsIFormControl {
  public:
-  nsGenericHTMLFormElement(already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo,
+  nsGenericHTMLFormElement(already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo,
                            uint8_t aType);
 
   NS_DECL_ISUPPORTS_INHERITED
@@ -950,29 +958,25 @@ class nsGenericHTMLFormElement : public nsGenericHTMLElement,
 
   // nsIFormControl
   virtual mozilla::dom::HTMLFieldSetElement* GetFieldSet() override;
-  virtual mozilla::dom::Element* GetFormElement() override;
+  virtual mozilla::dom::HTMLFormElement* GetFormElement() override;
   mozilla::dom::HTMLFormElement* GetForm() const { return mForm; }
-  virtual void SetForm(nsIDOMHTMLFormElement* aForm) override;
+  virtual void SetForm(mozilla::dom::HTMLFormElement* aForm) override;
   virtual void ClearForm(bool aRemoveFromForm, bool aUnbindOrDelete) override;
 
-  nsresult GetForm(nsIDOMHTMLFormElement** aForm);
-
   NS_IMETHOD SaveState() override { return NS_OK; }
 
-  virtual bool RestoreState(nsPresState* aState) override { return false; }
+  virtual bool RestoreState(mozilla::PresState* aState) override {
+    return false;
+  }
   virtual bool AllowDrop() override { return true; }
 
   // nsIContent
-  virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
-                              nsIContent* aBindingParent,
-                              bool aCompileEventHandlers) override;
-  virtual void UnbindFromTree(bool aDeep = true,
-                              bool aNullParent = true) override;
+  virtual nsresult BindToTree(BindContext&, nsINode& aParent) override;
+  virtual void UnbindFromTree(bool aNullParent = true) override;
   virtual IMEState GetDesiredIMEState() override;
   virtual mozilla::EventStates IntrinsicState() const override;
 
-  virtual nsresult GetEventTargetParent(
-      mozilla::EventChainPreVisitor& aVisitor) override;
+  void GetEventTargetParent(mozilla::EventChainPreVisitor& aVisitor) override;
   virtual nsresult PreHandleEvent(
       mozilla::EventChainVisitor& aVisitor) override;
 
@@ -1077,7 +1081,7 @@ class nsGenericHTMLFormElement : public nsGenericHTMLElement,
   void RemoveFormIdObserver();
 
   /**
-   * This method is a a callback for IDTargetObserver (from nsIDocument).
+   * This method is a a callback for IDTargetObserver (from Document).
    * It will be called each time the element associated with the id in @form
    * changes.
    */
@@ -1085,7 +1089,7 @@ class nsGenericHTMLFormElement : public nsGenericHTMLElement,
                             void* aData);
 
   // Returns true if the event should not be handled from GetEventTargetParent
-  bool IsElementDisabledForEvents(mozilla::EventMessage aMessage,
+  bool IsElementDisabledForEvents(mozilla::WidgetEvent* aEvent,
                                   nsIFrame* aFrame);
 
   // The focusability state of this form control.  eUnfocusable means that it
@@ -1107,13 +1111,14 @@ class nsGenericHTMLFormElement : public nsGenericHTMLElement,
 class nsGenericHTMLFormElementWithState : public nsGenericHTMLFormElement {
  public:
   nsGenericHTMLFormElementWithState(
-      already_AddRefed<mozilla::dom::NodeInfo>& aNodeInfo, uint8_t aType);
+      already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo,
+      mozilla::dom::FromParser aFromParser, uint8_t aType);
 
   /**
    * Get the presentation state for a piece of content, or create it if it does
    * not exist.  Generally used by SaveState().
    */
-  nsPresState* GetPrimaryPresState();
+  mozilla::PresState* GetPrimaryPresState();
 
   /**
    * Get the layout history object for a particular piece of content.
@@ -1125,28 +1130,40 @@ class nsGenericHTMLFormElementWithState : public nsGenericHTMLFormElement {
   already_AddRefed<nsILayoutHistoryState> GetLayoutHistory(bool aRead);
 
   /**
-   * Restore the state for a form control.  Ends up calling
+   * Called when we have been cloned and adopted, and the information of the
+   * node has been changed.
+   */
+  virtual void NodeInfoChanged(Document* aOldDoc) override;
+
+ protected:
+  /**
+   * Restore the state for a form control in response to the element being
+   * inserted into the document by the parser.  Ends up calling
    * nsIFormControl::RestoreState().
    *
+   * GenerateStateKey() must already have been called.
+   *
    * @return false if RestoreState() was not called, the return
    *         value of RestoreState() otherwise.
    */
   bool RestoreFormControlState();
 
-  /**
-   * Called when we have been cloned and adopted, and the information of the
-   * node has been changed.
-   */
-  virtual void NodeInfoChanged(nsIDocument* aOldDoc) override;
-
- protected:
   /* Generates the state key for saving the form state in the session if not
-     computed already. The result is stored in mStateKey on success */
-  nsresult GenerateStateKey();
+     computed already. The result is stored in mStateKey. */
+  void GenerateStateKey();
+
+  int32_t GetParserInsertedControlNumberForStateKey() const override {
+    return mControlNumber;
+  }
 
   /* Used to store the key to that element in the session. Is void until
      GenerateStateKey has been used */
   nsCString mStateKey;
+
+  // A number for this form control that is unique within its owner document.
+  // This is only set to a number for elements inserted into the document by
+  // the parser from the network.  Otherwise, it is -1.
+  int32_t mControlNumber;
 };
 
 #define NS_INTERFACE_MAP_ENTRY_IF_TAG(_interface, _tag) \
@@ -1180,25 +1197,32 @@ typedef nsGenericHTMLElement* (*HTMLContentCreatorFunction)(
   inline nsGenericHTMLElement* NS_NewHTML##_elementName##Element(             \
       already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo,                   \
       mozilla::dom::FromParser aFromParser = mozilla::dom::NOT_FROM_PARSER) { \
-    return NS_NewHTMLSharedElement(mozilla::Move(aNodeInfo), aFromParser);    \
+    return NS_NewHTMLSharedElement(std::move(aNodeInfo), aFromParser);        \
   }
 
 /**
  * A macro to implement the NS_NewHTMLXXXElement() functions.
  */
-#define NS_IMPL_NS_NEW_HTML_ELEMENT(_elementName)                    \
-  nsGenericHTMLElement* NS_NewHTML##_elementName##Element(           \
-      already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo,          \
-      mozilla::dom::FromParser aFromParser) {                        \
-    return new mozilla::dom::HTML##_elementName##Element(aNodeInfo); \
+#define NS_IMPL_NS_NEW_HTML_ELEMENT(_elementName)                     \
+  nsGenericHTMLElement* NS_NewHTML##_elementName##Element(            \
+      already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo,           \
+      mozilla::dom::FromParser aFromParser) {                         \
+    RefPtr<mozilla::dom::NodeInfo> nodeInfo(aNodeInfo);               \
+    auto* nim = nodeInfo->NodeInfoManager();                          \
+    MOZ_ASSERT(nim);                                                  \
+    return new (nim)                                                  \
+        mozilla::dom::HTML##_elementName##Element(nodeInfo.forget()); \
   }
 
-#define NS_IMPL_NS_NEW_HTML_ELEMENT_CHECK_PARSER(_elementName)         \
-  nsGenericHTMLElement* NS_NewHTML##_elementName##Element(             \
-      already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo,            \
-      mozilla::dom::FromParser aFromParser) {                          \
-    return new mozilla::dom::HTML##_elementName##Element(aNodeInfo,    \
-                                                         aFromParser); \
+#define NS_IMPL_NS_NEW_HTML_ELEMENT_CHECK_PARSER(_elementName)  \
+  nsGenericHTMLElement* NS_NewHTML##_elementName##Element(      \
+      already_AddRefed<mozilla::dom::NodeInfo>&& aNodeInfo,     \
+      mozilla::dom::FromParser aFromParser) {                   \
+    RefPtr<mozilla::dom::NodeInfo> nodeInfo(aNodeInfo);         \
+    auto* nim = nodeInfo->NodeInfoManager();                    \
+    MOZ_ASSERT(nim);                                            \
+    return new (nim) mozilla::dom::HTML##_elementName##Element( \
+        nodeInfo.forget(), aFromParser);                        \
   }
 
 // Here, we expand 'NS_DECLARE_NS_NEW_HTML_ELEMENT()' by hand.
@@ -1247,6 +1271,7 @@ NS_DECLARE_NS_NEW_HTML_ELEMENT(LI)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Label)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Legend)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Link)
+NS_DECLARE_NS_NEW_HTML_ELEMENT(Marquee)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Map)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Menu)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(MenuItem)
@@ -1284,12 +1309,4 @@ NS_DECLARE_NS_NEW_HTML_ELEMENT(Track)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Unknown)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Video)
 
-inline nsISupports* ToSupports(nsGenericHTMLElement* aHTMLElement) {
-  return static_cast<nsIContent*>(aHTMLElement);
-}
-
-inline nsISupports* ToCanonicalSupports(nsGenericHTMLElement* aHTMLElement) {
-  return static_cast<nsIContent*>(aHTMLElement);
-}
-
 #endif /* nsGenericHTMLElement_h___ */
