@@ -6,17 +6,20 @@
 #include "MediaStreamTrack.h"
 
 #include "DOMMediaStream.h"
+#include "MediaSegment.h"
 #include "MediaStreamError.h"
-#include "MediaStreamGraph.h"
-#include "MediaStreamListener.h"
+#include "MediaTrackGraphImpl.h"
+#include "MediaTrackListener.h"
+#include "mozilla/BasePrincipal.h"
 #include "mozilla/dom/Promise.h"
 #include "nsContentUtils.h"
+#include "nsGlobalWindowInner.h"
 #include "nsIUUIDGenerator.h"
 #include "nsServiceManagerUtils.h"
 #include "systemservices/MediaUtils.h"
 
 #ifdef LOG
-#undef LOG
+#  undef LOG
 #endif
 
 static mozilla::LazyLogModule gMediaStreamTrackLog("MediaStreamTrack");
@@ -45,25 +48,27 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(MediaStreamTrackSource)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 auto MediaStreamTrackSource::ApplyConstraints(
-    nsPIDOMWindowInner* aWindow, const dom::MediaTrackConstraints& aConstraints,
-    CallerType aCallerType) -> already_AddRefed<PledgeVoid> {
-  RefPtr<PledgeVoid> p = new PledgeVoid();
-  p->Reject(new MediaStreamError(aWindow,
-                                 NS_LITERAL_STRING("OverconstrainedError"),
-                                 NS_LITERAL_STRING("")));
-  return p.forget();
+    const dom::MediaTrackConstraints& aConstraints, CallerType aCallerType)
+    -> RefPtr<ApplyConstraintsPromise> {
+  return ApplyConstraintsPromise::CreateAndReject(
+      MakeRefPtr<MediaMgrError>(MediaMgrError::Name::OverconstrainedError,
+                                NS_LITERAL_STRING("")),
+      __func__);
 }
 
 /**
- * PrincipalHandleListener monitors changes in PrincipalHandle of the media
- * flowing through the MediaStreamGraph.
+ * MTGListener monitors state changes of the media flowing through the
+ * MediaTrackGraph.
+ *
+ *
+ * For changes to PrincipalHandle the following applies:
  *
  * When the main thread principal for a MediaStreamTrack changes, its principal
  * will be set to the combination of the previous principal and the new one.
  *
- * As a PrincipalHandle change later happens on the MediaStreamGraph thread, we
+ * As a PrincipalHandle change later happens on the MediaTrackGraph thread, we
  * will be notified. If the latest principal on main thread matches the
- * PrincipalHandle we just saw on MSG thread, we will set the track's principal
+ * PrincipalHandle we just saw on MTG thread, we will set the track's principal
  * to the new one.
  *
  * We know at this point that the old principal has been flushed out and data
@@ -71,17 +76,11 @@ auto MediaStreamTrackSource::ApplyConstraints(
  *
  * In case of multiple changes to the main thread state, the track's principal
  * will be a combination of its old principal and all the new ones until the
- * latest main thread principal matches the PrincipalHandle on the MSG thread.
+ * latest main thread principal matches the PrincipalHandle on the MTG thread.
  */
-class MediaStreamTrack::PrincipalHandleListener
-    : public MediaStreamTrackListener {
+class MediaStreamTrack::MTGListener : public MediaTrackListener {
  public:
-  explicit PrincipalHandleListener(MediaStreamTrack* aTrack) : mTrack(aTrack) {}
-
-  void Forget() {
-    MOZ_ASSERT(NS_IsMainThread());
-    mTrack = nullptr;
-  }
+  explicit MTGListener(MediaStreamTrack* aTrack) : mTrack(aTrack) {}
 
   void DoNotifyPrincipalHandleChanged(
       const PrincipalHandle& aNewPrincipalHandle) {
@@ -95,39 +94,130 @@ class MediaStreamTrack::PrincipalHandleListener
   }
 
   void NotifyPrincipalHandleChanged(
-      MediaStreamGraph* aGraph,
+      MediaTrackGraph* aGraph,
       const PrincipalHandle& aNewPrincipalHandle) override {
-    aGraph->DispatchToMainThreadAfterStreamStateUpdate(
+    aGraph->DispatchToMainThreadStableState(
         NewRunnableMethod<StoreCopyPassByConstLRef<PrincipalHandle>>(
-            "dom::MediaStreamTrack::PrincipalHandleListener::"
+            "dom::MediaStreamTrack::MTGListener::"
             "DoNotifyPrincipalHandleChanged",
-            this, &PrincipalHandleListener::DoNotifyPrincipalHandleChanged,
+            this, &MTGListener::DoNotifyPrincipalHandleChanged,
             aNewPrincipalHandle));
   }
 
+  void NotifyRemoved(MediaTrackGraph* aGraph) override {
+    // `mTrack` is a WeakPtr and must be destroyed on main thread.
+    // We dispatch ourselves to main thread here in case the MediaTrackGraph
+    // is holding the last reference to us.
+    aGraph->DispatchToMainThreadStableState(
+        NS_NewRunnableFunction("MediaStreamTrack::MTGListener::mTrackReleaser",
+                               [self = RefPtr<MTGListener>(this)]() {}));
+  }
+
+  void DoNotifyEnded() {
+    MOZ_ASSERT(NS_IsMainThread());
+
+    if (!mTrack) {
+      return;
+    }
+
+    if (!mTrack->GetParentObject()) {
+      return;
+    }
+
+    AbstractThread* mainThread =
+        nsGlobalWindowInner::Cast(mTrack->GetParentObject())
+            ->AbstractMainThreadFor(TaskCategory::Other);
+    mainThread->Dispatch(NewRunnableMethod("MediaStreamTrack::OverrideEnded",
+                                           mTrack.get(),
+                                           &MediaStreamTrack::OverrideEnded));
+  }
+
+  void NotifyEnded(MediaTrackGraph* aGraph) override {
+    aGraph->DispatchToMainThreadStableState(
+        NewRunnableMethod("MediaStreamTrack::MTGListener::DoNotifyEnded", this,
+                          &MTGListener::DoNotifyEnded));
+  }
+
  protected:
-  // These fields may only be accessed on the main thread
-  MediaStreamTrack* mTrack;
+  // Main thread only.
+  WeakPtr<MediaStreamTrack> mTrack;
 };
 
-MediaStreamTrack::MediaStreamTrack(DOMMediaStream* aStream, TrackID aTrackID,
-                                   TrackID aInputTrackID,
-                                   MediaStreamTrackSource* aSource,
-                                   const MediaTrackConstraints& aConstraints)
-    : mOwningStream(aStream),
-      mTrackID(aTrackID),
-      mInputTrackID(aInputTrackID),
-      mSource(aSource),
-      mPrincipal(aSource->GetPrincipal()),
-      mReadyState(MediaStreamTrackState::Live),
-      mEnabled(true),
-      mMuted(false),
-      mConstraints(aConstraints) {
-  GetSource().RegisterSink(this);
+class MediaStreamTrack::TrackSink : public MediaStreamTrackSource::Sink {
+ public:
+  explicit TrackSink(MediaStreamTrack* aTrack) : mTrack(aTrack) {}
 
-  if (GetOwnedStream()) {
-    mPrincipalHandleListener = new PrincipalHandleListener(this);
-    AddListener(mPrincipalHandleListener);
+  /**
+   * Keep the track source alive. This track and any clones are controlling the
+   * lifetime of the source by being registered as its sinks.
+   */
+  bool KeepsSourceAlive() const override { return true; }
+
+  bool Enabled() const override {
+    if (!mTrack) {
+      return false;
+    }
+    return mTrack->Enabled();
+  }
+
+  void PrincipalChanged() override {
+    if (mTrack) {
+      mTrack->PrincipalChanged();
+    }
+  }
+
+  void MutedChanged(bool aNewState) override {
+    if (mTrack) {
+      mTrack->MutedChanged(aNewState);
+    }
+  }
+
+  void OverrideEnded() override {
+    if (mTrack) {
+      mTrack->OverrideEnded();
+    }
+  }
+
+ private:
+  WeakPtr<MediaStreamTrack> mTrack;
+};
+
+MediaStreamTrack::MediaStreamTrack(nsPIDOMWindowInner* aWindow,
+                                   mozilla::MediaTrack* aInputTrack,
+                                   MediaStreamTrackSource* aSource,
+                                   MediaStreamTrackState aReadyState,
+                                   bool aMuted,
+                                   const MediaTrackConstraints& aConstraints)
+    : mWindow(aWindow),
+      mInputTrack(aInputTrack),
+      mSource(aSource),
+      mSink(MakeUnique<TrackSink>(this)),
+      mPrincipal(aSource->GetPrincipal()),
+      mReadyState(aReadyState),
+      mEnabled(true),
+      mMuted(aMuted),
+      mConstraints(aConstraints) {
+  if (!Ended()) {
+    GetSource().RegisterSink(mSink.get());
+
+    // Even if the input track is destroyed we need mTrack so that methods
+    // like AddListener still work. Keeping the number of paths to a minimum
+    // also helps prevent bugs elsewhere. We'll be ended through the
+    // MediaStreamTrackSource soon enough.
+    auto graph = mInputTrack->IsDestroyed()
+                     ? MediaTrackGraph::GetInstanceIfExists(
+                           mWindow, mInputTrack->mSampleRate,
+                           MediaTrackGraph::DEFAULT_OUTPUT_DEVICE)
+                     : mInputTrack->Graph();
+    MOZ_DIAGNOSTIC_ASSERT(graph,
+                          "A destroyed input track is only expected when "
+                          "cloning, but since we're live there must be another "
+                          "live track that is keeping the graph alive");
+
+    mTrack = graph->CreateForwardedInputTrack(mInputTrack->mType);
+    mPort = mTrack->AllocateInputPort(mInputTrack);
+    mMTGListener = new MTGListener(this);
+    AddListener(mMTGListener);
   }
 
   nsresult rv;
@@ -148,26 +238,13 @@ MediaStreamTrack::MediaStreamTrack(DOMMediaStream* aStream, TrackID aTrackID,
 MediaStreamTrack::~MediaStreamTrack() { Destroy(); }
 
 void MediaStreamTrack::Destroy() {
-  if (mSource) {
-    mSource->UnregisterSink(this);
-  }
-  if (mPrincipalHandleListener) {
-    if (GetOwnedStream()) {
-      RemoveListener(mPrincipalHandleListener);
-    }
-    mPrincipalHandleListener->Forget();
-    mPrincipalHandleListener = nullptr;
-  }
+  SetReadyState(MediaStreamTrackState::Ended);
   // Remove all listeners -- avoid iterating over the list we're removing from
-  const nsTArray<RefPtr<MediaStreamTrackListener>> trackListeners(
-      mTrackListeners);
-  for (auto listener : trackListeners) {
+  for (const auto& listener : mTrackListeners.Clone()) {
     RemoveListener(listener);
   }
   // Do the same as above for direct listeners
-  const nsTArray<RefPtr<DirectMediaStreamTrackListener>> directTrackListeners(
-      mDirectTrackListeners);
-  for (auto listener : directTrackListeners) {
+  for (const auto& listener : mDirectTrackListeners.Clone()) {
     RemoveDirectListener(listener);
   }
 }
@@ -177,18 +254,17 @@ NS_IMPL_CYCLE_COLLECTION_CLASS(MediaStreamTrack)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(MediaStreamTrack,
                                                 DOMEventTargetHelper)
   tmp->Destroy();
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mOwningStream)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mWindow)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mSource)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mOriginalTrack)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mPrincipal)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mPendingPrincipal)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_WEAK_PTR
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(MediaStreamTrack,
                                                   DOMEventTargetHelper)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mOwningStream)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mWindow)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mSource)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mOriginalTrack)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mPrincipal)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mPendingPrincipal)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
@@ -198,9 +274,9 @@ NS_IMPL_RELEASE_INHERITED(MediaStreamTrack, DOMEventTargetHelper)
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(MediaStreamTrack)
 NS_INTERFACE_MAP_END_INHERITING(DOMEventTargetHelper)
 
-nsPIDOMWindowInner* MediaStreamTrack::GetParentObject() const {
-  MOZ_RELEASE_ASSERT(mOwningStream);
-  return mOwningStream->GetParentObject();
+JSObject* MediaStreamTrack::WrapObject(JSContext* aCx,
+                                       JS::Handle<JSObject*> aGivenProto) {
+  return MediaStreamTrack_Binding::Wrap(aCx, this, aGivenProto);
 }
 
 void MediaStreamTrack::GetId(nsAString& aID) const { aID = mID; }
@@ -214,10 +290,14 @@ void MediaStreamTrack::SetEnabled(bool aEnabled) {
   }
 
   mEnabled = aEnabled;
-  GetOwnedStream()->SetTrackEnabled(
-      mTrackID,
-      mEnabled ? DisabledTrackMode::ENABLED : DisabledTrackMode::SILENCE_BLACK);
-  GetSource().SinkEnabledStateChanged();
+
+  if (Ended()) {
+    return;
+  }
+
+  mTrack->SetEnabled(mEnabled ? DisabledTrackMode::ENABLED
+                              : DisabledTrackMode::SILENCE_BLACK);
+  NotifyEnabledChanged();
 }
 
 void MediaStreamTrack::Stop() {
@@ -228,23 +308,7 @@ void MediaStreamTrack::Stop() {
     return;
   }
 
-  if (!mSource) {
-    MOZ_ASSERT(false);
-    return;
-  }
-
-  mSource->UnregisterSink(this);
-
-  MOZ_ASSERT(mOwningStream,
-             "Every MediaStreamTrack needs an owning DOMMediaStream");
-  DOMMediaStream::TrackPort* port = mOwningStream->FindOwnedTrackPort(*this);
-  MOZ_ASSERT(port,
-             "A MediaStreamTrack must exist in its owning DOMMediaStream");
-  RefPtr<Pledge<bool>> p =
-      port->BlockSourceTrackId(mInputTrackID, BlockingMode::CREATION);
-  Unused << p;
-
-  mReadyState = MediaStreamTrackState::Ended;
+  SetReadyState(MediaStreamTrackState::Ended);
 
   NotifyEnded();
 }
@@ -262,9 +326,8 @@ void MediaStreamTrack::GetSettings(dom::MediaTrackSettings& aResult,
     return;
   }
   if (aResult.mFacingMode.WasPassed()) {
-    aResult.mFacingMode.Value().Assign(NS_ConvertASCIItoUTF16(
-        VideoFacingModeEnumValues::strings[uint8_t(VideoFacingModeEnum::User)]
-            .value));
+    aResult.mFacingMode.Value().AssignASCII(
+        VideoFacingModeEnumValues::GetString(VideoFacingModeEnum::User));
   }
 }
 
@@ -280,10 +343,7 @@ already_AddRefed<Promise> MediaStreamTrack::ApplyConstraints(
                          this, NS_ConvertUTF16toUTF8(str).get()));
   }
 
-  typedef media::Pledge<bool, MediaStreamError*> PledgeVoid;
-
-  nsPIDOMWindowInner* window = mOwningStream->GetParentObject();
-  nsIGlobalObject* go = window ? window->AsGlobal() : nullptr;
+  nsIGlobalObject* go = mWindow ? mWindow->AsGlobal() : nullptr;
 
   RefPtr<Promise> promise = Promise::Create(go, aRv);
   if (aRv.Failed()) {
@@ -297,26 +357,41 @@ already_AddRefed<Promise> MediaStreamTrack::ApplyConstraints(
   // applied values.
 
   // Keep a reference to this, to make sure it's still here when we get back.
-  RefPtr<MediaStreamTrack> that = this;
-  RefPtr<PledgeVoid> p =
-      GetSource().ApplyConstraints(window, aConstraints, aCallerType);
-  p->Then(
-      [this, that, promise, aConstraints](bool& aDummy) mutable {
-        mConstraints = aConstraints;
-        promise->MaybeResolve(false);
-      },
-      [promise](MediaStreamError*& reason) mutable {
-        promise->MaybeReject(reason);
-      });
+  RefPtr<MediaStreamTrack> self(this);
+  GetSource()
+      .ApplyConstraints(aConstraints, aCallerType)
+      ->Then(
+          GetCurrentThreadSerialEventTarget(), __func__,
+          [this, self, promise, aConstraints](bool aDummy) {
+            if (!mWindow || !mWindow->IsCurrentInnerWindow()) {
+              return;  // Leave Promise pending after navigation by design.
+            }
+            mConstraints = aConstraints;
+            promise->MaybeResolve(false);
+          },
+          [this, self, promise](const RefPtr<MediaMgrError>& aError) {
+            if (!mWindow || !mWindow->IsCurrentInnerWindow()) {
+              return;  // Leave Promise pending after navigation by design.
+            }
+            promise->MaybeReject(
+                MakeRefPtr<MediaStreamError>(mWindow, *aError));
+          });
   return promise.forget();
 }
 
-MediaStreamGraph* MediaStreamTrack::Graph() {
-  return GetOwnedStream()->Graph();
+ProcessedMediaTrack* MediaStreamTrack::GetTrack() const {
+  MOZ_DIAGNOSTIC_ASSERT(!Ended());
+  return mTrack;
 }
 
-MediaStreamGraphImpl* MediaStreamTrack::GraphImpl() {
-  return GetOwnedStream()->GraphImpl();
+MediaTrackGraph* MediaStreamTrack::Graph() const {
+  MOZ_DIAGNOSTIC_ASSERT(!Ended());
+  return mTrack->Graph();
+}
+
+MediaTrackGraphImpl* MediaStreamTrack::GraphImpl() const {
+  MOZ_DIAGNOSTIC_ASSERT(!Ended());
+  return mTrack->GraphImpl();
 }
 
 void MediaStreamTrack::SetPrincipal(nsIPrincipal* aPrincipal) {
@@ -325,12 +400,12 @@ void MediaStreamTrack::SetPrincipal(nsIPrincipal* aPrincipal) {
   }
   mPrincipal = aPrincipal;
 
-  LOG(LogLevel::Info, ("MediaStreamTrack %p principal changed to %p. Now: "
-                       "null=%d, codebase=%d, expanded=%d, system=%d",
-                       this, mPrincipal.get(), mPrincipal->GetIsNullPrincipal(),
-                       mPrincipal->GetIsCodebasePrincipal(),
-                       mPrincipal->GetIsExpandedPrincipal(),
-                       mPrincipal->GetIsSystemPrincipal()));
+  LOG(LogLevel::Info,
+      ("MediaStreamTrack %p principal changed to %p. Now: "
+       "null=%d, codebase=%d, expanded=%d, system=%d",
+       this, mPrincipal.get(), mPrincipal->GetIsNullPrincipal(),
+       mPrincipal->GetIsContentPrincipal(),
+       mPrincipal->GetIsExpandedPrincipal(), mPrincipal->IsSystemPrincipal()));
   for (PrincipalChangeObserver<MediaStreamTrack>* observer :
        mPrincipalChangeObservers) {
     observer->PrincipalChanged(this);
@@ -353,7 +428,7 @@ void MediaStreamTrack::NotifyPrincipalHandleChanged(
     const PrincipalHandle& aNewPrincipalHandle) {
   PrincipalHandle handle(aNewPrincipalHandle);
   LOG(LogLevel::Info, ("MediaStreamTrack %p principalHandle changed on "
-                       "MediaStreamGraph thread to %p. Current principal: %p, "
+                       "MediaTrackGraph thread to %p. Current principal: %p, "
                        "pending: %p",
                        this, GetPrincipalFromHandle(handle), mPrincipal.get(),
                        mPendingPrincipal.get()));
@@ -366,8 +441,17 @@ void MediaStreamTrack::NotifyPrincipalHandleChanged(
 void MediaStreamTrack::MutedChanged(bool aNewState) {
   MOZ_ASSERT(NS_IsMainThread());
 
+  /**
+   * 4.3.1 Life-cycle and Media flow - Media flow
+   * To set a track's muted state to newState, the User Agent MUST run the
+   * following steps:
+   *  1. Let track be the MediaStreamTrack in question.
+   *  2. Set track's muted attribute to newState.
+   *  3. If newState is true let eventName be mute, otherwise unmute.
+   *  4. Fire a simple event named eventName on track.
+   */
+
   if (mMuted == aNewState) {
-    MOZ_ASSERT_UNREACHABLE("Muted state didn't actually change");
     return;
   }
 
@@ -383,8 +467,7 @@ void MediaStreamTrack::MutedChanged(bool aNewState) {
 void MediaStreamTrack::NotifyEnded() {
   MOZ_ASSERT(mReadyState == MediaStreamTrackState::Ended);
 
-  auto consumers(mConsumers);
-  for (const auto& consumer : consumers) {
+  for (const auto& consumer : mConsumers.Clone()) {
     if (consumer) {
       consumer->NotifyEnded(this);
     } else {
@@ -394,9 +477,25 @@ void MediaStreamTrack::NotifyEnded() {
   }
 }
 
+void MediaStreamTrack::NotifyEnabledChanged() {
+  GetSource().SinkEnabledStateChanged();
+
+  for (const auto& consumer : mConsumers.Clone()) {
+    if (consumer) {
+      consumer->NotifyEnabledChanged(this, Enabled());
+    } else {
+      MOZ_ASSERT_UNREACHABLE("A consumer was not explicitly removed");
+      mConsumers.RemoveElement(consumer);
+    }
+  }
+}
+
 bool MediaStreamTrack::AddPrincipalChangeObserver(
     PrincipalChangeObserver<MediaStreamTrack>* aObserver) {
-  return mPrincipalChangeObservers.AppendElement(aObserver) != nullptr;
+  // XXX(Bug 1631371) Check if this should use a fallible operation as it
+  // pretended earlier.
+  mPrincipalChangeObservers.AppendElement(aObserver);
+  return true;
 }
 
 bool MediaStreamTrack::RemovePrincipalChangeObserver(
@@ -424,17 +523,10 @@ void MediaStreamTrack::RemoveConsumer(MediaStreamTrackConsumer* aConsumer) {
 }
 
 already_AddRefed<MediaStreamTrack> MediaStreamTrack::Clone() {
-  // MediaStreamTracks are currently governed by streams, so we need a dummy
-  // DOMMediaStream to own our track clone. The dummy will never see any
-  // dynamically created tracks (no input stream) so no need for a SourceGetter.
-  RefPtr<DOMMediaStream> newStream =
-      new DOMMediaStream(mOwningStream->GetParentObject(), nullptr);
-
-  MediaStreamGraph* graph = Graph();
-  newStream->InitOwnedStreamCommon(graph);
-  newStream->InitPlaybackStreamCommon(graph);
-
-  return newStream->CloneDOMTrack(*this, mTrackID);
+  RefPtr<MediaStreamTrack> newTrack = CloneInternal();
+  newTrack->SetEnabled(Enabled());
+  newTrack->SetMuted(Muted());
+  return newTrack.forget();
 }
 
 void MediaStreamTrack::SetReadyState(MediaStreamTrackState aState) {
@@ -442,9 +534,27 @@ void MediaStreamTrack::SetReadyState(MediaStreamTrackState aState) {
                aState == MediaStreamTrackState::Live),
              "We don't support overriding the ready state from ended to live");
 
+  if (Ended()) {
+    return;
+  }
+
   if (mReadyState == MediaStreamTrackState::Live &&
-      aState == MediaStreamTrackState::Ended && mSource) {
-    mSource->UnregisterSink(this);
+      aState == MediaStreamTrackState::Ended) {
+    if (mSource) {
+      mSource->UnregisterSink(mSink.get());
+    }
+    if (mMTGListener) {
+      RemoveListener(mMTGListener);
+    }
+    if (mPort) {
+      mPort->Destroy();
+    }
+    if (mTrack) {
+      mTrack->Destroy();
+    }
+    mPort = nullptr;
+    mTrack = nullptr;
+    mMTGListener = nullptr;
   }
 
   mReadyState = aState;
@@ -459,101 +569,66 @@ void MediaStreamTrack::OverrideEnded() {
 
   LOG(LogLevel::Info, ("MediaStreamTrack %p ended", this));
 
-  if (!mSource) {
-    MOZ_ASSERT(false);
-    return;
-  }
-
-  mSource->UnregisterSink(this);
-
-  mReadyState = MediaStreamTrackState::Ended;
+  SetReadyState(MediaStreamTrackState::Ended);
 
   NotifyEnded();
 
   DispatchTrustedEvent(NS_LITERAL_STRING("ended"));
 }
 
-DOMMediaStream* MediaStreamTrack::GetInputDOMStream() {
-  MediaStreamTrack* originalTrack =
-      mOriginalTrack ? mOriginalTrack.get() : this;
-  MOZ_RELEASE_ASSERT(originalTrack->mOwningStream);
-  return originalTrack->mOwningStream;
-}
-
-MediaStream* MediaStreamTrack::GetInputStream() {
-  DOMMediaStream* inputDOMStream = GetInputDOMStream();
-  MOZ_RELEASE_ASSERT(inputDOMStream->GetInputStream());
-  return inputDOMStream->GetInputStream();
-}
-
-ProcessedMediaStream* MediaStreamTrack::GetOwnedStream() {
-  if (!mOwningStream) {
-    return nullptr;
-  }
-
-  return mOwningStream->GetOwnedStream();
-}
-
-void MediaStreamTrack::AddListener(MediaStreamTrackListener* aListener) {
+void MediaStreamTrack::AddListener(MediaTrackListener* aListener) {
   LOG(LogLevel::Debug,
       ("MediaStreamTrack %p adding listener %p", this, aListener));
-  MOZ_ASSERT(GetOwnedStream());
-
-  GetOwnedStream()->AddTrackListener(aListener, mTrackID);
   mTrackListeners.AppendElement(aListener);
+
+  if (Ended()) {
+    return;
+  }
+  mTrack->AddListener(aListener);
 }
 
-void MediaStreamTrack::RemoveListener(MediaStreamTrackListener* aListener) {
+void MediaStreamTrack::RemoveListener(MediaTrackListener* aListener) {
   LOG(LogLevel::Debug,
       ("MediaStreamTrack %p removing listener %p", this, aListener));
+  mTrackListeners.RemoveElement(aListener);
 
-  if (GetOwnedStream()) {
-    GetOwnedStream()->RemoveTrackListener(aListener, mTrackID);
-    mTrackListeners.RemoveElement(aListener);
+  if (Ended()) {
+    return;
   }
+  mTrack->RemoveListener(aListener);
 }
 
-void MediaStreamTrack::AddDirectListener(
-    DirectMediaStreamTrackListener* aListener) {
+void MediaStreamTrack::AddDirectListener(DirectMediaTrackListener* aListener) {
   LOG(LogLevel::Debug, ("MediaStreamTrack %p (%s) adding direct listener %p to "
-                        "stream %p, track %d",
+                        "track %p",
                         this, AsAudioStreamTrack() ? "audio" : "video",
-                        aListener, GetOwnedStream(), mTrackID));
-  MOZ_ASSERT(GetOwnedStream());
-
-  GetOwnedStream()->AddDirectTrackListener(aListener, mTrackID);
+                        aListener, mTrack.get()));
   mDirectTrackListeners.AppendElement(aListener);
+
+  if (Ended()) {
+    return;
+  }
+  mTrack->AddDirectListener(aListener);
 }
 
 void MediaStreamTrack::RemoveDirectListener(
-    DirectMediaStreamTrackListener* aListener) {
+    DirectMediaTrackListener* aListener) {
   LOG(LogLevel::Debug,
-      ("MediaStreamTrack %p removing direct listener %p from stream %p", this,
-       aListener, GetOwnedStream()));
+      ("MediaStreamTrack %p removing direct listener %p from track %p", this,
+       aListener, mTrack.get()));
+  mDirectTrackListeners.RemoveElement(aListener);
 
-  if (GetOwnedStream()) {
-    GetOwnedStream()->RemoveDirectTrackListener(aListener, mTrackID);
-    mDirectTrackListeners.RemoveElement(aListener);
+  if (Ended()) {
+    return;
   }
+  mTrack->RemoveDirectListener(aListener);
 }
 
 already_AddRefed<MediaInputPort> MediaStreamTrack::ForwardTrackContentsTo(
-    ProcessedMediaStream* aStream, TrackID aDestinationTrackID) {
+    ProcessedMediaTrack* aTrack) {
   MOZ_ASSERT(NS_IsMainThread());
-  MOZ_RELEASE_ASSERT(aStream);
-  RefPtr<MediaInputPort> port = aStream->AllocateInputPort(
-      GetOwnedStream(), mTrackID, aDestinationTrackID);
-  return port.forget();
-}
-
-bool MediaStreamTrack::IsForwardedThrough(MediaInputPort* aPort) {
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(aPort);
-  if (!aPort) {
-    return false;
-  }
-  return aPort->GetSource() == GetOwnedStream() &&
-         aPort->PassTrackThrough(mTrackID);
+  MOZ_RELEASE_ASSERT(aTrack);
+  return aTrack->AllocateInputPort(mTrack);
 }
 
 }  // namespace dom
