@@ -39,31 +39,34 @@ void ChromiumCDMProxy::Init(PromiseId aPromiseId, const nsAString& aOrigin,
                             const nsAString& aTopLevelOrigin,
                             const nsAString& aGMPName) {
   MOZ_ASSERT(NS_IsMainThread());
+
+  RefPtr<GMPCrashHelper> helper(std::move(mCrashHelper));
+
   NS_ENSURE_TRUE_VOID(!mKeys.IsNull());
 
-  EME_LOG(
-      "ChromiumCDMProxy::Init (pid=%u, origin=%s, topLevelOrigin=%s, gmp=%s)",
-      aPromiseId, NS_ConvertUTF16toUTF8(aOrigin).get(),
-      NS_ConvertUTF16toUTF8(aTopLevelOrigin).get(),
-      NS_ConvertUTF16toUTF8(aGMPName).get());
+  EME_LOG("ChromiumCDMProxy::Init(this=%p, pid=%" PRIu32
+          ", origin=%s, topLevelOrigin=%s, "
+          "gmp=%s)",
+          this, aPromiseId, NS_ConvertUTF16toUTF8(aOrigin).get(),
+          NS_ConvertUTF16toUTF8(aTopLevelOrigin).get(),
+          NS_ConvertUTF16toUTF8(aGMPName).get());
 
   if (!mGMPThread) {
-    RejectPromise(
-        aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
+    RejectPromiseWithStateError(
+        aPromiseId,
         NS_LITERAL_CSTRING("Couldn't get GMP thread ChromiumCDMProxy::Init"));
     return;
   }
 
   if (aGMPName.IsEmpty()) {
-    RejectPromise(aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                  nsPrintfCString("Unknown GMP for keysystem '%s'",
-                                  NS_ConvertUTF16toUTF8(mKeySystem).get()));
+    RejectPromiseWithStateError(
+        aPromiseId, nsPrintfCString("Unknown GMP for keysystem '%s'",
+                                    NS_ConvertUTF16toUTF8(mKeySystem).get()));
     return;
   }
 
   gmp::NodeId nodeId(aOrigin, aTopLevelOrigin, aGMPName);
   RefPtr<AbstractThread> thread = mGMPThread;
-  RefPtr<GMPCrashHelper> helper(mCrashHelper);
   RefPtr<ChromiumCDMProxy> self(this);
   nsCString keySystem = NS_ConvertUTF16toUTF8(mKeySystem);
   RefPtr<Runnable> task(NS_NewRunnableFunction(
@@ -74,8 +77,8 @@ void ChromiumCDMProxy::Init(PromiseId aPromiseId, const nsAString& aOrigin,
         RefPtr<gmp::GeckoMediaPluginService> service =
             gmp::GeckoMediaPluginService::GetGeckoMediaPluginService();
         if (!service) {
-          self->RejectPromise(
-              aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
+          self->RejectPromiseWithStateError(
+              aPromiseId,
               NS_LITERAL_CSTRING("Couldn't get GeckoMediaPluginService in "
                                  "ChromiumCDMProxy::Init"));
           return;
@@ -91,36 +94,47 @@ void ChromiumCDMProxy::Init(PromiseId aPromiseId, const nsAString& aOrigin,
               cdm->Init(self->mCallback.get(),
                         self->mDistinctiveIdentifierRequired,
                         self->mPersistentStateRequired, self->mMainThread)
-                  ->Then(self->mMainThread, __func__,
-                         [self, aPromiseId, cdm](bool /* unused */) {
-                           // CDM init succeeded
-                           {
-                             MutexAutoLock lock(self->mCDMMutex);
-                             self->mCDM = cdm;
-                           }
-                           if (self->mIsShutdown) {
-                             self->RejectPromise(
-                                 aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                                 NS_LITERAL_CSTRING(
-                                     "ChromiumCDMProxy shutdown during "
-                                     "ChromiumCDMProxy::Init"));
-                             // If shutdown happened while waiting to init, we
-                             // need to explicitly shutdown the CDM to avoid it
-                             // referencing this proxy which is on its way out.
-                             self->ShutdownCDMIfExists();
-                             return;
-                           }
-                           self->OnCDMCreated(aPromiseId);
-                         },
-                         [self, aPromiseId](MediaResult aResult) {
-                           // CDM init failed
-                           self->RejectPromise(aPromiseId, aResult.Code(),
-                                               aResult.Message());
-                         });
+                  ->Then(
+                      self->mMainThread, __func__,
+                      [self, aPromiseId, cdm](bool /* unused */) {
+                        // CDM init succeeded
+                        {
+                          MutexAutoLock lock(self->mCDMMutex);
+                          self->mCDM = cdm;
+                        }
+                        if (self->mIsShutdown) {
+                          self->RejectPromiseWithStateError(
+                              aPromiseId, NS_LITERAL_CSTRING(
+                                              "ChromiumCDMProxy shutdown "
+                                              "during ChromiumCDMProxy::Init"));
+                          // If shutdown happened while waiting to init, we
+                          // need to explicitly shutdown the CDM to avoid it
+                          // referencing this proxy which is on its way out.
+                          self->ShutdownCDMIfExists();
+                          return;
+                        }
+                        self->OnCDMCreated(aPromiseId);
+                      },
+                      [self, aPromiseId](MediaResult aResult) {
+                        // CDM init failed.
+                        ErrorResult rv;
+                        // XXXbz MediaResult should really store a
+                        // CopyableErrorResult or something.  See
+                        // <https://bugzilla.mozilla.org/show_bug.cgi?id=1612216>.
+                        rv.Throw(aResult.Code());
+                        self->RejectPromise(aPromiseId, std::move(rv),
+                                            aResult.Message());
+                      });
             },
             [self, aPromiseId](MediaResult rv) {
               // service->GetCDM failed
-              self->RejectPromise(aPromiseId, rv.Code(), rv.Description());
+              ErrorResult result;
+              // XXXbz MediaResult should really store a CopyableErrorResult or
+              // something.  See
+              // <https://bugzilla.mozilla.org/show_bug.cgi?id=1612216>.
+              result.Throw(rv.Code());
+              self->RejectPromise(aPromiseId, std::move(result),
+                                  rv.Description());
             });
       }));
 
@@ -128,9 +142,9 @@ void ChromiumCDMProxy::Init(PromiseId aPromiseId, const nsAString& aOrigin,
 }
 
 void ChromiumCDMProxy::OnCDMCreated(uint32_t aPromiseId) {
-  EME_LOG("ChromiumCDMProxy::OnCDMCreated(pid=%u) isMainThread=%d this=%p",
-          aPromiseId, NS_IsMainThread(), this);
-
+  EME_LOG("ChromiumCDMProxy::OnCDMCreated(this=%p, pid=%" PRIu32
+          ") isMainThread=%d",
+          this, aPromiseId, NS_IsMainThread());
   MOZ_ASSERT(NS_IsMainThread());
   if (mKeys.IsNull()) {
     return;
@@ -142,8 +156,10 @@ void ChromiumCDMProxy::OnCDMCreated(uint32_t aPromiseId) {
     mKeys->OnCDMCreated(aPromiseId, cdm->PluginId());
   } else {
     // No CDM? Shouldn't be possible, but reject the promise anyway...
-    mKeys->RejectPromise(aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                         NS_LITERAL_CSTRING("Null CDM in OnCDMCreated()"));
+    NS_NAMED_LITERAL_CSTRING(err, "Null CDM in OnCDMCreated()");
+    ErrorResult rv;
+    rv.ThrowInvalidStateError(err);
+    mKeys->RejectPromise(aPromiseId, std::move(rv), err);
   }
 }
 
@@ -206,18 +222,20 @@ void ChromiumCDMProxy::CreateSession(uint32_t aCreateSessionToken,
                                      const nsAString& aInitDataType,
                                      nsTArray<uint8_t>& aInitData) {
   MOZ_ASSERT(NS_IsMainThread());
-  EME_LOG(
-      "ChromiumCDMProxy::CreateSession(token=%u, type=%d, pid=%u) "
-      "initDataLen=%zu",
-      aCreateSessionToken, (int)aSessionType, aPromiseId, aInitData.Length());
+  EME_LOG("ChromiumCDMProxy::CreateSession(this=%p, token=%" PRIu32
+          ", type=%d, pid=%" PRIu32
+          ") "
+          "initDataLen=%zu",
+          this, aCreateSessionToken, (int)aSessionType, aPromiseId,
+          aInitData.Length());
 
   uint32_t sessionType = ToCDMSessionType(aSessionType);
   uint32_t initDataType = ToCDMInitDataType(aInitDataType);
 
   RefPtr<gmp::ChromiumCDMParent> cdm = GetCDMParent();
   if (!cdm) {
-    RejectPromise(aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                  NS_LITERAL_CSTRING("Null CDM in CreateSession"));
+    RejectPromiseWithStateError(
+        aPromiseId, NS_LITERAL_CSTRING("Null CDM in CreateSession"));
     return;
   }
 
@@ -225,7 +243,7 @@ void ChromiumCDMProxy::CreateSession(uint32_t aCreateSessionToken,
                                          nsTArray<uint8_t>>(
       "gmp::ChromiumCDMParent::CreateSession", cdm,
       &gmp::ChromiumCDMParent::CreateSession, aCreateSessionToken, sessionType,
-      initDataType, aPromiseId, Move(aInitData)));
+      initDataType, aPromiseId, std::move(aInitData)));
 }
 
 void ChromiumCDMProxy::LoadSession(PromiseId aPromiseId,
@@ -235,8 +253,8 @@ void ChromiumCDMProxy::LoadSession(PromiseId aPromiseId,
 
   RefPtr<gmp::ChromiumCDMParent> cdm = GetCDMParent();
   if (!cdm) {
-    RejectPromise(aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                  NS_LITERAL_CSTRING("Null CDM in LoadSession"));
+    RejectPromiseWithStateError(aPromiseId,
+                                NS_LITERAL_CSTRING("Null CDM in LoadSession"));
     return;
   }
 
@@ -249,52 +267,56 @@ void ChromiumCDMProxy::LoadSession(PromiseId aPromiseId,
 void ChromiumCDMProxy::SetServerCertificate(PromiseId aPromiseId,
                                             nsTArray<uint8_t>& aCert) {
   MOZ_ASSERT(NS_IsMainThread());
-  EME_LOG("ChromiumCDMProxy::SetServerCertificate(pid=%u) certLen=%zu",
-          aPromiseId, aCert.Length());
+  EME_LOG("ChromiumCDMProxy::SetServerCertificate(this=%p, pid=%" PRIu32
+          ") certLen=%zu",
+          this, aPromiseId, aCert.Length());
 
   RefPtr<gmp::ChromiumCDMParent> cdm = GetCDMParent();
   if (!cdm) {
-    RejectPromise(aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                  NS_LITERAL_CSTRING("Null CDM in SetServerCertificate"));
+    RejectPromiseWithStateError(
+        aPromiseId, NS_LITERAL_CSTRING("Null CDM in SetServerCertificate"));
     return;
   }
 
   mGMPThread->Dispatch(NewRunnableMethod<uint32_t, nsTArray<uint8_t>>(
       "gmp::ChromiumCDMParent::SetServerCertificate", cdm,
-      &gmp::ChromiumCDMParent::SetServerCertificate, aPromiseId, Move(aCert)));
+      &gmp::ChromiumCDMParent::SetServerCertificate, aPromiseId,
+      std::move(aCert)));
 }
 
 void ChromiumCDMProxy::UpdateSession(const nsAString& aSessionId,
                                      PromiseId aPromiseId,
                                      nsTArray<uint8_t>& aResponse) {
   MOZ_ASSERT(NS_IsMainThread());
-  EME_LOG("ChromiumCDMProxy::UpdateSession(sid='%s', pid=%u) responseLen=%zu",
-          NS_ConvertUTF16toUTF8(aSessionId).get(), aPromiseId,
+  EME_LOG("ChromiumCDMProxy::UpdateSession(this=%p, sid='%s', pid=%" PRIu32
+          ") "
+          "responseLen=%zu",
+          this, NS_ConvertUTF16toUTF8(aSessionId).get(), aPromiseId,
           aResponse.Length());
 
   RefPtr<gmp::ChromiumCDMParent> cdm = GetCDMParent();
   if (!cdm) {
-    RejectPromise(aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                  NS_LITERAL_CSTRING("Null CDM in UpdateSession"));
+    RejectPromiseWithStateError(
+        aPromiseId, NS_LITERAL_CSTRING("Null CDM in UpdateSession"));
     return;
   }
   mGMPThread->Dispatch(
       NewRunnableMethod<nsCString, uint32_t, nsTArray<uint8_t>>(
           "gmp::ChromiumCDMParent::UpdateSession", cdm,
           &gmp::ChromiumCDMParent::UpdateSession,
-          NS_ConvertUTF16toUTF8(aSessionId), aPromiseId, Move(aResponse)));
+          NS_ConvertUTF16toUTF8(aSessionId), aPromiseId, std::move(aResponse)));
 }
 
 void ChromiumCDMProxy::CloseSession(const nsAString& aSessionId,
                                     PromiseId aPromiseId) {
   MOZ_ASSERT(NS_IsMainThread());
-  EME_LOG("ChromiumCDMProxy::CloseSession(sid='%s', pid=%u)",
-          NS_ConvertUTF16toUTF8(aSessionId).get(), aPromiseId);
+  EME_LOG("ChromiumCDMProxy::CloseSession(this=%p, sid='%s', pid=%" PRIu32 ")",
+          this, NS_ConvertUTF16toUTF8(aSessionId).get(), aPromiseId);
 
   RefPtr<gmp::ChromiumCDMParent> cdm = GetCDMParent();
   if (!cdm) {
-    RejectPromise(aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                  NS_LITERAL_CSTRING("Null CDM in CloseSession"));
+    RejectPromiseWithStateError(aPromiseId,
+                                NS_LITERAL_CSTRING("Null CDM in CloseSession"));
     return;
   }
   mGMPThread->Dispatch(NewRunnableMethod<nsCString, uint32_t>(
@@ -306,13 +328,13 @@ void ChromiumCDMProxy::CloseSession(const nsAString& aSessionId,
 void ChromiumCDMProxy::RemoveSession(const nsAString& aSessionId,
                                      PromiseId aPromiseId) {
   MOZ_ASSERT(NS_IsMainThread());
-  EME_LOG("ChromiumCDMProxy::RemoveSession(sid='%s', pid=%u)",
-          NS_ConvertUTF16toUTF8(aSessionId).get(), aPromiseId);
+  EME_LOG("ChromiumCDMProxy::RemoveSession(this=%p, sid='%s', pid=%" PRIu32 ")",
+          this, NS_ConvertUTF16toUTF8(aSessionId).get(), aPromiseId);
 
   RefPtr<gmp::ChromiumCDMParent> cdm = GetCDMParent();
   if (!cdm) {
-    RejectPromise(aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                  NS_LITERAL_CSTRING("Null CDM in RemoveSession"));
+    RejectPromiseWithStateError(
+        aPromiseId, NS_LITERAL_CSTRING("Null CDM in RemoveSession"));
     return;
   }
   mGMPThread->Dispatch(NewRunnableMethod<nsCString, uint32_t>(
@@ -333,23 +355,47 @@ void ChromiumCDMProxy::Shutdown() {
   ShutdownCDMIfExists();
 }
 
-void ChromiumCDMProxy::RejectPromise(PromiseId aId, nsresult aCode,
+void ChromiumCDMProxy::RejectPromise(PromiseId aId, ErrorResult&& aException,
                                      const nsCString& aReason) {
   if (!NS_IsMainThread()) {
+    // Use CopyableErrorResult to store our exception in the runnable,
+    // because ErrorResult is not OK to move across threads.
     mMainThread->Dispatch(
-        NewRunnableMethod<PromiseId, nsresult, nsCString>(
+        NewRunnableMethod<PromiseId, StoreCopyPassByRRef<CopyableErrorResult>,
+                          nsCString>(
             "ChromiumCDMProxy::RejectPromise", this,
-            &ChromiumCDMProxy::RejectPromise, aId, aCode, aReason),
+            &ChromiumCDMProxy::RejectPromiseOnMainThread, aId,
+            std::move(aException), aReason),
         NS_DISPATCH_NORMAL);
     return;
   }
-  EME_LOG("ChromiumCDMProxy::RejectPromise(pid=%u, code=0x%x, reason='%s')",
-          aId, static_cast<uint32_t>(aCode), aReason.get());
+  EME_LOG("ChromiumCDMProxy::RejectPromise(this=%p, pid=%" PRIu32
+          ", code=0x%x, "
+          "reason='%s')",
+          this, aId, aException.ErrorCodeAsInt(), aReason.get());
   if (!mKeys.IsNull()) {
-    mKeys->RejectPromise(aId, aCode, aReason);
+    mKeys->RejectPromise(aId, std::move(aException), aReason);
   }
 }
 
+void ChromiumCDMProxy::RejectPromiseWithStateError(PromiseId aId,
+                                                   const nsCString& aReason) {
+  ErrorResult rv;
+  rv.ThrowInvalidStateError(aReason);
+  RejectPromise(aId, std::move(rv), aReason);
+}
+
+void ChromiumCDMProxy::RejectPromiseOnMainThread(
+    PromiseId aId, CopyableErrorResult&& aException, const nsCString& aReason) {
+  // Moving into or out of a non-copyable ErrorResult will assert that both
+  // ErorResults are from our current thread.  Avoid the assertion by moving
+  // into a current-thread CopyableErrorResult first.  Note that this is safe,
+  // because CopyableErrorResult never holds state that can't move across
+  // threads.
+  CopyableErrorResult rv(std::move(aException));
+  RejectPromise(aId, std::move(rv), aReason);
+}
+
 void ChromiumCDMProxy::ResolvePromise(PromiseId aId) {
   if (!NS_IsMainThread()) {
     mMainThread->Dispatch(
@@ -359,7 +405,8 @@ void ChromiumCDMProxy::ResolvePromise(PromiseId aId) {
     return;
   }
 
-  EME_LOG("ChromiumCDMProxy::ResolvePromise(pid=%u)", aId);
+  EME_LOG("ChromiumCDMProxy::ResolvePromise(this=%p, pid=%" PRIu32 ")", this,
+          aId);
   if (!mKeys.IsNull()) {
     mKeys->ResolvePromise(aId);
   } else {
@@ -372,8 +419,9 @@ const nsCString& ChromiumCDMProxy::GetNodeId() const { return mNodeId; }
 void ChromiumCDMProxy::OnSetSessionId(uint32_t aCreateSessionToken,
                                       const nsAString& aSessionId) {
   MOZ_ASSERT(NS_IsMainThread());
-  EME_LOG("ChromiumCDMProxy::OnSetSessionId(token=%u, sid='%s')",
-          aCreateSessionToken, NS_ConvertUTF16toUTF8(aSessionId).get());
+  EME_LOG("ChromiumCDMProxy::OnSetSessionId(this=%p, token=%" PRIu32
+          ", sid='%s')",
+          this, aCreateSessionToken, NS_ConvertUTF16toUTF8(aSessionId).get());
 
   if (mKeys.IsNull()) {
     return;
@@ -480,10 +528,10 @@ void ChromiumCDMProxy::OnSessionError(const nsAString& aSessionId,
 }
 
 void ChromiumCDMProxy::OnRejectPromise(uint32_t aPromiseId,
-                                       nsresult aDOMException,
+                                       ErrorResult&& aException,
                                        const nsCString& aMsg) {
   MOZ_ASSERT(NS_IsMainThread());
-  RejectPromise(aPromiseId, aDOMException, aMsg);
+  RejectPromise(aPromiseId, std::move(aException), aMsg);
 }
 
 const nsString& ChromiumCDMProxy::KeySystem() const { return mKeySystem; }
@@ -504,13 +552,14 @@ RefPtr<DecryptPromise> ChromiumCDMProxy::Decrypt(MediaRawData* aSample) {
 void ChromiumCDMProxy::GetStatusForPolicy(PromiseId aPromiseId,
                                           const nsAString& aMinHdcpVersion) {
   MOZ_ASSERT(NS_IsMainThread());
-  EME_LOG("ChromiumCDMProxy::GetStatusForPolicy(pid=%u) minHdcpVersion=%s",
-          aPromiseId, NS_ConvertUTF16toUTF8(aMinHdcpVersion).get());
+  EME_LOG("ChromiumCDMProxy::GetStatusForPolicy(this=%p, pid=%" PRIu32
+          ") minHdcpVersion=%s",
+          this, aPromiseId, NS_ConvertUTF16toUTF8(aMinHdcpVersion).get());
 
   RefPtr<gmp::ChromiumCDMParent> cdm = GetCDMParent();
   if (!cdm) {
-    RejectPromise(aPromiseId, NS_ERROR_DOM_INVALID_STATE_ERR,
-                  NS_LITERAL_CSTRING("Null CDM in GetStatusForPolicy"));
+    RejectPromiseWithStateError(
+        aPromiseId, NS_LITERAL_CSTRING("Null CDM in GetStatusForPolicy"));
     return;
   }
 
