@@ -5,11 +5,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #if !defined(GMPCrashHelper_h_)
-#define GMPCrashHelper_h_
+#  define GMPCrashHelper_h_
 
-#include "MainThreadUtils.h"
-#include "nsISupportsImpl.h"
-#include "nsPIDOMWindow.h"
+#  include "MainThreadUtils.h"
+#  include "nsISupportsImpl.h"
+#  include "nsPIDOMWindow.h"
 
 namespace mozilla {
 
@@ -20,8 +20,7 @@ namespace mozilla {
 // that instances are destroyed on the main thread.
 class GMPCrashHelper {
  public:
-  NS_METHOD_(MozExternalRefCountType) AddRef(void);
-  NS_METHOD_(MozExternalRefCountType) Release(void);
+  NS_INLINE_DECL_THREADSAFE_REFCOUNTING_WITH_DESTROY(GMPCrashHelper, Destroy());
 
   // Called on the main thread.
   virtual already_AddRefed<nsPIDOMWindowInner>
@@ -30,8 +29,6 @@ class GMPCrashHelper {
  protected:
   virtual ~GMPCrashHelper() { MOZ_ASSERT(NS_IsMainThread()); }
   void Destroy();
-  mozilla::ThreadSafeAutoRefCnt mRefCnt;
-  NS_DECL_OWNINGTHREAD
 };
 
 }  // namespace mozilla
