@@ -4,16 +4,34 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsPrintfCString.h"
-#include "MediaQueue.h"
 #include "AudioSink.h"
-#include "VideoUtils.h"
 #include "AudioConverter.h"
-
+#include "AudioDeviceInfo.h"
+#include "MediaQueue.h"
+#include "VideoUtils.h"
 #include "mozilla/CheckedInt.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/IntegerPrintfMacros.h"
-#include "MediaPrefs.h"
+#include "mozilla/StaticPrefs_media.h"
+#include "nsPrintfCString.h"
+
+#ifdef MOZ_GECKO_PROFILER
+#  include "ProfilerMarkerPayload.h"
+#  define PROFILER_MARKER(tag, sample)                                    \
+    do {                                                                  \
+      uint64_t startTime = (sample)->mTime.ToMicroseconds();              \
+      uint64_t endTime = (sample)->GetEndTime().ToMicroseconds();         \
+      auto profilerTag = (tag);                                           \
+      mOwnerThread->Dispatch(NS_NewRunnableFunction(                      \
+          "AudioSink:AddMarker", [profilerTag, startTime, endTime] {      \
+            PROFILER_ADD_MARKER_WITH_PAYLOAD(profilerTag, MEDIA_PLAYBACK, \
+                                             MediaSampleMarkerPayload,    \
+                                             (startTime, endTime));       \
+          }));                                                            \
+    } while (0)
+#else
+#  define PROFILER_MARKER(tag, sample)
+#endif
 
 namespace mozilla {
 
@@ -25,19 +43,21 @@ extern LazyLogModule gMediaDecoderLog;
   MOZ_LOG(gMediaDecoderLog, LogLevel::Verbose, \
           ("AudioSink=%p " msg, this, ##__VA_ARGS__))
 
-namespace media {
-
 // The amount of audio frames that is used to fuzz rounding errors.
 static const int64_t AUDIO_FUZZ_FRAMES = 1;
 
 // Amount of audio frames we will be processing ahead of use
 static const int32_t LOW_AUDIO_USECS = 300000;
 
+using media::TimeUnit;
+
 AudioSink::AudioSink(AbstractThread* aThread,
                      MediaQueue<AudioData>& aAudioQueue,
-                     const TimeUnit& aStartTime, const AudioInfo& aInfo)
+                     const TimeUnit& aStartTime, const AudioInfo& aInfo,
+                     AudioDeviceInfo* aAudioDevice)
     : mStartTime(aStartTime),
       mInfo(aInfo),
+      mAudioDevice(aAudioDevice),
       mPlaying(true),
       mMonitor("AudioSink"),
       mWritten(0),
@@ -48,7 +68,7 @@ AudioSink::AudioSink(AbstractThread* aThread,
       mFramesParsed(0),
       mIsAudioDataAudible(false),
       mAudioQueue(aAudioQueue) {
-  bool resampling = MediaPrefs::AudioSinkResampling();
+  bool resampling = StaticPrefs::media_resampling_enabled();
 
   if (resampling) {
     mOutputRate = 48000;
@@ -64,35 +84,31 @@ AudioSink::AudioSink(AbstractThread* aThread,
   }
   MOZ_DIAGNOSTIC_ASSERT(mOutputRate, "output rate can't be 0.");
 
-  bool monoAudioEnabled = MediaPrefs::MonoAudio();
-
-  mOutputChannels =
-      monoAudioEnabled
-          ? 1
-          : (MediaPrefs::AudioSinkForceStereo() ? 2 : mInfo.mChannels);
+  mOutputChannels = DecideAudioPlaybackChannels(mInfo);
 }
 
-AudioSink::~AudioSink() {}
+AudioSink::~AudioSink() = default;
 
-RefPtr<GenericPromise> AudioSink::Init(const PlaybackParams& aParams) {
+nsresult AudioSink::Init(const PlaybackParams& aParams,
+                         RefPtr<MediaSink::EndedPromise>& aEndedPromise) {
   MOZ_ASSERT(mOwnerThread->IsCurrentThreadIn());
 
   mAudioQueueListener = mAudioQueue.PushEvent().Connect(
       mOwnerThread, this, &AudioSink::OnAudioPushed);
   mAudioQueueFinishListener = mAudioQueue.FinishEvent().Connect(
       mOwnerThread, this, &AudioSink::NotifyAudioNeeded);
-  mProcessedQueueListener = mProcessedQueue.PopEvent().Connect(
+  mProcessedQueueListener = mProcessedQueue.PopFrontEvent().Connect(
       mOwnerThread, this, &AudioSink::OnAudioPopped);
 
   // To ensure at least one audio packet will be popped from AudioQueue and
   // ready to be played.
   NotifyAudioNeeded();
-  RefPtr<GenericPromise> p = mEndPromise.Ensure(__func__);
+  aEndedPromise = mEndedPromise.Ensure(__func__);
   nsresult rv = InitializeAudioStream(aParams);
   if (NS_FAILED(rv)) {
-    mEndPromise.Reject(rv, __func__);
+    mEndedPromise.Reject(rv, __func__);
   }
-  return p;
+  return rv;
 }
 
 TimeUnit AudioSink::GetPosition() {
@@ -101,6 +117,11 @@ TimeUnit AudioSink::GetPosition() {
     TimeUnit pos = TimeUnit::FromMicroseconds(tmp);
     NS_ASSERTION(pos >= mLastGoodPosition,
                  "AudioStream position shouldn't go backward");
+    TimeUnit tmp = mStartTime + pos;
+    if (!tmp.IsValid()) {
+      mErrored = true;
+      return mStartTime + mLastGoodPosition;
+    }
     // Update the last good position when we got a good one.
     if (pos >= mLastGoodPosition) {
       mLastGoodPosition = pos;
@@ -135,7 +156,7 @@ void AudioSink::Shutdown() {
   }
   mProcessedQueue.Reset();
   mProcessedQueue.Finish();
-  mEndPromise.ResolveIfExists(true, __func__);
+  mEndedPromise.ResolveIfExists(true, __func__);
 }
 
 void AudioSink::SetVolume(double aVolume) {
@@ -175,13 +196,15 @@ nsresult AudioSink::InitializeAudioStream(const PlaybackParams& aParams) {
   mAudioStream = new AudioStream(*this);
   // When AudioQueue is empty, there is no way to know the channel layout of
   // the coming audio data, so we use the predefined channel map instead.
-  uint32_t channelMap =
+  AudioConfig::ChannelLayout::ChannelMap channelMap =
       mConverter ? mConverter->OutputConfig().Layout().Map()
-                 : AudioStream::GetPreferredChannelMap(mOutputChannels);
+                 : AudioConfig::ChannelLayout(mOutputChannels).Map();
   // The layout map used here is already processed by mConverter with
   // mOutputChannels into SMPTE format, so there is no need to worry if
-  // MediaPrefs::MonoAudio() or MediaPrefs::AudioSinkForceStereo() is applied.
-  nsresult rv = mAudioStream->Init(mOutputChannels, channelMap, mOutputRate);
+  // StaticPrefs::accessibility_monoaudio_enable() or
+  // StaticPrefs::media_forcestereo_enabled() is applied.
+  nsresult rv = mAudioStream->Init(mOutputChannels, channelMap, mOutputRate,
+                                   mAudioDevice);
   if (NS_FAILED(rv)) {
     mAudioStream->Shutdown();
     mAudioStream = nullptr;
@@ -193,9 +216,7 @@ nsresult AudioSink::InitializeAudioStream(const PlaybackParams& aParams) {
   mAudioStream->SetVolume(aParams.mVolume);
   mAudioStream->SetPlaybackRate(aParams.mPlaybackRate);
   mAudioStream->SetPreservesPitch(aParams.mPreservesPitch);
-  mAudioStream->Start();
-
-  return NS_OK;
+  return mAudioStream->Start();
 }
 
 TimeUnit AudioSink::GetEndTime() const {
@@ -250,20 +271,21 @@ UniquePtr<AudioStream::Chunk> AudioSink::PopFrames(uint32_t aFrames) {
     mCurrentData = mProcessedQueue.PeekFront();
     {
       MonitorAutoLock mon(mMonitor);
-      mCursor = MakeUnique<AudioBufferCursor>(mCurrentData->mAudioData.get(),
+      mCursor = MakeUnique<AudioBufferCursor>(mCurrentData->Data(),
                                               mCurrentData->mChannels,
-                                              mCurrentData->mFrames);
+                                              mCurrentData->Frames());
     }
-    MOZ_ASSERT(mCurrentData->mFrames > 0);
+    MOZ_ASSERT(mCurrentData->Frames() > 0);
     mProcessedQueueLength -=
-        FramesToUsecs(mCurrentData->mFrames, mOutputRate).value();
+        FramesToUsecs(mCurrentData->Frames(), mOutputRate).value();
   }
 
   auto framesToPop = std::min(aFrames, mCursor->Available());
 
   SINK_LOG_V("playing audio at time=%" PRId64 " offset=%u length=%u",
              mCurrentData->mTime.ToMicroseconds(),
-             mCurrentData->mFrames - mCursor->Available(), framesToPop);
+             mCurrentData->Frames() - mCursor->Available(), framesToPop);
+  PROFILER_MARKER("PlayAudio", mCurrentData);
 
   UniquePtr<AudioStream::Chunk> chunk =
       MakeUnique<Chunk>(mCurrentData, framesToPop, mCursor->Ptr());
@@ -298,7 +320,13 @@ bool AudioSink::Ended() const {
 void AudioSink::Drained() {
   SINK_LOG("Drained");
   mPlaybackComplete = true;
-  mEndPromise.ResolveIfExists(true, __func__);
+  mEndedPromise.ResolveIfExists(true, __func__);
+}
+
+void AudioSink::Errored() {
+  SINK_LOG("Errored");
+  mPlaybackComplete = true;
+  mEndedPromise.RejectIfExists(NS_ERROR_FAILURE, __func__);
 }
 
 void AudioSink::CheckIsAudible(const AudioData* aData) {
@@ -333,7 +361,7 @@ void AudioSink::NotifyAudioNeeded() {
     RefPtr<AudioData> data = mAudioQueue.PopFront();
 
     // Ignore the element with 0 frames and try next.
-    if (!data->mFrames) {
+    if (!data->Frames()) {
       continue;
     }
 
@@ -362,9 +390,23 @@ void AudioSink::NotifyAudioNeeded() {
         mFramesParsed = result.value();
       }
 
-      mConverter =
-          MakeUnique<AudioConverter>(AudioConfig(data->mChannels, data->mRate),
-                                     AudioConfig(mOutputChannels, mOutputRate));
+      const AudioConfig::ChannelLayout inputLayout =
+          data->mChannelMap
+              ? AudioConfig::ChannelLayout::SMPTEDefault(data->mChannelMap)
+              : AudioConfig::ChannelLayout(data->mChannels);
+      const AudioConfig::ChannelLayout outputLayout =
+          mOutputChannels == data->mChannels
+              ? inputLayout
+              : AudioConfig::ChannelLayout(mOutputChannels);
+      AudioConfig inConfig =
+          AudioConfig(inputLayout, data->mChannels, data->mRate);
+      AudioConfig outConfig =
+          AudioConfig(outputLayout, mOutputChannels, mOutputRate);
+      if (!AudioConverter::CanConvert(inConfig, outConfig)) {
+        mErrored = true;
+        return;
+      }
+      mConverter = MakeUnique<AudioConverter>(inConfig, outConfig);
     }
 
     // See if there's a gap in the audio. If there is, push silence into the
@@ -377,7 +419,7 @@ void AudioSink::NotifyAudioNeeded() {
     // hardware.
     CheckedInt64 missingFrames = sampleTime - mFramesParsed;
 
-    if (!missingFrames.isValid()) {
+    if (!missingFrames.isValid() || !sampleTime.isValid()) {
       NS_WARNING("Int overflow in AudioSink");
       mErrored = true;
       return;
@@ -400,27 +442,23 @@ void AudioSink::NotifyAudioNeeded() {
       }
       if (mConverter->InputConfig() != mConverter->OutputConfig()) {
         AlignedAudioBuffer convertedData =
-            mConverter->Process(AudioSampleBuffer(Move(silenceBuffer)))
+            mConverter->Process(AudioSampleBuffer(std::move(silenceBuffer)))
                 .Forget();
-        silenceData = CreateAudioFromBuffer(Move(convertedData), data);
+        silenceData = CreateAudioFromBuffer(std::move(convertedData), data);
       } else {
-        silenceData = CreateAudioFromBuffer(Move(silenceBuffer), data);
+        silenceData = CreateAudioFromBuffer(std::move(silenceBuffer), data);
       }
       PushProcessedAudio(silenceData);
     }
 
     mLastEndTime = data->GetEndTime();
-    mFramesParsed += data->mFrames;
+    mFramesParsed += data->Frames();
 
     if (mConverter->InputConfig() != mConverter->OutputConfig()) {
-      // We must ensure that the size in the buffer contains exactly the number
-      // of frames, in case one of the audio producer over allocated the buffer.
-      AlignedAudioBuffer buffer(Move(data->mAudioData));
-      buffer.SetLength(size_t(data->mFrames) * data->mChannels);
-
+      AlignedAudioBuffer buffer(data->MoveableData());
       AlignedAudioBuffer convertedData =
-          mConverter->Process(AudioSampleBuffer(Move(buffer))).Forget();
-      data = CreateAudioFromBuffer(Move(convertedData), data);
+          mConverter->Process(AudioSampleBuffer(std::move(buffer))).Forget();
+      data = CreateAudioFromBuffer(std::move(convertedData), data);
     }
     if (PushProcessedAudio(data)) {
       mLastProcessedPacket = Some(data);
@@ -435,12 +473,12 @@ void AudioSink::NotifyAudioNeeded() {
 }
 
 uint32_t AudioSink::PushProcessedAudio(AudioData* aData) {
-  if (!aData || !aData->mFrames) {
+  if (!aData || !aData->Frames()) {
     return 0;
   }
   mProcessedQueue.Push(aData);
-  mProcessedQueueLength += FramesToUsecs(aData->mFrames, mOutputRate).value();
-  return aData->mFrames;
+  mProcessedQueueLength += FramesToUsecs(aData->Frames(), mOutputRate).value();
+  return aData->Frames();
 }
 
 already_AddRefed<AudioData> AudioSink::CreateAudioFromBuffer(
@@ -456,8 +494,9 @@ already_AddRefed<AudioData> AudioSink::CreateAudioFromBuffer(
     return nullptr;
   }
   RefPtr<AudioData> data =
-      new AudioData(aReference->mOffset, aReference->mTime, duration, frames,
-                    Move(aBuffer), mOutputChannels, mOutputRate);
+      new AudioData(aReference->mOffset, aReference->mTime, std::move(aBuffer),
+                    mOutputChannels, mOutputRate);
+  MOZ_DIAGNOSTIC_ASSERT(duration == data->mDuration, "must be equal");
   return data.forget();
 }
 
@@ -485,23 +524,25 @@ uint32_t AudioSink::DrainConverter(uint32_t aMaxFrames) {
   }
 
   RefPtr<AudioData> data =
-      CreateAudioFromBuffer(Move(convertedData), lastPacket);
+      CreateAudioFromBuffer(std::move(convertedData), lastPacket);
   if (!data) {
     return 0;
   }
   mProcessedQueue.Push(data);
-  return data->mFrames;
+  return data->Frames();
 }
 
-nsCString AudioSink::GetDebugInfo() {
+void AudioSink::GetDebugInfo(dom::MediaSinkDebugInfo& aInfo) {
   MOZ_ASSERT(mOwnerThread->IsCurrentThreadIn());
-  return nsPrintfCString(
-      "AudioSink: StartTime=%" PRId64 " LastGoodPosition=%" PRId64
-      " Playing=%d  OutputRate=%u Written=%" PRId64
-      " Errored=%d PlaybackComplete=%d",
-      mStartTime.ToMicroseconds(), mLastGoodPosition.ToMicroseconds(), mPlaying,
-      mOutputRate, mWritten, bool(mErrored), bool(mPlaybackComplete));
+  aInfo.mAudioSinkWrapper.mAudioSink.mStartTime = mStartTime.ToMicroseconds();
+  aInfo.mAudioSinkWrapper.mAudioSink.mLastGoodPosition =
+      mLastGoodPosition.ToMicroseconds();
+  aInfo.mAudioSinkWrapper.mAudioSink.mIsPlaying = mPlaying;
+  aInfo.mAudioSinkWrapper.mAudioSink.mOutputRate = mOutputRate;
+  aInfo.mAudioSinkWrapper.mAudioSink.mWritten = mWritten;
+  aInfo.mAudioSinkWrapper.mAudioSink.mHasErrored = bool(mErrored);
+  aInfo.mAudioSinkWrapper.mAudioSink.mPlaybackComplete =
+      bool(mPlaybackComplete);
 }
 
-}  // namespace media
 }  // namespace mozilla
