@@ -7,13 +7,17 @@
 #ifndef AudioContext_h_
 #define AudioContext_h_
 
+#include "X11UndefineNone.h"
+#include "AudioParamDescriptorMap.h"
 #include "mozilla/dom/OfflineAudioContextBinding.h"
+#include "mozilla/dom/AudioContextBinding.h"
 #include "MediaBufferDecoder.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/DOMEventTargetHelper.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/dom/TypedArray.h"
 #include "mozilla/RelativeTimeline.h"
+#include "mozilla/TypedEnumBits.h"
 #include "mozilla/UniquePtr.h"
 #include "nsCOMPtr.h"
 #include "nsCycleCollectionParticipant.h"
@@ -22,12 +26,6 @@
 #include "js/TypeDecls.h"
 #include "nsIMemoryReporter.h"
 
-// X11 has a #define for CurrentTime. Unbelievable :-(.
-// See dom/media/DOMMediaStream.h for more fun!
-#ifdef CurrentTime
-#undef CurrentTime
-#endif
-
 namespace WebCore {
 class PeriodicWave;
 }  // namespace WebCore
@@ -38,9 +36,9 @@ namespace mozilla {
 
 class DOMMediaStream;
 class ErrorResult;
-class MediaStream;
-class MediaStreamGraph;
-class AudioNodeStream;
+class MediaTrack;
+class MediaTrackGraph;
+class AudioNodeTrack;
 
 namespace dom {
 
@@ -65,18 +63,21 @@ class IIRFilterNode;
 class MediaElementAudioSourceNode;
 class MediaStreamAudioDestinationNode;
 class MediaStreamAudioSourceNode;
+class MediaStreamTrack;
+class MediaStreamTrackAudioSourceNode;
 class OscillatorNode;
 class PannerNode;
 class ScriptProcessorNode;
 class StereoPannerNode;
 class WaveShaperNode;
+class Worklet;
 class PeriodicWave;
 struct PeriodicWaveConstraints;
 class Promise;
 enum class OscillatorType : uint8_t;
 
 // This is addrefed by the OscillatorNodeEngine on the main thread
-// and then used from the MSG thread.
+// and then used from the MTG thread.
 // It can be released either from the graph thread or the main thread.
 class BasicWaveFormCache {
  public:
@@ -92,7 +93,7 @@ class BasicWaveFormCache {
   uint32_t mSampleRate;
 };
 
-/* This runnable allows the MSG to notify the main thread when audio is actually
+/* This runnable allows the MTG to notify the main thread when audio is actually
  * flowing */
 class StateChangeTask final : public Runnable {
  public:
@@ -103,7 +104,7 @@ class StateChangeTask final : public Runnable {
 
   /* This constructor should be used when this event is sent from the audio
    * thread. */
-  StateChangeTask(AudioNodeStream* aStream, void* aPromise,
+  StateChangeTask(AudioNodeTrack* aTrack, void* aPromise,
                   AudioContextState aNewState);
 
   NS_IMETHOD Run() override;
@@ -111,11 +112,21 @@ class StateChangeTask final : public Runnable {
  private:
   RefPtr<AudioContext> mAudioContext;
   void* mPromise;
-  RefPtr<AudioNodeStream> mAudioNodeStream;
+  RefPtr<AudioNodeTrack> mAudioNodeTrack;
   AudioContextState mNewState;
 };
 
 enum class AudioContextOperation { Suspend, Resume, Close };
+// When suspending or resuming an AudioContext, some operations have to notify
+// the main thread, so that the Promise is resolved, the state is modified, and
+// the statechanged event is sent. Some other operations don't go back to the
+// main thread, for example when the AudioContext is paused by something that is
+// not caused by the page itself: opening a debugger, breaking on a breakpoint,
+// reloading a document.
+enum class AudioContextOperationFlags { None, SendStateChange };
+MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(AudioContextOperationFlags);
+
+struct AudioContextOptions;
 
 class AudioContext final : public DOMEventTargetHelper,
                            public nsIMemoryReporter,
@@ -136,6 +147,8 @@ class AudioContext final : public DOMEventTargetHelper,
 
   nsPIDOMWindowInner* GetParentObject() const { return GetOwner(); }
 
+  nsISerialEventTarget* GetMainThread() const;
+
   virtual void DisconnectFromOwner() override;
 
   void Shutdown();  // idempotent
@@ -146,8 +159,9 @@ class AudioContext final : public DOMEventTargetHelper,
   using DOMEventTargetHelper::DispatchTrustedEvent;
 
   // Constructor for regular AudioContext
-  static already_AddRefed<AudioContext> Constructor(const GlobalObject& aGlobal,
-                                                    ErrorResult& aRv);
+  static already_AddRefed<AudioContext> Constructor(
+      const GlobalObject& aGlobal, const AudioContextOptions& aOptions,
+      ErrorResult& aRv);
 
   // Constructor for offline AudioContext with options object
   static already_AddRefed<AudioContext> Constructor(
@@ -167,20 +181,37 @@ class AudioContext final : public DOMEventTargetHelper,
 
   float SampleRate() const { return mSampleRate; }
 
-  bool ShouldSuspendNewStream() const { return mSuspendCalled; }
+  bool ShouldSuspendNewTrack() const { return mSuspendCalled || mCloseCalled; }
 
   double CurrentTime();
 
   AudioListener* Listener();
 
   AudioContextState State() const { return mAudioContextState; }
+
+  double BaseLatency() const {
+    // Gecko does not do any buffering between rendering the audio and sending
+    // it to the audio subsystem.
+    return 0.0;
+  }
+
+  double OutputLatency();
+
+  void GetOutputTimestamp(AudioTimestamp& aTimeStamp);
+
+  Worklet* GetAudioWorklet(ErrorResult& aRv);
+
   bool IsRunning() const;
 
+  // Called when an AudioScheduledSourceNode started or the source node starts,
+  // this method might resume the AudioContext if it was not allowed to start.
+  void StartBlockedAudioContextIfAllowed();
+
   // Those three methods return a promise to content, that is resolved when an
-  // (possibly long) operation is completed on the MSG (and possibly other)
+  // (possibly long) operation is completed on the MTG (and possibly other)
   // thread(s). To avoid having to match the calls and asychronous result when
   // the operation is completed, we keep a reference to the promises on the main
-  // thread, and then send the promises pointers down the MSG thread, as a void*
+  // thread, and then send the promises pointers down the MTG thread, as a void*
   // (to make it very clear that the pointer is to merely be treated as an ID).
   // When back on the main thread, we can resolve or reject the promise, by
   // casting it back to a `Promise*` while asserting we're back on the main
@@ -190,9 +221,17 @@ class AudioContext final : public DOMEventTargetHelper,
   already_AddRefed<Promise> Close(ErrorResult& aRv);
   IMPL_EVENT_HANDLER(statechange)
 
-  already_AddRefed<AudioBufferSourceNode> CreateBufferSource(ErrorResult& aRv);
+  // These two functions are similar with Suspend() and Resume(), the difference
+  // is they are designed for calling from chrome side, not content side. eg.
+  // calling from inner window, so we won't need to return promise for caller.
+  void SuspendFromChrome();
+  void ResumeFromChrome();
+  // Called on completion of offline rendering:
+  void OfflineClose();
 
-  already_AddRefed<ConstantSourceNode> CreateConstantSource(ErrorResult& aRv);
+  already_AddRefed<AudioBufferSourceNode> CreateBufferSource();
+
+  already_AddRefed<ConstantSourceNode> CreateConstantSource();
 
   already_AddRefed<AudioBuffer> CreateBuffer(uint32_t aNumberOfChannels,
                                              uint32_t aLength,
@@ -218,6 +257,9 @@ class AudioContext final : public DOMEventTargetHelper,
       HTMLMediaElement& aMediaElement, ErrorResult& aRv);
   already_AddRefed<MediaStreamAudioSourceNode> CreateMediaStreamSource(
       DOMMediaStream& aMediaStream, ErrorResult& aRv);
+  already_AddRefed<MediaStreamTrackAudioSourceNode>
+  CreateMediaStreamTrackSource(MediaStreamTrack& aMediaStreamTrack,
+                               ErrorResult& aRv);
 
   already_AddRefed<DelayNode> CreateDelay(double aMaxDelayTime,
                                           ErrorResult& aRv);
@@ -260,8 +302,8 @@ class AudioContext final : public DOMEventTargetHelper,
 
   bool IsOffline() const { return mIsOffline; }
 
-  MediaStreamGraph* Graph() const;
-  MediaStream* DestinationStream() const;
+  MediaTrackGraph* Graph() const;
+  AudioNodeTrack* DestinationTrack() const;
 
   // Nodes register here if they will produce sound even if they have silent
   // or no input connections.  The AudioContext will keep registered nodes
@@ -276,10 +318,6 @@ class AudioContext final : public DOMEventTargetHelper,
   // unregistering would not be safe.
   void UnregisterActiveNode(AudioNode* aNode);
 
-  void UnregisterAudioBufferSourceNode(AudioBufferSourceNode* aNode);
-  void UnregisterPannerNode(PannerNode* aNode);
-  void UpdatePannerSource();
-
   uint32_t MaxChannelCount() const;
 
   uint32_t ActiveNodeCount() const;
@@ -287,8 +325,6 @@ class AudioContext final : public DOMEventTargetHelper,
   void Mute() const;
   void Unmute() const;
 
-  JSObject* GetGlobalJSObject() const;
-
   void RegisterNode(AudioNode* aNode);
   void UnregisterNode(AudioNode* aNode);
 
@@ -296,7 +332,14 @@ class AudioContext final : public DOMEventTargetHelper,
 
   BasicWaveFormCache* GetBasicWaveFormCache();
 
-  bool CheckClosed(ErrorResult& aRv);
+  void ShutdownWorklet();
+  // Steals from |aParamMap|
+  void SetParamMapForWorkletName(const nsAString& aName,
+                                 AudioParamDescriptorMap* aParamMap);
+  const AudioParamDescriptorMap* GetParamMapForWorkletName(
+      const nsAString& aName) {
+    return mWorkletParamDescriptors.GetValue(aName);
+  }
 
   void Dispatch(already_AddRefed<nsIRunnable>&& aRunnable);
 
@@ -310,12 +353,33 @@ class AudioContext final : public DOMEventTargetHelper,
 
   friend struct ::mozilla::WebAudioDecodeJob;
 
-  nsTArray<MediaStream*> GetAllStreams() const;
+  nsTArray<mozilla::MediaTrack*> GetAllTracks() const;
+
+  void ResumeInternal(AudioContextOperationFlags aFlags);
+  void SuspendInternal(void* aPromise, AudioContextOperationFlags aFlags);
+  void CloseInternal(void* aPromise, AudioContextOperationFlags aFlags);
+
+  // Will report error message to console and dispatch testing event if needed
+  // when AudioContext is blocked by autoplay policy.
+  void ReportBlocked();
+
+  void ReportToConsole(uint32_t aErrorFlags, const char* aMsg) const;
+
+  // This function should be called everytime we decide whether allow to start
+  // audio context, it's used to update Telemetry related variables.
+  void UpdateAutoplayAssumptionStatus();
+
+  // These functions are used for updating Telemetry.
+  // - MaybeUpdateAutoplayTelemetry: update category 'AllowedAfterBlocked'
+  // - MaybeUpdateAutoplayTelemetryWhenShutdown: update category 'NeverBlocked'
+  //   and 'NeverAllowed', so we need to call it when shutdown AudioContext
+  void MaybeUpdateAutoplayTelemetry();
+  void MaybeUpdateAutoplayTelemetryWhenShutdown();
 
  private:
-  // Each AudioContext has an id, that is passed down the MediaStreams that
+  // Each AudioContext has an id, that is passed down the MediaTracks that
   // back the AudioNodes, so we can easily compute the set of all the
-  // MediaStreams for a given context, on the MediasStreamGraph side.
+  // MediaTracks for a given context, on the MediasTrackGraph side.
   const AudioContextId mId;
   // Note that it's important for mSampleRate to be initialized before
   // mDestination, as mDestination's constructor needs to access it!
@@ -323,18 +387,23 @@ class AudioContext final : public DOMEventTargetHelper,
   AudioContextState mAudioContextState;
   RefPtr<AudioDestinationNode> mDestination;
   RefPtr<AudioListener> mListener;
+  RefPtr<Worklet> mWorklet;
   nsTArray<UniquePtr<WebAudioDecodeJob>> mDecodeJobs;
-  // This array is used to keep the suspend/resume/close promises alive until
+  // This array is used to keep the suspend/close promises alive until
   // they are resolved, so we can safely pass them accross threads.
   nsTArray<RefPtr<Promise>> mPromiseGripArray;
+  // This array is used to onlly keep the resume promises alive until they are
+  // resolved, so we can safely pass them accross threads. If the audio context
+  // is not allowed to play, the promise would be pending in this array and be
+  // resolved until audio context has been allowed and user call resume() again.
+  nsTArray<RefPtr<Promise>> mPendingResumePromises;
   // See RegisterActiveNode.  These will keep the AudioContext alive while it
   // is rendering and the window remains alive.
   nsTHashtable<nsRefPtrHashKey<AudioNode>> mActiveNodes;
   // Raw (non-owning) references to all AudioNodes for this AudioContext.
   nsTHashtable<nsPtrHashKey<AudioNode>> mAllNodes;
-  // Hashsets containing all the PannerNodes, to compute the doppler shift.
-  // These are weak pointers.
-  nsTHashtable<nsPtrHashKey<PannerNode>> mPannerNodes;
+  nsDataHashtable<nsStringHashKey, AudioParamDescriptorMap>
+      mWorkletParamDescriptors;
   // Cache to avoid recomputing basic waveforms all the time.
   RefPtr<BasicWaveFormCache> mBasicWaveFormCache;
   // Number of channels passed in the OfflineAudioContext ctor.
@@ -347,6 +416,25 @@ class AudioContext final : public DOMEventTargetHelper,
   // Suspend has been called with no following resume.
   bool mSuspendCalled;
   bool mIsDisconnecting;
+  // This flag stores the value of previous status of `allowed-to-start`.
+  bool mWasAllowedToStart;
+
+  // True if this AudioContext has been suspended by the page.
+  bool mSuspendedByContent;
+
+  // These variables are used for telemetry, they're not reflect the actual
+  // status of AudioContext, they are based on the "assumption" of enabling
+  // blocking web audio. Because we want to record Telemetry no matter user
+  // enable blocking autoplay or not.
+  // - 'mWasEverAllowedToStart' would be true when AudioContext had ever been
+  //   allowed to start if we enable blocking web audio.
+  // - 'mWasEverBlockedToStart' would be true when AudioContext had ever been
+  //   blocked to start if we enable blocking web audio.
+  // - 'mWouldBeAllowedToStart' stores the value of previous status of
+  //   `allowed-to-start` if we enable blocking web audio.
+  bool mWasEverAllowedToStart;
+  bool mWasEverBlockedToStart;
+  bool mWouldBeAllowedToStart;
 };
 
 static const dom::AudioContext::AudioContextId NO_AUDIO_CONTEXT = 0;
@@ -354,4 +442,8 @@ static const dom::AudioContext::AudioContextId NO_AUDIO_CONTEXT = 0;
 }  // namespace dom
 }  // namespace mozilla
 
+inline nsISupports* ToSupports(mozilla::dom::AudioContext* p) {
+  return NS_CYCLE_COLLECTION_CLASSNAME(mozilla::dom::AudioContext)::Upcast(p);
+}
+
 #endif
