@@ -7,7 +7,6 @@
 
 #include "prlink.h"
 #include "plstr.h"
-#include "nsIPluginInstanceOwner.h"
 #include "nsPluginsDir.h"
 #include "nsPluginHost.h"
 #include "nsIBlocklistService.h"
@@ -19,31 +18,49 @@
 #include "nsNetUtil.h"
 #include <cctype>
 #include "mozilla/Encoding.h"
-#include "mozilla/dom/ContentChild.h"
 #include "mozilla/dom/FakePluginTagInitBinding.h"
+#include "mozilla/StaticPrefs_plugin.h"
+
+#if defined(XP_MACOSX) && defined(MOZ_SANDBOX)
+#  include "mozilla/SandboxSettings.h"
+#  include "nsCocoaFeatures.h"
+#endif
 
 using mozilla::dom::FakePluginTagInit;
 using namespace mozilla;
 
 // These legacy flags are used in the plugin registry. The states are now
 // stored in prefs, but we still need to be able to import them.
-#define NS_PLUGIN_FLAG_ENABLED      0x0001    // is this plugin enabled?
-// no longer used                   0x0002    // reuse only if regenerating pluginreg.dat
-#define NS_PLUGIN_FLAG_FROMCACHE    0x0004    // this plugintag info was loaded from cache
-// no longer used                   0x0008    // reuse only if regenerating pluginreg.dat
-#define NS_PLUGIN_FLAG_CLICKTOPLAY  0x0020    // this is a click-to-play plugin
+#define NS_PLUGIN_FLAG_ENABLED 0x0001  // is this plugin enabled?
+// no longer used                   0x0002    // reuse only if regenerating
+// pluginreg.dat
+#define NS_PLUGIN_FLAG_FROMCACHE \
+  0x0004  // this plugintag info was loaded from cache
+// no longer used                   0x0008    // reuse only if regenerating
+// pluginreg.dat
+#define NS_PLUGIN_FLAG_CLICKTOPLAY 0x0020  // this is a click-to-play plugin
 
 static const char kPrefDefaultEnabledState[] = "plugin.default.state";
-static const char kPrefDefaultEnabledStateXpi[] = "plugin.defaultXpi.state";
+
+// The defaults here will be read from prefs and overwritten
+#if defined(MOZ_SANDBOX)
+#  if defined(XP_WIN) || defined(XP_MACOSX)
+static int32_t sFlashSandboxLevel = 3;
+static int32_t sDefaultSandboxLevel = 0;
+#  endif
+#  if defined(XP_MACOSX)
+static bool sEnableSandboxLogging = false;
+#  endif
+#endif /* MOZ_SANDBOX */
+static bool sInitializedSandboxingInfo = false;
 
 // check comma delimited extensions
 static bool ExtensionInList(const nsCString& aExtensionList,
-                            const nsACString& aExtension)
-{
+                            const nsACString& aExtension) {
   nsCCharSeparatedTokenizer extensions(aExtensionList, ',');
   while (extensions.hasMoreTokens()) {
     const nsACString& extension = extensions.nextToken();
-    if (extension.Equals(aExtension, nsCaseInsensitiveCStringComparator())) {
+    if (extension.Equals(aExtension, nsCaseInsensitiveCStringComparator)) {
       return true;
     }
   }
@@ -52,11 +69,10 @@ static bool ExtensionInList(const nsCString& aExtensionList,
 
 // Search for an extension in an extensions array, and return its
 // matching mime type
-static bool SearchExtensions(const nsTArray<nsCString> & aExtensions,
-                             const nsTArray<nsCString> & aMimeTypes,
-                             const nsACString & aFindExtension,
-                             nsACString & aMatchingType)
-{
+static bool SearchExtensions(const nsTArray<nsCString>& aExtensions,
+                             const nsTArray<nsCString>& aMimeTypes,
+                             const nsACString& aFindExtension,
+                             nsACString& aMatchingType) {
   uint32_t mimes = aMimeTypes.Length();
   MOZ_ASSERT(mimes == aExtensions.Length(),
              "These arrays should have matching elements");
@@ -73,9 +89,7 @@ static bool SearchExtensions(const nsTArray<nsCString> & aExtensions,
   return false;
 }
 
-static nsCString
-MakeNiceFileName(const nsCString & aFileName)
-{
+static nsCString MakeNiceFileName(const nsCString& aFileName) {
   nsCString niceName = aFileName;
   int32_t niceNameLength = aFileName.RFind(".");
   NS_ASSERTION(niceNameLength != kNotFound, "aFileName doesn't have a '.'?");
@@ -97,9 +111,8 @@ MakeNiceFileName(const nsCString & aFileName)
   return niceName;
 }
 
-static nsCString
-MakePrefNameForPlugin(const char* const subname, nsIInternalPluginTag* aTag)
-{
+static nsCString MakePrefNameForPlugin(const char* const subname,
+                                       nsIInternalPluginTag* aTag) {
   nsCString pref;
   nsAutoCString pluginName(aTag->GetNiceFileName());
 
@@ -120,39 +133,12 @@ MakePrefNameForPlugin(const char* const subname, nsIInternalPluginTag* aTag)
   return pref;
 }
 
-static nsresult
-CStringArrayToXPCArray(nsTArray<nsCString> & aArray,
-                       uint32_t* aCount,
-                       char16_t*** aResults)
-{
-  uint32_t count = aArray.Length();
-  if (!count) {
-    *aResults = nullptr;
-    *aCount = 0;
-    return NS_OK;
-  }
-
-  *aResults =
-    static_cast<char16_t**>(moz_xmalloc(count * sizeof(**aResults)));
-  *aCount = count;
-
-  for (uint32_t i = 0; i < count; i++) {
-    (*aResults)[i] = ToNewUnicode(NS_ConvertUTF8toUTF16(aArray[i]));
-  }
-
-  return NS_OK;
-}
-
-static nsCString
-GetStatePrefNameForPlugin(nsIInternalPluginTag* aTag)
-{
+static nsCString GetStatePrefNameForPlugin(nsIInternalPluginTag* aTag) {
   return MakePrefNameForPlugin("state", aTag);
 }
 
-static nsresult
-IsEnabledStateLockedForPlugin(nsIInternalPluginTag* aTag,
-                              bool* aIsEnabledStateLocked)
-{
+static nsresult IsEnabledStateLockedForPlugin(nsIInternalPluginTag* aTag,
+                                              bool* aIsEnabledStateLocked) {
   *aIsEnabledStateLocked = false;
   nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
 
@@ -167,166 +153,127 @@ IsEnabledStateLockedForPlugin(nsIInternalPluginTag* aTag,
 }
 
 /* nsIInternalPluginTag */
-nsIInternalPluginTag::nsIInternalPluginTag()
-{
-}
+
+uint32_t nsIInternalPluginTag::sNextId;
+
+nsIInternalPluginTag::nsIInternalPluginTag() = default;
 
 nsIInternalPluginTag::nsIInternalPluginTag(const char* aName,
                                            const char* aDescription,
                                            const char* aFileName,
                                            const char* aVersion)
-  : mName(aName)
-  , mDescription(aDescription)
-  , mFileName(aFileName)
-  , mVersion(aVersion)
-{
-}
+    : mName(aName),
+      mDescription(aDescription),
+      mFileName(aFileName),
+      mVersion(aVersion) {}
 
-nsIInternalPluginTag::nsIInternalPluginTag(const char* aName,
-                                           const char* aDescription,
-                                           const char* aFileName,
-                                           const char* aVersion,
-                                           const nsTArray<nsCString>& aMimeTypes,
-                                           const nsTArray<nsCString>& aMimeDescriptions,
-                                           const nsTArray<nsCString>& aExtensions)
-  : mName(aName)
-  , mDescription(aDescription)
-  , mFileName(aFileName)
-  , mVersion(aVersion)
-  , mMimeTypes(aMimeTypes)
-  , mMimeDescriptions(aMimeDescriptions)
-  , mExtensions(aExtensions)
-{
-}
+nsIInternalPluginTag::nsIInternalPluginTag(
+    const char* aName, const char* aDescription, const char* aFileName,
+    const char* aVersion, const nsTArray<nsCString>& aMimeTypes,
+    const nsTArray<nsCString>& aMimeDescriptions,
+    const nsTArray<nsCString>& aExtensions)
+    : mName(aName),
+      mDescription(aDescription),
+      mFileName(aFileName),
+      mVersion(aVersion),
+      mMimeTypes(aMimeTypes.Clone()),
+      mMimeDescriptions(aMimeDescriptions.Clone()),
+      mExtensions(aExtensions.Clone()) {}
 
-nsIInternalPluginTag::~nsIInternalPluginTag()
-{
-}
+nsIInternalPluginTag::~nsIInternalPluginTag() = default;
 
-bool
-nsIInternalPluginTag::HasExtension(const nsACString& aExtension,
-                                   nsACString& aMatchingType) const
-{
+bool nsIInternalPluginTag::HasExtension(const nsACString& aExtension,
+                                        nsACString& aMatchingType) const {
   return SearchExtensions(mExtensions, mMimeTypes, aExtension, aMatchingType);
 }
 
-bool
-nsIInternalPluginTag::HasMimeType(const nsACString& aMimeType) const
-{
+bool nsIInternalPluginTag::HasMimeType(const nsACString& aMimeType) const {
   return mMimeTypes.Contains(aMimeType,
                              nsCaseInsensitiveCStringArrayComparator());
 }
 
 /* nsPluginTag */
 
-uint32_t nsPluginTag::sNextId;
-
-nsPluginTag::nsPluginTag(nsPluginInfo* aPluginInfo,
-                         int64_t aLastModifiedTime,
-                         bool fromExtension,
+nsPluginTag::nsPluginTag(nsPluginInfo* aPluginInfo, int64_t aLastModifiedTime,
                          uint32_t aBlocklistState)
-  : nsIInternalPluginTag(aPluginInfo->fName, aPluginInfo->fDescription,
-                         aPluginInfo->fFileName, aPluginInfo->fVersion),
-    mId(sNextId++),
-    mContentProcessRunningCount(0),
-    mHadLocalInstance(false),
-    mLibrary(nullptr),
-    mIsFlashPlugin(false),
-    mSupportsAsyncRender(false),
-    mFullPath(aPluginInfo->fFullPath),
-    mLastModifiedTime(aLastModifiedTime),
-    mSandboxLevel(0),
-    mIsSandboxLoggingEnabled(false),
-    mIsFromExtension(fromExtension),
-    mBlocklistState(aBlocklistState)
-{
-  InitMime(aPluginInfo->fMimeTypeArray,
-           aPluginInfo->fMimeDescriptionArray,
-           aPluginInfo->fExtensionArray,
-           aPluginInfo->fVariantCount);
+    : nsIInternalPluginTag(aPluginInfo->fName, aPluginInfo->fDescription,
+                           aPluginInfo->fFileName, aPluginInfo->fVersion),
+      mId(sNextId++),
+      mContentProcessRunningCount(0),
+      mHadLocalInstance(false),
+      mLibrary(nullptr),
+      mIsFlashPlugin(false),
+      mSupportsAsyncRender(false),
+      mFullPath(aPluginInfo->fFullPath),
+      mLastModifiedTime(aLastModifiedTime),
+      mSandboxLevel(0),
+      mIsSandboxLoggingEnabled(false),
+      mBlocklistState(aBlocklistState) {
+  InitMime(aPluginInfo->fMimeTypeArray, aPluginInfo->fMimeDescriptionArray,
+           aPluginInfo->fExtensionArray, aPluginInfo->fVariantCount);
   InitSandboxLevel();
   EnsureMembersAreUTF8();
   FixupVersion();
 }
 
-nsPluginTag::nsPluginTag(const char* aName,
-                         const char* aDescription,
-                         const char* aFileName,
-                         const char* aFullPath,
-                         const char* aVersion,
-                         const char* const* aMimeTypes,
+nsPluginTag::nsPluginTag(const char* aName, const char* aDescription,
+                         const char* aFileName, const char* aFullPath,
+                         const char* aVersion, const char* const* aMimeTypes,
                          const char* const* aMimeDescriptions,
-                         const char* const* aExtensions,
-                         int32_t aVariants,
-                         int64_t aLastModifiedTime,
-                         bool fromExtension,
-                         uint32_t aBlocklistState,
+                         const char* const* aExtensions, int32_t aVariants,
+                         int64_t aLastModifiedTime, uint32_t aBlocklistState,
                          bool aArgsAreUTF8)
-  : nsIInternalPluginTag(aName, aDescription, aFileName, aVersion),
-    mId(sNextId++),
-    mContentProcessRunningCount(0),
-    mHadLocalInstance(false),
-    mLibrary(nullptr),
-    mIsFlashPlugin(false),
-    mSupportsAsyncRender(false),
-    mFullPath(aFullPath),
-    mLastModifiedTime(aLastModifiedTime),
-    mSandboxLevel(0),
-    mIsSandboxLoggingEnabled(false),
-    mIsFromExtension(fromExtension),
-    mBlocklistState(aBlocklistState)
-{
+    : nsIInternalPluginTag(aName, aDescription, aFileName, aVersion),
+      mId(sNextId++),
+      mContentProcessRunningCount(0),
+      mHadLocalInstance(false),
+      mLibrary(nullptr),
+      mIsFlashPlugin(false),
+      mSupportsAsyncRender(false),
+      mFullPath(aFullPath),
+      mLastModifiedTime(aLastModifiedTime),
+      mSandboxLevel(0),
+      mIsSandboxLoggingEnabled(false),
+      mBlocklistState(aBlocklistState) {
   InitMime(aMimeTypes, aMimeDescriptions, aExtensions,
            static_cast<uint32_t>(aVariants));
   InitSandboxLevel();
-  if (!aArgsAreUTF8)
-    EnsureMembersAreUTF8();
+  if (!aArgsAreUTF8) EnsureMembersAreUTF8();
   FixupVersion();
 }
 
-nsPluginTag::nsPluginTag(uint32_t aId,
-                         const char* aName,
-                         const char* aDescription,
-                         const char* aFileName,
-                         const char* aFullPath,
-                         const char* aVersion,
+nsPluginTag::nsPluginTag(uint32_t aId, const char* aName,
+                         const char* aDescription, const char* aFileName,
+                         const char* aFullPath, const char* aVersion,
                          nsTArray<nsCString> aMimeTypes,
                          nsTArray<nsCString> aMimeDescriptions,
-                         nsTArray<nsCString> aExtensions,
-                         bool aIsFlashPlugin,
-                         bool aSupportsAsyncRender,
-                         int64_t aLastModifiedTime,
-                         bool aFromExtension,
-                         int32_t aSandboxLevel,
-                         uint32_t aBlocklistState)
-  : nsIInternalPluginTag(aName, aDescription, aFileName, aVersion, aMimeTypes,
-                         aMimeDescriptions, aExtensions),
-    mId(aId),
-    mContentProcessRunningCount(0),
-    mLibrary(nullptr),
-    mIsFlashPlugin(aIsFlashPlugin),
-    mSupportsAsyncRender(aSupportsAsyncRender),
-    mLastModifiedTime(aLastModifiedTime),
-    mSandboxLevel(aSandboxLevel),
-    mIsSandboxLoggingEnabled(false),
-    mNiceFileName(),
-    mIsFromExtension(aFromExtension),
-    mBlocklistState(aBlocklistState)
-{
-}
+                         nsTArray<nsCString> aExtensions, bool aIsFlashPlugin,
+                         bool aSupportsAsyncRender, int64_t aLastModifiedTime,
+                         int32_t aSandboxLevel, uint32_t aBlocklistState)
+    : nsIInternalPluginTag(aName, aDescription, aFileName, aVersion, aMimeTypes,
+                           aMimeDescriptions, aExtensions),
+      mId(aId),
+      mContentProcessRunningCount(0),
+      mHadLocalInstance(false),
+      mLibrary(nullptr),
+      mIsFlashPlugin(aIsFlashPlugin),
+      mSupportsAsyncRender(aSupportsAsyncRender),
+      mLastModifiedTime(aLastModifiedTime),
+      mSandboxLevel(aSandboxLevel),
+      mIsSandboxLoggingEnabled(false),
+      mNiceFileName(),
+      mBlocklistState(aBlocklistState) {}
 
-nsPluginTag::~nsPluginTag()
-{
+nsPluginTag::~nsPluginTag() {
   NS_ASSERTION(!mNext, "Risk of exhausting the stack space, bug 486349");
 }
 
-NS_IMPL_ISUPPORTS(nsPluginTag, nsPluginTag,  nsIInternalPluginTag, nsIPluginTag)
+NS_IMPL_ISUPPORTS(nsPluginTag, nsPluginTag, nsIInternalPluginTag, nsIPluginTag)
 
 void nsPluginTag::InitMime(const char* const* aMimeTypes,
                            const char* const* aMimeDescriptions,
                            const char* const* aExtensions,
-                           uint32_t aVariantCount)
-{
+                           uint32_t aVariantCount) {
   if (!aMimeTypes) {
     return;
   }
@@ -342,10 +289,6 @@ void nsPluginTag::InitMime(const char* const* aMimeTypes,
     // to properly handle a mixed-case type.
     ToLowerCase(mimeType);
 
-    if (!nsPluginHost::IsTypeWhitelisted(mimeType.get())) {
-      continue;
-    }
-
     // Look for certain special plugins.
     switch (nsPluginHost::GetSpecialType(mimeType)) {
       case nsPluginHost::eSpecialType_Flash:
@@ -372,7 +315,7 @@ void nsPluginTag::InitMime(const char* const* aMimeTypes,
       // so we can search for the opening round bracket
       char cur = '\0';
       char pre = '\0';
-      char * p = PL_strrchr(aMimeDescriptions[i], '(');
+      char* p = PL_strrchr(aMimeDescriptions[i], '(');
       if (p && (p != aMimeDescriptions[i])) {
         if ((p - 1) && *(p - 1) == ' ') {
           pre = *(p - 1);
@@ -403,62 +346,26 @@ void nsPluginTag::InitMime(const char* const* aMimeTypes,
   }
 }
 
-void
-nsPluginTag::InitSandboxLevel()
-{
-#if defined(XP_WIN) && defined(MOZ_SANDBOX)
-  nsAutoCString sandboxPref("dom.ipc.plugins.sandbox-level.");
-  sandboxPref.Append(GetNiceFileName());
-  if (NS_FAILED(Preferences::GetInt(sandboxPref.get(), &mSandboxLevel))) {
-    mSandboxLevel = Preferences::GetInt("dom.ipc.plugins.sandbox-level.default"
-);
-  }
-
-#if defined(_AMD64_)
-  // As level 2 is now the default NPAPI sandbox level for 64-bit flash, we
-  // don't want to allow a lower setting. This should be changed if the
-  // firefox.js pref file is changed.
-  if (mIsFlashPlugin && mSandboxLevel < 2) {
-    mSandboxLevel = 2;
-  }
-#endif
-#endif
-
-#if defined(XP_MACOSX) && defined(MOZ_SANDBOX)
-  // At present, the Mac Flash NPAPI plugin sandbox is controlled via
-  // a boolean with no support for different levels. When the sandbox
-  // is enabled, we set the level to 1.
-  if (mIsFlashPlugin) {
-    // Allow enabling the sandbox via the pref
-    // security.sandbox.mac.flash.enabled or via the environment variable
-    // MOZ_SANDBOX_MAC_FLASH_FORCE (which is useful while the sandbox is
-    // off by default).
-    if (Preferences::GetBool("security.sandbox.mac.flash.enabled") ||
-        PR_GetEnv("MOZ_SANDBOX_MAC_FLASH_FORCE")) {
-      mSandboxLevel = 1;
-
-      // Enable sandbox logging in the plugin process if it has
-      // been turned on via prefs or environment variables.
-      if (Preferences::GetBool("security.sandbox.logging.enabled") ||
-          PR_GetEnv("MOZ_SANDBOX_LOGGING") ||
-          PR_GetEnv("MOZ_SANDBOX_MAC_FLASH_LOGGING")) {
-            mIsSandboxLoggingEnabled = true;
-      }
-    }
-  }
-#endif
+void nsPluginTag::InitSandboxLevel() {
+  MOZ_ASSERT(sInitializedSandboxingInfo,
+             "Should have initialized global sandboxing info");
+#if defined(MOZ_SANDBOX)
+#  if defined(XP_MACOSX)
+  mSandboxLevel = mIsFlashPlugin ? sFlashSandboxLevel : sDefaultSandboxLevel;
+  mIsSandboxLoggingEnabled = mIsFlashPlugin && sEnableSandboxLogging;
+#  elif defined(XP_WIN)
+  mSandboxLevel = mIsFlashPlugin ? sFlashSandboxLevel : sDefaultSandboxLevel;
+#  endif /* defined(XP_MACOSX) / defined(XP_WIN) */
+#endif   /* defined(MOZ_SANDBOX) */
 }
 
 #if !defined(XP_WIN) && !defined(XP_MACOSX)
-static void
-ConvertToUTF8(nsCString& aString)
-{
+static void ConvertToUTF8(nsCString& aString) {
   Unused << UTF_8_ENCODING->DecodeWithoutBOMHandling(aString, aString);
 }
 #endif
 
-nsresult nsPluginTag::EnsureMembersAreUTF8()
-{
+nsresult nsPluginTag::EnsureMembersAreUTF8() {
 #if defined(XP_WIN) || defined(XP_MACOSX)
   return NS_OK;
 #else
@@ -473,8 +380,7 @@ nsresult nsPluginTag::EnsureMembersAreUTF8()
 #endif
 }
 
-void nsPluginTag::FixupVersion()
-{
+void nsPluginTag::FixupVersion() {
 #if defined(XP_LINUX)
   if (mIsFlashPlugin) {
     mVersion.ReplaceChar(',', '.');
@@ -483,117 +389,112 @@ void nsPluginTag::FixupVersion()
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetDescription(nsACString& aDescription)
-{
+nsPluginTag::GetDescription(nsACString& aDescription) {
   aDescription = mDescription;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetFilename(nsACString& aFileName)
-{
+nsPluginTag::GetIsFlashPlugin(bool* aIsFlash) {
+  *aIsFlash = mIsFlashPlugin;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPluginTag::GetFilename(nsACString& aFileName) {
   aFileName = mFileName;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetFullpath(nsACString& aFullPath)
-{
+nsPluginTag::GetFullpath(nsACString& aFullPath) {
   aFullPath = mFullPath;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetVersion(nsACString& aVersion)
-{
+nsPluginTag::GetVersion(nsACString& aVersion) {
   aVersion = mVersion;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetName(nsACString& aName)
-{
+nsPluginTag::GetName(nsACString& aName) {
   aName = mName;
   return NS_OK;
 }
 
-bool
-nsPluginTag::IsActive()
-{
-  return IsEnabled() && !IsBlocklisted();
-}
+bool nsPluginTag::IsActive() { return IsEnabled() && !IsBlocklisted(); }
 
 NS_IMETHODIMP
-nsPluginTag::GetActive(bool *aResult)
-{
+nsPluginTag::GetActive(bool* aResult) {
   *aResult = IsActive();
   return NS_OK;
 }
 
-bool
-nsPluginTag::IsEnabled()
-{
+bool nsPluginTag::IsEnabled() {
   const PluginState state = GetPluginState();
   return (state == ePluginState_Enabled) || (state == ePluginState_Clicktoplay);
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetDisabled(bool* aDisabled)
-{
+nsPluginTag::GetDisabled(bool* aDisabled) {
   *aDisabled = !IsEnabled();
   return NS_OK;
 }
 
-bool
-nsPluginTag::IsBlocklisted()
-{
+bool nsPluginTag::IsBlocklisted() {
   return mBlocklistState == nsIBlocklistService::STATE_BLOCKED;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetBlocklisted(bool* aBlocklisted)
-{
+nsPluginTag::GetBlocklisted(bool* aBlocklisted) {
   *aBlocklisted = IsBlocklisted();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetIsEnabledStateLocked(bool* aIsEnabledStateLocked)
-{
+nsPluginTag::GetIsEnabledStateLocked(bool* aIsEnabledStateLocked) {
   return IsEnabledStateLockedForPlugin(this, aIsEnabledStateLocked);
 }
 
-bool
-nsPluginTag::IsClicktoplay()
-{
+bool nsPluginTag::IsClicktoplay() {
   const PluginState state = GetPluginState();
   return (state == ePluginState_Clicktoplay);
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetClicktoplay(bool *aClicktoplay)
-{
+nsPluginTag::GetClicktoplay(bool* aClicktoplay) {
   *aClicktoplay = IsClicktoplay();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetEnabledState(uint32_t *aEnabledState)
-{
+nsPluginTag::GetEnabledState(uint32_t* aEnabledState) {
   int32_t enabledState;
-  nsresult rv = Preferences::GetInt(GetStatePrefNameForPlugin(this).get(),
-                                    &enabledState);
-  if (NS_SUCCEEDED(rv) &&
-      enabledState >= nsIPluginTag::STATE_DISABLED &&
+  nsresult rv = NS_OK;
+  if (mIsFlashPlugin) {
+    enabledState = StaticPrefs::plugin_state_flash();
+    if (enabledState == nsIPluginTag::STATE_ENABLED) {
+      enabledState = nsIPluginTag::STATE_CLICKTOPLAY;
+    }
+  } else {
+    rv = Preferences::GetInt(GetStatePrefNameForPlugin(this).get(),
+                             &enabledState);
+  }
+  if (NS_SUCCEEDED(rv) && enabledState >= nsIPluginTag::STATE_DISABLED &&
       enabledState <= nsIPluginTag::STATE_ENABLED) {
     *aEnabledState = (uint32_t)enabledState;
     return rv;
   }
 
-  const char* const pref = mIsFromExtension ? kPrefDefaultEnabledStateXpi
-                                            : kPrefDefaultEnabledState;
-
-  enabledState = Preferences::GetInt(pref, nsIPluginTag::STATE_ENABLED);
+  // Something went wrong fetching the plugin's state (e.g. it wasn't flash
+  // and the preference was not present) - use the default state:
+  enabledState = Preferences::GetInt(kPrefDefaultEnabledState,
+                                     nsIPluginTag::STATE_ENABLED);
+  if (enabledState == nsIPluginTag::STATE_ENABLED && mIsFlashPlugin) {
+    enabledState = nsIPluginTag::STATE_CLICKTOPLAY;
+  }
   if (enabledState >= nsIPluginTag::STATE_DISABLED &&
       enabledState <= nsIPluginTag::STATE_ENABLED) {
     *aEnabledState = (uint32_t)enabledState;
@@ -604,10 +505,11 @@ nsPluginTag::GetEnabledState(uint32_t *aEnabledState)
 }
 
 NS_IMETHODIMP
-nsPluginTag::SetEnabledState(uint32_t aEnabledState)
-{
-  if (aEnabledState >= ePluginState_MaxValue)
-    return NS_ERROR_ILLEGAL_VALUE;
+nsPluginTag::SetEnabledState(uint32_t aEnabledState) {
+  if (aEnabledState >= ePluginState_MaxValue) return NS_ERROR_ILLEGAL_VALUE;
+  if (aEnabledState == nsIPluginTag::STATE_ENABLED && mIsFlashPlugin) {
+    aEnabledState = nsIPluginTag::STATE_CLICKTOPLAY;
+  }
   uint32_t oldState = nsIPluginTag::STATE_DISABLED;
   GetEnabledState(&oldState);
   if (oldState != aEnabledState) {
@@ -619,44 +521,47 @@ nsPluginTag::SetEnabledState(uint32_t aEnabledState)
   return NS_OK;
 }
 
-nsPluginTag::PluginState
-nsPluginTag::GetPluginState()
-{
+nsPluginTag::PluginState nsPluginTag::GetPluginState() {
   uint32_t enabledState = nsIPluginTag::STATE_DISABLED;
   GetEnabledState(&enabledState);
   return (PluginState)enabledState;
 }
 
-void
-nsPluginTag::SetPluginState(PluginState state)
-{
-  static_assert((uint32_t)nsPluginTag::ePluginState_Disabled == nsIPluginTag::STATE_DISABLED, "nsPluginTag::ePluginState_Disabled must match nsIPluginTag::STATE_DISABLED");
-  static_assert((uint32_t)nsPluginTag::ePluginState_Clicktoplay == nsIPluginTag::STATE_CLICKTOPLAY, "nsPluginTag::ePluginState_Clicktoplay must match nsIPluginTag::STATE_CLICKTOPLAY");
-  static_assert((uint32_t)nsPluginTag::ePluginState_Enabled == nsIPluginTag::STATE_ENABLED, "nsPluginTag::ePluginState_Enabled must match nsIPluginTag::STATE_ENABLED");
+void nsPluginTag::SetPluginState(PluginState state) {
+  static_assert((uint32_t)nsPluginTag::ePluginState_Disabled ==
+                    nsIPluginTag::STATE_DISABLED,
+                "nsPluginTag::ePluginState_Disabled must match "
+                "nsIPluginTag::STATE_DISABLED");
+  static_assert((uint32_t)nsPluginTag::ePluginState_Clicktoplay ==
+                    nsIPluginTag::STATE_CLICKTOPLAY,
+                "nsPluginTag::ePluginState_Clicktoplay must match "
+                "nsIPluginTag::STATE_CLICKTOPLAY");
+  static_assert((uint32_t)nsPluginTag::ePluginState_Enabled ==
+                    nsIPluginTag::STATE_ENABLED,
+                "nsPluginTag::ePluginState_Enabled must match "
+                "nsIPluginTag::STATE_ENABLED");
   SetEnabledState((uint32_t)state);
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetMimeTypes(uint32_t* aCount, char16_t*** aResults)
-{
-  return CStringArrayToXPCArray(mMimeTypes, aCount, aResults);
+nsPluginTag::GetMimeTypes(nsTArray<nsCString>& aResults) {
+  aResults = mMimeTypes.Clone();
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetMimeDescriptions(uint32_t* aCount, char16_t*** aResults)
-{
-  return CStringArrayToXPCArray(mMimeDescriptions, aCount, aResults);
+nsPluginTag::GetMimeDescriptions(nsTArray<nsCString>& aResults) {
+  aResults = mMimeDescriptions.Clone();
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetExtensions(uint32_t* aCount, char16_t*** aResults)
-{
-  return CStringArrayToXPCArray(mExtensions, aCount, aResults);
+nsPluginTag::GetExtensions(nsTArray<nsCString>& aResults) {
+  aResults = mExtensions.Clone();
+  return NS_OK;
 }
 
-bool
-nsPluginTag::HasSameNameAndMimes(const nsPluginTag *aPluginTag) const
-{
+bool nsPluginTag::HasSameNameAndMimes(const nsPluginTag* aPluginTag) const {
   NS_ENSURE_TRUE(aPluginTag, false);
 
   if ((!mName.Equals(aPluginTag->mName)) ||
@@ -674,14 +579,12 @@ nsPluginTag::HasSameNameAndMimes(const nsPluginTag *aPluginTag) const
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetLoaded(bool* aIsLoaded)
-{
+nsPluginTag::GetLoaded(bool* aIsLoaded) {
   *aIsLoaded = !!mPlugin;
   return NS_OK;
 }
 
-void nsPluginTag::TryUnloadPlugin(bool inShutdown)
-{
+void nsPluginTag::TryUnloadPlugin(bool inShutdown) {
   // We never want to send NPP_Shutdown to an in-process plugin unless
   // this process is shutting down.
   if (!mPlugin) {
@@ -693,9 +596,59 @@ void nsPluginTag::TryUnloadPlugin(bool inShutdown)
   }
 }
 
-const nsCString&
-nsPluginTag::GetNiceFileName()
-{
+/* static */ void nsPluginTag::EnsureSandboxInformation() {
+  if (sInitializedSandboxingInfo) {
+    return;
+  }
+  MOZ_ASSERT(NS_IsMainThread(), "Should be on the main thread.");
+#if defined(XP_WIN) && defined(MOZ_SANDBOX)
+  Preferences::GetInt("dom.ipc.plugins.sandbox-level.default",
+                      &sDefaultSandboxLevel);
+  Preferences::GetInt("dom.ipc.plugins.sandbox-level.flash",
+                      &sFlashSandboxLevel);
+#  if defined(_AMD64_)
+  // Level 3 is now the default NPAPI sandbox level for 64-bit flash.
+  // We permit the user to drop the sandbox level by at most 1.  This should
+  // be kept up to date with the default value in the firefox.js pref file.
+  sFlashSandboxLevel = std::max(sFlashSandboxLevel, 2);
+#  endif
+#elif defined(XP_MACOSX) && defined(MOZ_SANDBOX)
+  int legacyOSMinorMax = Preferences::GetInt(
+      "dom.ipc.plugins.sandbox-level.flash.max-legacy-os-minor", 10);
+  const char* levelPref = "dom.ipc.plugins.sandbox-level.flash";
+
+  if (PR_GetEnv("MOZ_DISABLE_NPAPI_SANDBOX")) {
+    // Flash sandbox disabled
+    sFlashSandboxLevel = 0;
+  } else if (nsCocoaFeatures::macOSVersionMajor() == 10 &&
+             nsCocoaFeatures::macOSVersionMinor() <= legacyOSMinorMax) {
+    const char* legacyLevelPref = "dom.ipc.plugins.sandbox-level.flash.legacy";
+    int32_t compatLevel = Preferences::GetInt(legacyLevelPref, 0);
+    int32_t level = Preferences::GetInt(levelPref, 0);
+    sFlashSandboxLevel = std::min(compatLevel, level);
+  } else {
+    sFlashSandboxLevel = Preferences::GetInt(levelPref, 0);
+  }
+  sFlashSandboxLevel = ClampFlashSandboxLevel(sFlashSandboxLevel);
+
+  // At present, Flash is the only supported plugin on macOS.
+  // Other test plugins are used during testing and they will use
+  // the default plugin sandbox level.
+  Preferences::GetInt("dom.ipc.plugins.sandbox-level.default",
+                      &sDefaultSandboxLevel);
+
+  // Enable sandbox logging in the plugin process if it has
+  // been turned on via prefs or environment variables.
+  sEnableSandboxLogging =
+      sFlashSandboxLevel > 0 &&
+      (Preferences::GetBool("security.sandbox.logging.enabled") ||
+       PR_GetEnv("MOZ_SANDBOX_LOGGING") ||
+       PR_GetEnv("MOZ_SANDBOX_MAC_FLASH_LOGGING"));
+#endif
+  sInitializedSandboxingInfo = true;
+}
+
+const nsCString& nsPluginTag::GetNiceFileName() {
   if (!mNiceFileName.IsEmpty()) {
     return mNiceFileName;
   }
@@ -710,75 +663,58 @@ nsPluginTag::GetNiceFileName()
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetNiceName(nsACString & aResult)
-{
+nsPluginTag::GetNiceName(nsACString& aResult) {
   aResult = GetNiceFileName();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPluginTag::GetBlocklistState(uint32_t *aResult)
-{
+nsPluginTag::GetBlocklistState(uint32_t* aResult) {
   *aResult = mBlocklistState;
   return NS_OK;
 }
 
-void
-nsPluginTag::SetBlocklistState(uint32_t aBlocklistState)
-{
+void nsPluginTag::SetBlocklistState(uint32_t aBlocklistState) {
   mBlocklistState = aBlocklistState;
 }
 
-uint32_t
-nsPluginTag::BlocklistState()
-{
-  return mBlocklistState;
-}
+uint32_t nsPluginTag::BlocklistState() { return mBlocklistState; }
 
 NS_IMETHODIMP
-nsPluginTag::GetLastModifiedTime(PRTime* aLastModifiedTime)
-{
+nsPluginTag::GetLastModifiedTime(PRTime* aLastModifiedTime) {
   MOZ_ASSERT(aLastModifiedTime);
   *aLastModifiedTime = mLastModifiedTime;
   return NS_OK;
 }
 
-bool nsPluginTag::IsFromExtension() const
-{
-  return mIsFromExtension;
+NS_IMETHODIMP
+nsPluginTag::GetId(uint32_t* aId) {
+  *aId = mId;
+  return NS_OK;
 }
 
 /* nsFakePluginTag */
 
-uint32_t nsFakePluginTag::sNextId;
-
 nsFakePluginTag::nsFakePluginTag()
-  : mId(sNextId++),
-    mState(nsPluginTag::ePluginState_Disabled)
-{
-}
+    : mId(sNextId++), mState(nsPluginTag::ePluginState_Disabled) {}
 
 nsFakePluginTag::nsFakePluginTag(uint32_t aId,
                                  already_AddRefed<nsIURI>&& aHandlerURI,
-                                 const char* aName,
-                                 const char* aDescription,
+                                 const char* aName, const char* aDescription,
                                  const nsTArray<nsCString>& aMimeTypes,
                                  const nsTArray<nsCString>& aMimeDescriptions,
                                  const nsTArray<nsCString>& aExtensions,
                                  const nsCString& aNiceName,
                                  const nsString& aSandboxScript)
-  : nsIInternalPluginTag(aName, aDescription, nullptr, nullptr,
-                         aMimeTypes, aMimeDescriptions, aExtensions),
-    mId(aId),
-    mHandlerURI(aHandlerURI),
-    mNiceName(aNiceName),
-    mSandboxScript(aSandboxScript),
-    mState(nsPluginTag::ePluginState_Enabled)
-{}
+    : nsIInternalPluginTag(aName, aDescription, nullptr, nullptr, aMimeTypes,
+                           aMimeDescriptions, aExtensions),
+      mId(aId),
+      mHandlerURI(aHandlerURI),
+      mNiceName(aNiceName),
+      mSandboxScript(aSandboxScript),
+      mState(nsPluginTag::ePluginState_Enabled) {}
 
-nsFakePluginTag::~nsFakePluginTag()
-{
-}
+nsFakePluginTag::~nsFakePluginTag() = default;
 
 NS_IMPL_ADDREF(nsFakePluginTag)
 NS_IMPL_RELEASE(nsFakePluginTag)
@@ -794,16 +730,14 @@ NS_INTERFACE_TABLE_HEAD(nsFakePluginTag)
 NS_INTERFACE_TABLE_TAIL
 
 /* static */
-nsresult
-nsFakePluginTag::Create(const FakePluginTagInit& aInitDictionary,
-                        nsFakePluginTag** aPluginTag)
-{
+nsresult nsFakePluginTag::Create(const FakePluginTagInit& aInitDictionary,
+                                 nsFakePluginTag** aPluginTag) {
   NS_ENSURE_TRUE(sNextId <= PR_INT32_MAX, NS_ERROR_OUT_OF_MEMORY);
   NS_ENSURE_TRUE(!aInitDictionary.mMimeEntries.IsEmpty(), NS_ERROR_INVALID_ARG);
 
   RefPtr<nsFakePluginTag> tag = new nsFakePluginTag();
-  nsresult rv = NS_NewURI(getter_AddRefs(tag->mHandlerURI),
-                          aInitDictionary.mHandlerURI);
+  nsresult rv =
+      NS_NewURI(getter_AddRefs(tag->mHandlerURI), aInitDictionary.mHandlerURI);
   NS_ENSURE_SUCCESS(rv, rv);
 
   CopyUTF16toUTF8(aInitDictionary.mNiceName, tag->mNiceName);
@@ -825,65 +759,60 @@ nsFakePluginTag::Create(const FakePluginTagInit& aInitDictionary,
   return NS_OK;
 }
 
-bool
-nsFakePluginTag::HandlerURIMatches(nsIURI* aURI)
-{
+bool nsFakePluginTag::HandlerURIMatches(nsIURI* aURI) {
   bool equals = false;
   return NS_SUCCEEDED(mHandlerURI->Equals(aURI, &equals)) && equals;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetHandlerURI(nsIURI **aResult)
-{
+nsFakePluginTag::GetHandlerURI(nsIURI** aResult) {
   NS_IF_ADDREF(*aResult = mHandlerURI);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetSandboxScript(nsAString& aSandboxScript)
-{
+nsFakePluginTag::GetSandboxScript(nsAString& aSandboxScript) {
   aSandboxScript = mSandboxScript;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetDescription(/* utf-8 */ nsACString& aResult)
-{
+nsFakePluginTag::GetDescription(/* utf-8 */ nsACString& aResult) {
   aResult = mDescription;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetFilename(/* utf-8 */ nsACString& aResult)
-{
+nsFakePluginTag::GetIsFlashPlugin(bool* aIsFlash) {
+  *aIsFlash = false;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFakePluginTag::GetFilename(/* utf-8 */ nsACString& aResult) {
   aResult = mFileName;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetFullpath(/* utf-8 */ nsACString& aResult)
-{
+nsFakePluginTag::GetFullpath(/* utf-8 */ nsACString& aResult) {
   aResult = mFullPath;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetVersion(/* utf-8 */ nsACString& aResult)
-{
+nsFakePluginTag::GetVersion(/* utf-8 */ nsACString& aResult) {
   aResult = mVersion;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetName(/* utf-8 */ nsACString& aResult)
-{
+nsFakePluginTag::GetName(/* utf-8 */ nsACString& aResult) {
   aResult = mName;
   return NS_OK;
 }
 
-const nsCString&
-nsFakePluginTag::GetNiceFileName()
-{
+const nsCString& nsFakePluginTag::GetNiceFileName() {
   // We don't try to mimic the special-cased flash/java names if the fake plugin
   // claims one of their MIME types, but do allow directly setting niceName if
   // emulating those is desired.
@@ -895,65 +824,55 @@ nsFakePluginTag::GetNiceFileName()
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetNiceName(/* utf-8 */ nsACString& aResult)
-{
+nsFakePluginTag::GetNiceName(/* utf-8 */ nsACString& aResult) {
   aResult = GetNiceFileName();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetBlocklistState(uint32_t* aResult)
-{
+nsFakePluginTag::GetBlocklistState(uint32_t* aResult) {
   // Fake tags don't currently support blocklisting
   *aResult = nsIBlocklistService::STATE_NOT_BLOCKED;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetBlocklisted(bool* aBlocklisted)
-{
+nsFakePluginTag::GetBlocklisted(bool* aBlocklisted) {
   // Fake tags can't be blocklisted
   *aBlocklisted = false;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetIsEnabledStateLocked(bool* aIsEnabledStateLocked)
-{
+nsFakePluginTag::GetIsEnabledStateLocked(bool* aIsEnabledStateLocked) {
   return IsEnabledStateLockedForPlugin(this, aIsEnabledStateLocked);
 }
 
-bool
-nsFakePluginTag::IsEnabled()
-{
+bool nsFakePluginTag::IsEnabled() {
   return mState == nsPluginTag::ePluginState_Enabled ||
          mState == nsPluginTag::ePluginState_Clicktoplay;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetDisabled(bool* aDisabled)
-{
+nsFakePluginTag::GetDisabled(bool* aDisabled) {
   *aDisabled = !IsEnabled();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetClicktoplay(bool* aClicktoplay)
-{
+nsFakePluginTag::GetClicktoplay(bool* aClicktoplay) {
   *aClicktoplay = (mState == nsPluginTag::ePluginState_Clicktoplay);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetEnabledState(uint32_t* aEnabledState)
-{
+nsFakePluginTag::GetEnabledState(uint32_t* aEnabledState) {
   *aEnabledState = (uint32_t)mState;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::SetEnabledState(uint32_t aEnabledState)
-{
+nsFakePluginTag::SetEnabledState(uint32_t aEnabledState) {
   // There are static asserts above enforcing that this enum matches
   mState = (nsPluginTag::PluginState)aEnabledState;
   // FIXME-jsplugins update
@@ -961,34 +880,32 @@ nsFakePluginTag::SetEnabledState(uint32_t aEnabledState)
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetMimeTypes(uint32_t* aCount, char16_t*** aResults)
-{
-  return CStringArrayToXPCArray(mMimeTypes, aCount, aResults);
+nsFakePluginTag::GetMimeTypes(nsTArray<nsCString>& aResults) {
+  aResults = mMimeTypes.Clone();
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetMimeDescriptions(uint32_t* aCount, char16_t*** aResults)
-{
-  return CStringArrayToXPCArray(mMimeDescriptions, aCount, aResults);
+nsFakePluginTag::GetMimeDescriptions(nsTArray<nsCString>& aResults) {
+  aResults = mMimeDescriptions.Clone();
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetExtensions(uint32_t* aCount, char16_t*** aResults)
-{
-  return CStringArrayToXPCArray(mExtensions, aCount, aResults);
+nsFakePluginTag::GetExtensions(nsTArray<nsCString>& aResults) {
+  aResults = mExtensions.Clone();
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetActive(bool *aResult)
-{
+nsFakePluginTag::GetActive(bool* aResult) {
   // Fake plugins can't be blocklisted, so this is just !Disabled
   *aResult = IsEnabled();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetLastModifiedTime(PRTime* aLastModifiedTime)
-{
+nsFakePluginTag::GetLastModifiedTime(PRTime* aLastModifiedTime) {
   // FIXME-jsplugins What should this return, if anything?
   MOZ_ASSERT(aLastModifiedTime);
   *aLastModifiedTime = 0;
@@ -997,15 +914,13 @@ nsFakePluginTag::GetLastModifiedTime(PRTime* aLastModifiedTime)
 
 // We don't load fake plugins out of a library, so they should always be there.
 NS_IMETHODIMP
-nsFakePluginTag::GetLoaded(bool* ret)
-{
+nsFakePluginTag::GetLoaded(bool* ret) {
   *ret = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFakePluginTag::GetId(uint32_t* aId)
-{
+nsFakePluginTag::GetId(uint32_t* aId) {
   *aId = mId;
   return NS_OK;
 }
