@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: sw=4 ts=4 et :
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+ * vim: sw=2 ts=4 et :
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -10,13 +10,13 @@
 #include "mozilla/BasicEvents.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Telemetry.h"
+#include "mozilla/dom/Element.h"
 #include "PluginInstanceParent.h"
 #include "BrowserStreamParent.h"
 #include "PluginBackgroundDestroyer.h"
 #include "PluginModuleParent.h"
 #include "StreamNotifyParent.h"
 #include "npfunctions.h"
-#include "nsAutoPtr.h"
 #include "gfxASurface.h"
 #include "gfxContext.h"
 #include "gfxPlatform.h"
@@ -25,9 +25,8 @@
 #include "nsNPAPIPluginInstance.h"
 #include "nsPluginInstanceOwner.h"
 #include "nsFocusManager.h"
-#include "nsIDOMElement.h"
 #ifdef MOZ_X11
-#include "gfxXlibSurface.h"
+#  include "gfxXlibSurface.h"
 #endif
 #include "gfxUtils.h"
 #include "mozilla/gfx/2D.h"
@@ -35,36 +34,38 @@
 #include "ImageContainer.h"
 #include "GLContext.h"
 #include "GLContextProvider.h"
-#include "gfxPrefs.h"
 #include "LayersLogging.h"
 #include "mozilla/layers/TextureWrapperImage.h"
 #include "mozilla/layers/TextureClientRecycleAllocator.h"
 #include "mozilla/layers/ImageBridgeChild.h"
 #if defined(XP_WIN)
-#include "mozilla/layers/D3D11ShareHandleImage.h"
-#include "mozilla/gfx/DeviceManagerDx.h"
-#include "mozilla/layers/TextureD3D11.h"
+#  include "mozilla/layers/D3D11ShareHandleImage.h"
+#  include "mozilla/gfx/DeviceManagerDx.h"
+#  include "mozilla/layers/TextureD3D11.h"
 #endif
 
 #ifdef XP_MACOSX
-#include "MacIOSurfaceImage.h"
+#  include "MacIOSurfaceImage.h"
 #endif
 
 #if defined(OS_WIN)
-#include <windowsx.h>
-#include "gfxWindowsPlatform.h"
-#include "mozilla/plugins/PluginSurfaceParent.h"
-#include "nsClassHashtable.h"
-#include "nsHashKeys.h"
-#include "nsIWidget.h"
-#include "nsPluginNativeWindow.h"
-#include "PluginQuirks.h"
+#  include <windowsx.h>
+#  include "gfxWindowsPlatform.h"
+#  include "mozilla/plugins/PluginSurfaceParent.h"
+#  include "nsClassHashtable.h"
+#  include "nsHashKeys.h"
+#  include "nsIWidget.h"
+#  include "nsPluginNativeWindow.h"
+#  include "PluginQuirks.h"
+#  include "mozilla/layers/CompositorBridgeChild.h"
+#  include "GPUVideoImage.h"
+#  include "mozilla/layers/SynchronousTask.h"
 extern const wchar_t* kFlashFullscreenClass;
 #elif defined(MOZ_WIDGET_GTK)
-#include "mozilla/dom/ContentChild.h"
-#include <gdk/gdk.h>
+#  include "mozilla/dom/ContentChild.h"
+#  include <gdk/gdk.h>
 #elif defined(XP_MACOSX)
-#include <ApplicationServices/ApplicationServices.h>
+#  include <ApplicationServices/ApplicationServices.h>
 #endif  // defined(XP_MACOSX)
 
 using namespace mozilla::plugins;
@@ -122,8 +123,7 @@ PluginInstanceParent::PluginInstanceParent(PluginModuleParent* parent, NPP npp,
       mPluginHWND(nullptr),
       mChildPluginHWND(nullptr),
       mChildPluginsParentHWND(nullptr),
-      mPluginWndProc(nullptr),
-      mNestedEventState(false)
+      mPluginWndProc(nullptr)
 #endif  // defined(XP_WIN)
 #if defined(XP_MACOSX)
       ,
@@ -165,9 +165,8 @@ bool PluginInstanceParent::InitMetadata(const nsACString& aMimeType,
   if (!owner) {
     return false;
   }
-  nsCOMPtr<nsIURI> baseUri(owner->GetBaseURI());
   return NS_SUCCEEDED(
-      NS_MakeAbsoluteURI(mSrcAttribute, aSrcAttribute, baseUri));
+      NS_MakeAbsoluteURI(mSrcAttribute, aSrcAttribute, owner->GetBaseURI()));
 }
 
 void PluginInstanceParent::ActorDestroy(ActorDestroyReason why) {
@@ -235,7 +234,7 @@ PluginInstanceParent::AnswerNPN_GetValue_NPNVnetscapeWindow(
   // TODO: Need Android impl
   int id;
 #else
-#warning Implement me
+#  warning Implement me
 #endif
 
   *result = mNPNIface->getvalue(mNPP, NPNVnetscapeWindow, &id);
@@ -315,23 +314,12 @@ PluginInstanceParent::AnswerNPN_GetValue_NPNVdocumentOrigin(nsCString* value,
 }
 
 static inline bool AllowDirectBitmapSurfaceDrawing() {
-  if (!gfxPrefs::PluginAsyncDrawingEnabled()) {
+  if (!StaticPrefs::dom_ipc_plugins_asyncdrawing_enabled()) {
     return false;
   }
   return gfxPlatform::GetPlatform()->SupportsPluginDirectBitmapDrawing();
 }
 
-static inline bool AllowDirectDXGISurfaceDrawing() {
-  if (!gfxPrefs::PluginAsyncDrawingEnabled()) {
-    return false;
-  }
-#if defined(XP_WIN)
-  return gfxWindowsPlatform::GetPlatform()->SupportsPluginDirectDXGIDrawing();
-#else
-  return false;
-#endif
-}
-
 mozilla::ipc::IPCResult
 PluginInstanceParent::AnswerNPN_GetValue_SupportsAsyncBitmapSurface(
     bool* value) {
@@ -339,43 +327,29 @@ PluginInstanceParent::AnswerNPN_GetValue_SupportsAsyncBitmapSurface(
   return IPC_OK();
 }
 
-mozilla::ipc::IPCResult
-PluginInstanceParent::AnswerNPN_GetValue_SupportsAsyncDXGISurface(bool* value) {
-  *value = AllowDirectDXGISurfaceDrawing();
-  return IPC_OK();
+/* static */
+bool PluginInstanceParent::SupportsPluginDirectDXGISurfaceDrawing() {
+  bool value = false;
+#if defined(XP_WIN)
+  if (StaticPrefs::dom_ipc_plugins_allow_dxgi_surface()) {
+    auto cbc = CompositorBridgeChild::Get();
+    if (cbc) {
+      cbc->SendSupportsAsyncDXGISurface(&value);
+    }
+  }
+#endif
+  return value;
 }
 
 mozilla::ipc::IPCResult
 PluginInstanceParent::AnswerNPN_GetValue_PreferredDXGIAdapter(
     DxgiAdapterDesc* aOutDesc) {
   PodZero(aOutDesc);
-#ifdef XP_WIN
-  if (!AllowDirectDXGISurfaceDrawing()) {
-    return IPC_FAIL_NO_REASON(this);
+#if defined(XP_WIN)
+  auto cbc = CompositorBridgeChild::Get();
+  if (cbc) {
+    cbc->SendPreferredDXGIAdapter(aOutDesc);
   }
-
-  RefPtr<ID3D11Device> device = DeviceManagerDx::Get()->GetContentDevice();
-  if (!device) {
-    return IPC_FAIL_NO_REASON(this);
-  }
-
-  RefPtr<IDXGIDevice> dxgi;
-  if (FAILED(device->QueryInterface(__uuidof(IDXGIDevice),
-                                    getter_AddRefs(dxgi))) ||
-      !dxgi) {
-    return IPC_FAIL_NO_REASON(this);
-  }
-  RefPtr<IDXGIAdapter> adapter;
-  if (FAILED(dxgi->GetAdapter(getter_AddRefs(adapter))) || !adapter) {
-    return IPC_FAIL_NO_REASON(this);
-  }
-
-  DXGI_ADAPTER_DESC desc;
-  if (FAILED(adapter->GetDesc(&desc))) {
-    return IPC_FAIL_NO_REASON(this);
-  }
-
-  *aOutDesc = DxgiAdapterDesc::From(desc);
 #endif
   return IPC_OK();
 }
@@ -425,7 +399,7 @@ PluginInstanceParent::AnswerNPN_SetValue_NPPVpluginDrawingModel(
       allowed = true;
       break;
     case NPDrawingModelAsyncWindowsDXGISurface:
-      allowed = AllowDirectDXGISurfaceDrawing();
+      allowed = SupportsPluginDirectDXGISurfaceDrawing();
       break;
 #elif defined(MOZ_X11)
     case NPDrawingModelSyncX:
@@ -564,6 +538,181 @@ mozilla::ipc::IPCResult PluginInstanceParent::RecvRevokeCurrentDirectSurface() {
   return IPC_OK();
 }
 
+#if defined(XP_WIN)
+// Uses the ImageBridge to perform IGPUVideoSurfaceManager operations
+// in the GPU process.
+class AsyncPluginSurfaceManager : public IGPUVideoSurfaceManager {
+ public:
+  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(AsyncPluginSurfaceManager, override)
+
+  already_AddRefed<gfx::SourceSurface> Readback(
+      const SurfaceDescriptorGPUVideo& aSD) override {
+    SurfaceDescriptorPlugin pluginSD = aSD;
+    if (!InImageBridgeChildThread()) {
+      SynchronousTask task("AsyncPluginSurfaceManager readback sync");
+      RefPtr<gfx::SourceSurface> result;
+      ImageBridgeChild::GetSingleton()->GetThread()->Dispatch(
+          NewRunnableFunction("AsyncPluginSurfaceManager readback",
+                              &DoSyncReadback, &pluginSD, &result, &task));
+      task.Wait();
+      return result.forget();
+    }
+
+    return DoReadback(pluginSD);
+  }
+
+  void DeallocateSurfaceDescriptor(
+      const SurfaceDescriptorGPUVideo& aSD) override {
+    SurfaceDescriptorPlugin pluginSD = aSD;
+    if (!InImageBridgeChildThread()) {
+      ImageBridgeChild::GetSingleton()->GetThread()->Dispatch(
+          NewRunnableFunction("AsyncPluginSurfaceManager dealloc", &DoDealloc,
+                              &pluginSD));
+      return;
+    }
+
+    return DoDealloc(&pluginSD);
+  }
+
+  // Set of display surfaces for which the related plugin surface has been
+  // freed.  They are freed when the AsyncPluginSurfaceManager is told it is
+  // safe.
+  static HashSet<WindowsHandle> sOrphanedDisplaySurfaces;
+
+ private:
+  ~AsyncPluginSurfaceManager() {}
+
+  struct SurfaceDescriptorUserData {
+    explicit SurfaceDescriptorUserData(layers::SurfaceDescriptor& aSD)
+        : mSD(aSD) {}
+
+    ~SurfaceDescriptorUserData() {
+      auto ibc = ImageBridgeChild::GetSingleton();
+      if (!ibc) {
+        return;
+      }
+      DestroySurfaceDescriptor(ibc, &mSD);
+    }
+
+    layers::SurfaceDescriptor mSD;
+  };
+
+  static void DeleteSurfaceDescriptorUserData(void* aClosure) {
+    SurfaceDescriptorUserData* sd =
+        reinterpret_cast<SurfaceDescriptorUserData*>(aClosure);
+    delete sd;
+  }
+
+  static already_AddRefed<gfx::SourceSurface> DoReadback(
+      const SurfaceDescriptorPlugin& aSD) {
+    MOZ_ASSERT(InImageBridgeChildThread());
+
+    RefPtr<DataSourceSurface> source;
+    auto ibc = ImageBridgeChild::GetSingleton();
+    if (!ibc) {
+      return nullptr;
+    }
+
+    layers::SurfaceDescriptor dataSD = null_t();
+    ibc->SendReadbackAsyncPluginSurface(aSD, &dataSD);
+    if (!IsSurfaceDescriptorValid(dataSD)) {
+      NS_WARNING("Bad SurfaceDescriptor received in Readback");
+      return nullptr;
+    }
+
+    source = GetSurfaceForDescriptor(dataSD);
+    if (!source) {
+      DestroySurfaceDescriptor(ibc, &dataSD);
+      NS_WARNING("Failed to map SurfaceDescriptor in Readback");
+      return nullptr;
+    }
+
+    static UserDataKey sSurfaceDescriptor;
+    source->AddUserData(&sSurfaceDescriptor,
+                        new SurfaceDescriptorUserData(dataSD),
+                        DeleteSurfaceDescriptorUserData);
+
+    return source.forget();
+  }
+
+  static void DoSyncReadback(const SurfaceDescriptorPlugin* aSD,
+                             RefPtr<gfx::SourceSurface>* aResult,
+                             SynchronousTask* aTask) {
+    AutoCompleteTask act(aTask);
+    *aResult = DoReadback(*aSD);
+  }
+
+  static void DoDealloc(const SurfaceDescriptorPlugin* aSD) {
+    MOZ_ASSERT(InImageBridgeChildThread());
+
+    // If the plugin already Finalized and freed its surface then, since the
+    // compositor is now also done with the display surface, we can free
+    // it too.
+    WindowsHandle handle = aSD->displaySurf().handle();
+    auto surfIt = sOrphanedDisplaySurfaces.lookup(handle);
+    if (!surfIt) {
+      // We wil continue to use the surfaces with future GPUVideoImages.
+      return;
+    }
+
+    sOrphanedDisplaySurfaces.remove(surfIt);
+    auto ibc = ImageBridgeChild::GetSingleton();
+    if (!ibc) {
+      return;
+    }
+    ibc->SendRemoveAsyncPluginSurface(*aSD, true);
+  }
+};
+
+/* static */ HashSet<WindowsHandle>
+    AsyncPluginSurfaceManager::sOrphanedDisplaySurfaces;
+
+void InitDXGISurface(const gfx::SurfaceFormat& aFormat,
+                     const gfx::IntSize& aSize,
+                     SurfaceDescriptorPlugin* aSDPlugin,
+                     SynchronousTask* aTask) {
+  MOZ_ASSERT(InImageBridgeChildThread());
+
+  AutoCompleteTask act(aTask);
+  auto ibc = ImageBridgeChild::GetSingleton();
+  if (!ibc) {
+    return;
+  }
+
+  layers::SurfaceDescriptorPlugin sd;
+  if (!ibc->SendMakeAsyncPluginSurfaces(aFormat, aSize, &sd)) {
+    return;
+  }
+  *aSDPlugin = sd;
+}
+
+void FinalizeDXGISurface(const SurfaceDescriptorPlugin& aSD) {
+  MOZ_ASSERT(InImageBridgeChildThread());
+
+  Unused << AsyncPluginSurfaceManager::sOrphanedDisplaySurfaces.put(
+      aSD.displaySurf().handle());
+
+  auto ibc = ImageBridgeChild::GetSingleton();
+  if (!ibc) {
+    return;
+  }
+  ibc->SendRemoveAsyncPluginSurface(aSD, false);
+}
+
+void CopyDXGISurface(const SurfaceDescriptorPlugin& aSD,
+                     SynchronousTask* aTask) {
+  MOZ_ASSERT(InImageBridgeChildThread());
+
+  AutoCompleteTask act(aTask);
+  auto ibc = ImageBridgeChild::GetSingleton();
+  if (!ibc) {
+    return;
+  }
+  ibc->SendUpdateAsyncPluginSurface(aSD);
+}
+
+#endif  //  defined(XP_WIN)
+
 mozilla::ipc::IPCResult PluginInstanceParent::RecvInitDXGISurface(
     const gfx::SurfaceFormat& format, const gfx::IntSize& size,
     WindowsHandle* outHandle, NPError* outError) {
@@ -571,6 +720,7 @@ mozilla::ipc::IPCResult PluginInstanceParent::RecvInitDXGISurface(
   *outError = NPERR_GENERIC_ERROR;
 
 #if defined(XP_WIN)
+  MOZ_ASSERT(NS_IsMainThread());
   if (format != SurfaceFormat::B8G8R8A8 && format != SurfaceFormat::B8G8R8X8) {
     *outError = NPERR_INVALID_PARAM;
     return IPC_OK();
@@ -580,59 +730,61 @@ mozilla::ipc::IPCResult PluginInstanceParent::RecvInitDXGISurface(
     return IPC_OK();
   }
 
-  ImageContainer* container = GetImageContainer();
-  if (!container) {
+  if (!ImageBridgeChild::GetSingleton()) {
     return IPC_OK();
   }
 
-  RefPtr<ImageBridgeChild> forwarder = ImageBridgeChild::GetSingleton();
-  if (!forwarder) {
+  // Ask the ImageBridge thread to generate two SurfaceDescriptorPlugins --
+  // one for the GPU process to display and one for the Plugin process to
+  // render to.
+  SurfaceDescriptorPlugin sd;
+  SynchronousTask task("SendMakeAsyncPluginSurfaces sync");
+  ImageBridgeChild::GetSingleton()->GetThread()->Dispatch(
+      NewRunnableFunction("SendingMakeAsyncPluginSurfaces", &InitDXGISurface,
+                          format, size, &sd, &task));
+  task.Wait();
+
+  if (!sd.id()) {
+    NS_WARNING("SendMakeAsyncPluginSurfaces failed");
     return IPC_OK();
   }
 
-  RefPtr<ID3D11Device> d3d11 = DeviceManagerDx::Get()->GetContentDevice();
-  if (!d3d11) {
+  WindowsHandle pluginSurfHandle = sd.pluginSurf().handle();
+  bool ok = mAsyncSurfaceMap.put(pluginSurfHandle, AsyncSurfaceInfo{sd, size});
+  if (!ok) {
     return IPC_OK();
   }
 
-  // Create the texture we'll give to the plugin process.
-  HANDLE sharedHandle = 0;
-  RefPtr<ID3D11Texture2D> back;
-  {
-    CD3D11_TEXTURE2D_DESC desc(DXGI_FORMAT_B8G8R8A8_UNORM, size.width,
-                               size.height, 1, 1);
-    desc.MiscFlags = D3D11_RESOURCE_MISC_SHARED_KEYEDMUTEX;
-    desc.BindFlags = D3D11_BIND_SHADER_RESOURCE | D3D11_BIND_RENDER_TARGET;
-    if (FAILED(d3d11->CreateTexture2D(&desc, nullptr, getter_AddRefs(back))) ||
-        !back) {
-      return IPC_OK();
-    }
-
-    RefPtr<IDXGIResource> resource;
-    if (FAILED(back->QueryInterface(IID_IDXGIResource,
-                                    getter_AddRefs(resource))) ||
-        !resource) {
-      return IPC_OK();
-    }
-    if (FAILED(resource->GetSharedHandle(&sharedHandle) || !sharedHandle)) {
-      return IPC_OK();
-    }
-  }
-
-  RefPtr<D3D11SurfaceHolder> holder =
-      new D3D11SurfaceHolder(back, format, size);
-  mD3D11Surfaces.Put(reinterpret_cast<void*>(sharedHandle), holder);
-
-  *outHandle = reinterpret_cast<uintptr_t>(sharedHandle);
+  *outHandle = pluginSurfHandle;
   *outError = NPERR_NO_ERROR;
 #endif
   return IPC_OK();
 }
 
 mozilla::ipc::IPCResult PluginInstanceParent::RecvFinalizeDXGISurface(
-    const WindowsHandle& handle) {
+    const WindowsHandle& pluginSurfHandle) {
 #if defined(XP_WIN)
-  mD3D11Surfaces.Remove(reinterpret_cast<void*>(handle));
+  MOZ_ASSERT(NS_IsMainThread());
+
+  if (!ImageBridgeChild::GetSingleton()) {
+    return IPC_OK();
+  }
+
+  auto asiIt = mAsyncSurfaceMap.lookup(pluginSurfHandle);
+  if (!asiIt) {
+    NS_WARNING("Plugin surface did not exist to finalize");
+    return IPC_OK();
+  }
+
+  AsyncSurfaceInfo& asi = asiIt->value();
+
+  // Release the plugin surface but keep the display surface since it may
+  // still be displayed.  Also let the display surface know that it should
+  // not receive further requests to copy from the plugin surface.
+  ImageBridgeChild::GetSingleton()->GetThread()->Dispatch(NewRunnableFunction(
+      "SendingRemoveAsyncPluginSurface", &FinalizeDXGISurface, asi.mSD));
+
+  mAsyncSurfaceMap.remove(asiIt);
 #endif
   return IPC_OK();
 }
@@ -726,48 +878,42 @@ void PluginInstanceParent::SetCurrentImage(Image* aImage) {
 }
 
 mozilla::ipc::IPCResult PluginInstanceParent::RecvShowDirectDXGISurface(
-    const WindowsHandle& handle, const gfx::IntRect& dirty) {
+    const WindowsHandle& pluginSurfHandle, const gfx::IntRect& dirty) {
 #if defined(XP_WIN)
-  RefPtr<D3D11SurfaceHolder> surface;
-  if (!mD3D11Surfaces.Get(reinterpret_cast<void*>(handle),
-                          getter_AddRefs(surface))) {
-    return IPC_FAIL_NO_REASON(this);
-  }
-  if (!surface->IsValid()) {
-    return IPC_FAIL_NO_REASON(this);
+  MOZ_ASSERT(NS_IsMainThread());
+
+  if (!ImageBridgeChild::GetSingleton()) {
+    return IPC_OK();
   }
 
+  auto asiIt = mAsyncSurfaceMap.lookup(pluginSurfHandle);
+  if (!asiIt) {
+    NS_WARNING("Plugin surface did not exist to finalize");
+    return IPC_OK();
+  }
+
+  AsyncSurfaceInfo& asi = asiIt->value();
+
+  // Tell the ImageBridge to copy from the plugin surface to the display surface
+  SynchronousTask task("SendUpdateAsyncPluginSurface sync");
+  ImageBridgeChild::GetSingleton()->GetThread()->Dispatch(NewRunnableFunction(
+      "SendingUpdateAsyncPluginSurface", &CopyDXGISurface, asi.mSD, &task));
+  task.Wait();
+
+  // Make sure we have an ImageContainer for SetCurrentImage.
   ImageContainer* container = GetImageContainer();
   if (!container) {
-    return IPC_FAIL_NO_REASON(this);
+    return IPC_OK();
   }
 
-  RefPtr<TextureClientRecycleAllocator> allocator =
-      mParent->EnsureTextureAllocatorForDXGISurface();
-  RefPtr<TextureClient> texture = allocator->CreateOrRecycle(
-      surface->GetFormat(), surface->GetSize(), BackendSelector::Content,
-      TextureFlags::NO_FLAGS, ALLOC_FOR_OUT_OF_BAND_CONTENT);
-  if (!texture) {
-    NS_WARNING("Could not allocate a TextureClient for plugin!");
-    return IPC_FAIL_NO_REASON(this);
-  }
+  SetCurrentImage(
+      new GPUVideoImage(new AsyncPluginSurfaceManager(), asi.mSD, asi.mSize));
 
-  surface->CopyToTextureClient(texture);
-
-  gfx::IntSize size(surface->GetSize());
-  gfx::IntRect pictureRect(gfx::IntPoint(0, 0), size);
-
-  // Wrap the texture in an image and ship it off.
-  RefPtr<TextureWrapperImage> image =
-      new TextureWrapperImage(texture, pictureRect);
-  SetCurrentImage(image);
-
-  PLUGIN_LOG_DEBUG(("   (RecvShowDirect3D10Surface received handle=%p rect=%s)",
-                    reinterpret_cast<void*>(handle), Stringify(dirty).c_str()));
-  return IPC_OK();
-#else
-  return IPC_FAIL_NO_REASON(this);
+  PLUGIN_LOG_DEBUG(("   (RecvShowDirectDXGISurface received handle=%p rect=%s)",
+                    reinterpret_cast<void*>(pluginSurfHandle),
+                    Stringify(dirty).c_str()));
 #endif
+  return IPC_OK();
 }
 
 mozilla::ipc::IPCResult PluginInstanceParent::RecvShow(
@@ -832,9 +978,9 @@ mozilla::ipc::IPCResult PluginInstanceParent::RecvShow(
 #endif
 
   if (mFrontSurface) {
-  // This is the "old front buffer" we're about to hand back to
-  // the plugin.  We might still have drawing operations
-  // referencing it.
+    // This is the "old front buffer" we're about to hand back to
+    // the plugin.  We might still have drawing operations
+    // referencing it.
 #ifdef MOZ_X11
     if (mFrontSurface->GetType() == gfxSurfaceType::Xlib) {
       // Finish with the surface and XSync here to ensure the server has
@@ -850,8 +996,8 @@ mozilla::ipc::IPCResult PluginInstanceParent::RecvShow(
   }
 
   if (mFrontSurface && gfxSharedImageSurface::IsSharedImage(mFrontSurface))
-    *prevSurface =
-        static_cast<gfxSharedImageSurface*>(mFrontSurface.get())->GetShmem();
+    *prevSurface = std::move(
+        static_cast<gfxSharedImageSurface*>(mFrontSurface.get())->GetShmem());
   else
     *prevSurface = null_t();
 
@@ -865,8 +1011,7 @@ mozilla::ipc::IPCResult PluginInstanceParent::RecvShow(
 
     bool isPlugin = true;
     RefPtr<gfx::SourceSurface> sourceSurface =
-        gfxPlatform::GetPlatform()->GetSourceSurfaceForSurface(nullptr, surface,
-                                                               isPlugin);
+        gfxPlatform::GetSourceSurfaceForSurface(nullptr, surface, isPlugin);
     RefPtr<SourceSurfaceImage> image =
         new SourceSurfaceImage(surface->GetSize(), sourceSurface);
 
@@ -909,10 +1054,20 @@ nsresult PluginInstanceParent::AsyncSetWindow(NPWindow* aWindow) {
   MaybeCreateChildPopupSurrogate();
 #endif
 
-  if (!SendAsyncSetWindow(
-          gfxPlatform::GetPlatform()->ScreenReferenceSurface()->GetType(),
-          window))
+#if defined(OS_WIN)
+  // Windows async surfaces must be Win32.  In particular, it is incompatible
+  // with in-memory surface types.
+  gfxSurfaceType surfType = gfxSurfaceType::Win32;
+#else
+  gfxSurfaceType surfType =
+      gfxPlatform::GetPlatform()->ScreenReferenceSurface()->GetType();
+#endif
+
+  if (surfType == (gfxSurfaceType)-1) {
     return NS_ERROR_FAILURE;
+  }
+
+  if (!SendAsyncSetWindow(surfType, window)) return NS_ERROR_FAILURE;
 
   return NS_OK;
 }
@@ -1053,9 +1208,8 @@ nsresult PluginInstanceParent::BeginUpdateBackground(const nsIntRect& aRect,
              "Update outside of background area");
 #endif
 
-  RefPtr<gfx::DrawTarget> dt =
-      gfxPlatform::GetPlatform()->CreateDrawTargetForSurface(
-          mBackground, gfx::IntSize(sz.width, sz.height));
+  RefPtr<gfx::DrawTarget> dt = gfxPlatform::CreateDrawTargetForSurface(
+      mBackground, gfx::IntSize(sz.width, sz.height));
   dt.forget(aDrawTarget);
 
   return NS_OK;
@@ -1076,7 +1230,7 @@ nsresult PluginInstanceParent::EndUpdateBackground(const nsIntRect& aRect) {
   // view of its background, but it does mean that the plugin is
   // drawing onto pixels no older than those in the latest
   // EndUpdateBackground().
-  XSync(DefaultXDisplay(), False);
+  XSync(DefaultXDisplay(), X11False);
 #endif
 
   Unused << SendUpdateBackground(BackgroundDescriptor(), aRect);
@@ -1162,7 +1316,7 @@ PluginInstanceParent::BackgroundDescriptor() {
              "Expected shared image surface");
   gfxSharedImageSurface* shmem =
       static_cast<gfxSharedImageSurface*>(mBackground.get());
-  return shmem->GetShmem();
+  return mozilla::plugins::SurfaceDescriptor(std::move(shmem->GetShmem()));
 #endif
 
   // If this is ever used, which it shouldn't be, it will trigger a
@@ -1431,12 +1585,16 @@ int16_t PluginInstanceParent::NPP_HandleEvent(void* event) {
         // which fires WM_KILLFOCUS. Delayed delivery causes Flash to
         // misinterpret the event, dropping back out of fullscreen. Trap
         // this event and drop it.
-        wchar_t szClass[26];
-        HWND hwnd = GetForegroundWindow();
-        if (hwnd && hwnd != mPluginHWND &&
-            GetClassNameW(hwnd, szClass, sizeof(szClass) / sizeof(char16_t)) &&
-            !wcscmp(szClass, kFlashFullscreenClass)) {
-          return 0;
+        // mPluginHWND is always NULL for non-windowed plugins.
+        if (mPluginHWND) {
+          wchar_t szClass[26];
+          HWND hwnd = GetForegroundWindow();
+          if (hwnd && hwnd != mPluginHWND &&
+              GetClassNameW(hwnd, szClass,
+                            sizeof(szClass) / sizeof(char16_t)) &&
+              !wcscmp(szClass, kFlashFullscreenClass)) {
+            return 0;
+          }
         }
       } break;
 
@@ -1479,7 +1637,7 @@ int16_t PluginInstanceParent::NPP_HandleEvent(void* event) {
       // Release any active pointer grab so that the plugin X client can
       // grab the pointer if it wishes.
       Display* dpy = DefaultXDisplay();
-#ifdef MOZ_WIDGET_GTK
+#  ifdef MOZ_WIDGET_GTK
       // GDK attempts to (asynchronously) track whether there is an active
       // grab so ungrab through GDK.
       //
@@ -1491,11 +1649,11 @@ int16_t PluginInstanceParent::NPP_HandleEvent(void* event) {
       } else {
         gdk_pointer_ungrab(npevent->xbutton.time);
       }
-#else
+#  else
       XUngrabPointer(dpy, npevent->xbutton.time);
-#endif
+#  endif
       // Wait for the ungrab to complete.
-      XSync(dpy, False);
+      XSync(dpy, X11False);
       break;
   }
 #endif
@@ -1553,8 +1711,8 @@ int16_t PluginInstanceParent::NPP_HandleEvent(void* event) {
         return false;
       }
 
-      if (!CallNPP_HandleEvent_Shmem(npremoteevent, mShSurface, &handled,
-                                     &mShSurface))
+      if (!CallNPP_HandleEvent_Shmem(npremoteevent, std::move(mShSurface),
+                                     &handled, &mShSurface))
         return false;  // no good way to handle errors here...
 
       if (!mShSurface.IsReadable()) {
@@ -1845,8 +2003,7 @@ mozilla::ipc::IPCResult PluginInstanceParent::RecvSetNetscapeWindowAsParent(
 
   return IPC_OK();
 #else
-  NS_NOTREACHED(
-      "PluginInstanceParent::RecvSetNetscapeWindowAsParent not implemented!");
+  MOZ_ASSERT_UNREACHABLE("RecvSetNetscapeWindowAsParent not implemented!");
   return IPC_FAIL_NO_REASON(this);
 #endif
 }
@@ -1878,7 +2035,8 @@ LRESULT CALLBACK PluginInstanceParent::PluginWindowHookProc(HWND hWnd,
   PluginInstanceParent* self = reinterpret_cast<PluginInstanceParent*>(
       ::GetPropW(hWnd, kPluginInstanceParentProperty));
   if (!self) {
-    NS_NOTREACHED("PluginInstanceParent::PluginWindowHookProc null this ptr!");
+    MOZ_ASSERT_UNREACHABLE(
+        "PluginInstanceParent::PluginWindowHookProc null this ptr!");
     return DefWindowProc(hWnd, message, wParam, lParam);
   }
 
@@ -1896,7 +2054,7 @@ LRESULT CALLBACK PluginInstanceParent::PluginWindowHookProc(HWND hWnd,
   }
 
   if (self->mPluginWndProc == PluginWindowHookProc) {
-    NS_NOTREACHED(
+    MOZ_ASSERT_UNREACHABLE(
         "PluginWindowHookProc invoking mPluginWndProc w/"
         "mPluginWndProc == PluginWindowHookProc????");
     return DefWindowProc(hWnd, message, wParam, lParam);
@@ -1944,10 +2102,10 @@ void PluginInstanceParent::UnsubclassPluginWindow() {
   if (XRE_IsContentProcess()) {
     if (mPluginHWND) {
       // Remove 'this' from the plugin list safely
-      nsAutoPtr<PluginInstanceParent> tmp;
+      mozilla::UniquePtr<PluginInstanceParent> tmp;
       MOZ_ASSERT(sPluginInstanceList);
       sPluginInstanceList->Remove((void*)mPluginHWND, &tmp);
-      tmp.forget();
+      mozilla::Unused << tmp.release();
       if (!sPluginInstanceList->Count()) {
         delete sPluginInstanceList;
         sPluginInstanceList = nullptr;
@@ -2059,7 +2217,7 @@ mozilla::ipc::IPCResult PluginInstanceParent::AnswerPluginFocusChange(
     nsPluginInstanceOwner* owner = GetOwner();
     if (owner) {
       nsIFocusManager* fm = nsFocusManager::GetFocusManager();
-      nsCOMPtr<nsIDOMElement> element;
+      RefPtr<dom::Element> element;
       owner->GetDOMElement(getter_AddRefs(element));
       if (fm && element) {
         fm->SetFocus(element, 0);
@@ -2068,8 +2226,7 @@ mozilla::ipc::IPCResult PluginInstanceParent::AnswerPluginFocusChange(
   }
   return IPC_OK();
 #else
-  NS_NOTREACHED(
-      "PluginInstanceParent::AnswerPluginFocusChange not implemented!");
+  MOZ_ASSERT_UNREACHABLE("AnswerPluginFocusChange not implemented!");
   return IPC_FAIL_NO_REASON(this);
 #endif
 }
