@@ -12,7 +12,6 @@
 #include "nsIContentSecurityPolicy.h"
 #include "nsNetUtil.h"
 #include "mozilla/dom/nsCSPContext.h"
-#include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
 #include "nsStringFwd.h"
 
@@ -21,7 +20,7 @@
  * parser functionality directly in compiled code tests.
  * All the tests (except the fuzzy tests at the end) follow the same schemata:
  *   a) create an nsIContentSecurityPolicy object
- *   b) set the selfURI in SetRequestContext
+ *   b) set the selfURI in SetRequestContextWithPrincipal
  *   c) append one or more policies by calling AppendPolicy
  *   d) check if the policy count is correct by calling GetPolicyCount
  *   e) compare the result of the policy with the expected output
@@ -66,10 +65,10 @@ struct PolicyTest {
   char expectedResult[kMaxPolicyLength];
 };
 
-nsresult runTest(uint32_t aExpectedPolicyCount,  // this should be 0 for
-                                                 // policies which should fail
-                                                 // to parse
-                 const char* aPolicy, const char* aExpectedResult) {
+nsresult runTest(
+    uint32_t aExpectedPolicyCount,  // this should be 0 for policies which
+                                    // should fail to parse
+    const char* aPolicy, const char* aExpectedResult) {
   nsresult rv;
 
   // we init the csp with http://www.selfuri.com
@@ -80,7 +79,7 @@ nsresult runTest(uint32_t aExpectedPolicyCount,  // this should be 0 for
   nsCOMPtr<nsIPrincipal> selfURIPrincipal;
   mozilla::OriginAttributes attrs;
   selfURIPrincipal =
-      mozilla::BasePrincipal::CreateCodebasePrincipal(selfURI, attrs);
+      mozilla::BasePrincipal::CreateContentPrincipal(selfURI, attrs);
   NS_ENSURE_TRUE(selfURIPrincipal, NS_ERROR_FAILURE);
 
   // create a CSP object
@@ -90,7 +89,8 @@ nsresult runTest(uint32_t aExpectedPolicyCount,  // this should be 0 for
 
   // for testing the parser we only need to set a principal which is needed
   // to translate the keyword 'self' into an actual URI.
-  rv = csp->SetRequestContext(nullptr, selfURIPrincipal);
+  rv = csp->SetRequestContextWithPrincipal(selfURIPrincipal, selfURI,
+                                           EmptyString(), 0);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // append a policy
@@ -148,6 +148,7 @@ nsresult runTestSuite(const PolicyTest* aPolicies, uint32_t aPolicyCount,
   nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
   bool experimentalEnabledCache = false;
   bool strictDynamicEnabledCache = false;
+  bool navigateTo = false;
   if (prefs) {
     prefs->GetBoolPref("security.csp.experimentalEnabled",
                        &experimentalEnabledCache);
@@ -156,6 +157,9 @@ nsresult runTestSuite(const PolicyTest* aPolicies, uint32_t aPolicyCount,
     prefs->GetBoolPref("security.csp.enableStrictDynamic",
                        &strictDynamicEnabledCache);
     prefs->SetBoolPref("security.csp.enableStrictDynamic", true);
+
+    prefs->GetBoolPref("security.csp.enableNavigateTo", &navigateTo);
+    prefs->SetBoolPref("security.csp.enableNavigateTo", true);
   }
 
   for (uint32_t i = 0; i < aPolicyCount; i++) {
@@ -169,6 +173,7 @@ nsresult runTestSuite(const PolicyTest* aPolicies, uint32_t aPolicyCount,
                        experimentalEnabledCache);
     prefs->SetBoolPref("security.csp.enableStrictDynamic",
                        strictDynamicEnabledCache);
+    prefs->SetBoolPref("security.csp.enableNavigateTo", navigateTo);
   }
 
   return NS_OK;
@@ -176,7 +181,8 @@ nsresult runTestSuite(const PolicyTest* aPolicies, uint32_t aPolicyCount,
 
 // ============================= TestDirectives ========================
 
-TEST(CSPParser, Directives) {
+TEST(CSPParser, Directives)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "connect-src xn--mnchen-3ya.de",
@@ -209,20 +215,24 @@ TEST(CSPParser, Directives) {
       "script-src 'sha256-a'" },
     { "script-src 'sha256-siVR8vAcqP06h2ppeNwqgjr0yZ6yned4X2VF84j4GmI='",
       "script-src 'sha256-siVR8vAcqP06h2ppeNwqgjr0yZ6yned4X2VF84j4GmI='" },
-    { "referrer no-referrer",
-      "referrer no-referrer" },
-    { "require-sri-for script style",
-      "require-sri-for script style"},
     { "script-src 'nonce-foo' 'unsafe-inline' ",
       "script-src 'nonce-foo' 'unsafe-inline'" },
     { "script-src 'nonce-foo' 'strict-dynamic' 'unsafe-inline' https:  ",
       "script-src 'nonce-foo' 'strict-dynamic' 'unsafe-inline' https:" },
+    { "script-src 'nonce-foo' 'strict-dynamic' 'unsafe-inline' 'report-sample' https:  ",
+      "script-src 'nonce-foo' 'strict-dynamic' 'unsafe-inline' 'report-sample' https:" },
     { "default-src 'sha256-siVR8' 'strict-dynamic' 'unsafe-inline' https:  ",
       "default-src 'sha256-siVR8' 'unsafe-inline' https:" },
     { "worker-src https://example.com",
       "worker-src https://example.com" },
     { "worker-src http://worker.com; frame-src http://frame.com; child-src http://child.com",
       "worker-src http://worker.com; frame-src http://frame.com; child-src http://child.com" },
+    { "navigate-to http://example.com",
+      "navigate-to http://example.com"},
+    { "navigate-to 'unsafe-allow-redirects' http://example.com",
+      "navigate-to 'unsafe-allow-redirects' http://example.com"},
+    { "script-src 'unsafe-allow-redirects' http://example.com",
+      "script-src http://example.com"},
       // clang-format on
   };
 
@@ -232,11 +242,12 @@ TEST(CSPParser, Directives) {
 
 // ============================= TestKeywords ========================
 
-TEST(CSPParser, Keywords) {
+TEST(CSPParser, Keywords)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "script-src 'self'",
-      "script-src http://www.selfuri.com" },
+      "script-src 'self'" },
     { "script-src 'unsafe-inline'",
       "script-src 'unsafe-inline'" },
     { "script-src 'unsafe-eval'",
@@ -246,7 +257,7 @@ TEST(CSPParser, Keywords) {
     { "script-src 'none'",
       "script-src 'none'" },
     { "img-src 'none'; script-src 'unsafe-eval' 'unsafe-inline'; default-src 'self'",
-      "img-src 'none'; script-src 'unsafe-eval' 'unsafe-inline'; default-src http://www.selfuri.com" },
+      "img-src 'none'; script-src 'unsafe-eval' 'unsafe-inline'; default-src 'self'" },
       // clang-format on
   };
 
@@ -256,11 +267,12 @@ TEST(CSPParser, Keywords) {
 
 // =================== TestIgnoreUpperLowerCasePolicies ==============
 
-TEST(CSPParser, IgnoreUpperLowerCasePolicies) {
+TEST(CSPParser, IgnoreUpperLowerCasePolicies)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "script-src 'SELF'",
-      "script-src http://www.selfuri.com" },
+      "script-src 'self'" },
     { "sCriPt-src 'Unsafe-Inline'",
       "script-src 'unsafe-inline'" },
     { "SCRIPT-src 'unsafe-eval'",
@@ -270,7 +282,7 @@ TEST(CSPParser, IgnoreUpperLowerCasePolicies) {
     { "script-src 'NoNe'",
       "script-src 'none'" },
     { "img-sRc 'noNe'; scrIpt-src 'unsafe-EVAL' 'UNSAFE-inline'; deFAULT-src 'Self'",
-      "img-src 'none'; script-src 'unsafe-eval' 'unsafe-inline'; default-src http://www.selfuri.com" },
+      "img-src 'none'; script-src 'unsafe-eval' 'unsafe-inline'; default-src 'self'" },
     { "default-src HTTP://www.example.com",
       "default-src http://www.example.com" },
     { "default-src HTTP://WWW.EXAMPLE.COM",
@@ -285,14 +297,10 @@ TEST(CSPParser, IgnoreUpperLowerCasePolicies) {
       "script-src 'nonce-NONCENEEDSTOBEUPPERCASE'" },
     { "script-src 'SHA256-siVR8vAcqP06h2ppeNwqgjr0yZ6yned4X2VF84j4GmI='",
       "script-src 'sha256-siVR8vAcqP06h2ppeNwqgjr0yZ6yned4X2VF84j4GmI='" },
-    { "refERRer No-refeRRer",
-      "referrer no-referrer" },
     { "upgrade-INSECURE-requests",
       "upgrade-insecure-requests" },
     { "sanDBox alloW-foRMs",
       "sandbox allow-forms"},
-    { "require-SRI-for sCript stYle",
-      "require-sri-for script style"},
       // clang-format on
   };
 
@@ -302,7 +310,8 @@ TEST(CSPParser, IgnoreUpperLowerCasePolicies) {
 
 // ========================= TestPaths ===============================
 
-TEST(CSPParser, Paths) {
+TEST(CSPParser, Paths)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "script-src http://www.example.com",
@@ -400,7 +409,8 @@ TEST(CSPParser, Paths) {
 
 // ======================== TestSimplePolicies =======================
 
-TEST(CSPParser, SimplePolicies) {
+TEST(CSPParser, SimplePolicies)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "default-src *",
@@ -422,9 +432,9 @@ TEST(CSPParser, SimplePolicies) {
     { "script-src 'unsafe-eval' 'unsafe-inline' http://www.example.com",
       "script-src 'unsafe-eval' 'unsafe-inline' http://www.example.com" },
     { "object-src 'self'",
-      "object-src http://www.selfuri.com" },
+      "object-src 'self'" },
     { "style-src http://www.example.com 'self'",
-      "style-src http://www.example.com http://www.selfuri.com" },
+      "style-src http://www.example.com 'self'" },
     { "media-src http://www.example.com http://www.test.com",
       "media-src http://www.example.com http://www.test.com" },
     { "connect-src http://www.test.com example.com *.other.com;",
@@ -434,13 +444,13 @@ TEST(CSPParser, SimplePolicies) {
     { "style-src *.other.com example.com",
       "style-src http://*.other.com http://example.com"},
     { "default-src 'self'; img-src *;",
-      "default-src http://www.selfuri.com; img-src *" },
+      "default-src 'self'; img-src *" },
     { "object-src media1.example.com media2.example.com *.cdn.example.com;",
       "object-src http://media1.example.com http://media2.example.com http://*.cdn.example.com" },
     { "script-src trustedscripts.example.com",
       "script-src http://trustedscripts.example.com" },
     { "script-src 'self' ; default-src trustedscripts.example.com",
-      "script-src http://www.selfuri.com; default-src http://trustedscripts.example.com" },
+      "script-src 'self'; default-src http://trustedscripts.example.com" },
     { "default-src 'none'; report-uri http://localhost:49938/test",
       "default-src 'none'; report-uri http://localhost:49938/test" },
     { "   ;   default-src abc",
@@ -476,11 +486,12 @@ TEST(CSPParser, SimplePolicies) {
 
 // =================== TestPoliciesWithInvalidSrc ====================
 
-TEST(CSPParser, PoliciesWithInvalidSrc) {
+TEST(CSPParser, PoliciesWithInvalidSrc)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "script-src 'self'; SCRIPT-SRC http://www.example.com",
-      "script-src http://www.selfuri.com" },
+      "script-src 'self'" },
     { "script-src 'none' test.com; script-src example.com",
       "script-src http://test.com" },
     { "default-src **",
@@ -557,8 +568,6 @@ TEST(CSPParser, PoliciesWithInvalidSrc) {
       "connect-src 'none'" },
     { "script-src https://foo.com/%$",
       "script-src 'none'" },
-    { "require-SRI-for script elephants",
-      "require-sri-for script"},
     { "sandbox    foo",
       "sandbox"},
       // clang-format on
@@ -571,7 +580,8 @@ TEST(CSPParser, PoliciesWithInvalidSrc) {
 
 // ============================= TestBadPolicies =======================
 
-TEST(CSPParser, BadPolicies) {
+TEST(CSPParser, BadPolicies)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "script-sr 'self", "" },
@@ -580,12 +590,10 @@ TEST(CSPParser, BadPolicies) {
     { "defaut-src asdf", "" },
     { "default-src: aaa", "" },
     { "asdf http://test.com", ""},
-    { "referrer", ""},
-    { "referrer foo", ""},
-    { "require-sri-for", ""},
-    { "require-sri-for foo", ""},
     { "report-uri", ""},
     { "report-uri http://:foo", ""},
+    { "require-sri-for", ""},
+    { "require-sri-for style", ""},
       // clang-format on
   };
 
@@ -595,11 +603,12 @@ TEST(CSPParser, BadPolicies) {
 
 // ======================= TestGoodGeneratedPolicies =================
 
-TEST(CSPParser, GoodGeneratedPolicies) {
+TEST(CSPParser, GoodGeneratedPolicies)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "default-src 'self'; img-src *",
-      "default-src http://www.selfuri.com; img-src *" },
+      "default-src 'self'; img-src *" },
     { "report-uri /policy",
       "report-uri http://www.selfuri.com/policy"},
     { "img-src *",
@@ -701,7 +710,7 @@ TEST(CSPParser, GoodGeneratedPolicies) {
     { "default-src bar.com; script-src https://foo.com",
       "default-src http://bar.com; script-src https://foo.com" },
     { "default-src 'self'; script-src 'self' https://*:*",
-      "default-src http://www.selfuri.com; script-src http://www.selfuri.com https://*:*" },
+      "default-src 'self'; script-src 'self' https://*:*" },
     { "img-src http://self.com:34",
       "img-src http://self.com:34" },
     { "media-src http://subd.self.com:34",
@@ -763,7 +772,7 @@ TEST(CSPParser, GoodGeneratedPolicies) {
     { "font-src http://self.com/",
       "font-src http://self.com/" },
     { "script-src 'self'",
-      "script-src http://www.selfuri.com" },
+      "script-src 'self'" },
     { "default-src http://self.com/foo.png",
       "default-src http://self.com/foo.png" },
     { "script-src http://self.com/foo.js",
@@ -777,25 +786,25 @@ TEST(CSPParser, GoodGeneratedPolicies) {
     { "media-src http",
       "media-src http://http" },
     { "frame-src 'SELF'",
-      "frame-src http://www.selfuri.com" },
+      "frame-src 'self'" },
     { "DEFAULT-src 'self';",
-      "default-src http://www.selfuri.com" },
+      "default-src 'self'" },
     { "default-src 'self' http://FOO.COM",
-      "default-src http://www.selfuri.com http://foo.com" },
+      "default-src 'self' http://foo.com" },
     { "default-src 'self' HTTP://foo.com",
-      "default-src http://www.selfuri.com http://foo.com" },
+      "default-src 'self' http://foo.com" },
     { "default-src 'NONE'",
       "default-src 'none'" },
     { "script-src policy-uri ",
       "script-src http://policy-uri" },
     { "img-src 'self'; ",
-      "img-src http://www.selfuri.com" },
+      "img-src 'self'" },
     { "frame-ancestors foo-bar.com",
       "frame-ancestors http://foo-bar.com" },
     { "frame-ancestors http://a.com",
       "frame-ancestors http://a.com" },
     { "frame-ancestors 'self'",
-      "frame-ancestors http://www.selfuri.com" },
+      "frame-ancestors 'self'" },
     { "frame-ancestors http://self.com:88",
       "frame-ancestors http://self.com:88" },
     { "frame-ancestors http://a.b.c.d.e.f.g.h.i.j.k.l.x.com",
@@ -805,13 +814,13 @@ TEST(CSPParser, GoodGeneratedPolicies) {
     { "frame-ancestors http://sampleuser:samplepass@example.com",
       "frame-ancestors 'none'" },
     { "default-src 'none'; frame-ancestors 'self'",
-      "default-src 'none'; frame-ancestors http://www.selfuri.com" },
+      "default-src 'none'; frame-ancestors 'self'" },
     { "frame-ancestors http://self:80",
       "frame-ancestors http://self:80" },
     { "frame-ancestors http://self.com/bar",
       "frame-ancestors http://self.com/bar" },
     { "default-src 'self'; frame-ancestors 'self'",
-      "default-src http://www.selfuri.com; frame-ancestors http://www.selfuri.com" },
+      "default-src 'self'; frame-ancestors 'self'" },
     { "frame-ancestors http://bar.com/foo.png",
       "frame-ancestors http://bar.com/foo.png" },
       // clang-format on
@@ -823,7 +832,8 @@ TEST(CSPParser, GoodGeneratedPolicies) {
 
 // ==================== TestBadGeneratedPolicies ====================
 
-TEST(CSPParser, BadGeneratedPolicies) {
+TEST(CSPParser, BadGeneratedPolicies)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "foo.*.bar", ""},
@@ -849,7 +859,8 @@ TEST(CSPParser, BadGeneratedPolicies) {
 
 // ============ TestGoodGeneratedPoliciesForPathHandling =============
 
-TEST(CSPParser, GoodGeneratedPoliciesForPathHandling) {
+TEST(CSPParser, GoodGeneratedPoliciesForPathHandling)
+{
   // Once bug 808292 (Implement path-level host-source matching to CSP)
   // lands we have to update the expected output to include the parsed path
 
@@ -972,7 +983,8 @@ TEST(CSPParser, GoodGeneratedPoliciesForPathHandling) {
 
 // ============== TestBadGeneratedPoliciesForPathHandling ============
 
-TEST(CSPParser, BadGeneratedPoliciesForPathHandling) {
+TEST(CSPParser, BadGeneratedPoliciesForPathHandling)
+{
   static const PolicyTest policies[] = {
       // clang-format off
     { "img-src test1.example.com:88path-1/",
@@ -1001,7 +1013,8 @@ TEST(CSPParser, BadGeneratedPoliciesForPathHandling) {
 // Use a policy, eliminate one character at a time,
 // and feed it as input to the parser.
 
-TEST(CSPParser, ShorteningPolicies) {
+TEST(CSPParser, ShorteningPolicies)
+{
   char pol[] =
       "default-src http://www.sub1.sub2.example.com:88/path1/path2/ "
       "'unsafe-inline' 'none'";
@@ -1018,17 +1031,18 @@ TEST(CSPParser, ShorteningPolicies) {
   }
 }
 
-  // ============================= TestFuzzyPolicies ===================
+// ============================= TestFuzzyPolicies ===================
 
-  // We generate kFuzzyRuns inputs by (pseudo) randomly picking from the 128
-  // ASCII characters; feed them to the parser and verfy that the parser
-  // handles the input gracefully.
-  //
-  // Please note, that by using srand(0) we get deterministic results!
+// We generate kFuzzyRuns inputs by (pseudo) randomly picking from the 128
+// ASCII characters; feed them to the parser and verfy that the parser
+// handles the input gracefully.
+//
+// Please note, that by using srand(0) we get deterministic results!
 
 #if RUN_OFFLINE_TESTS
 
-TEST(CSPParser, FuzzyPolicies) {
+TEST(CSPParser, FuzzyPolicies)
+{
   // init srand with 0 so we get same results
   srand(0);
 
@@ -1052,15 +1066,16 @@ TEST(CSPParser, FuzzyPolicies) {
 
 #endif
 
-  // ======================= TestFuzzyPoliciesIncDir ===================
+// ======================= TestFuzzyPoliciesIncDir ===================
 
-  // In a similar fashion as in TestFuzzyPolicies, we again (pseudo) randomly
-  // generate input for the parser, but this time also include a valid directive
-  // followed by the random input.
+// In a similar fashion as in TestFuzzyPolicies, we again (pseudo) randomly
+// generate input for the parser, but this time also include a valid directive
+// followed by the random input.
 
 #if RUN_OFFLINE_TESTS
 
-TEST(CSPParser, FuzzyPoliciesIncDir) {
+TEST(CSPParser, FuzzyPoliciesIncDir)
+{
   // init srand with 0 so we get same results
   srand(0);
 
@@ -1091,14 +1106,15 @@ TEST(CSPParser, FuzzyPoliciesIncDir) {
 
 #endif
 
-  // ====================== TestFuzzyPoliciesIncDirLimASCII ============
+// ====================== TestFuzzyPoliciesIncDirLimASCII ============
 
-  // Same as TestFuzzyPoliciesIncDir() but using limited ASCII,
-  // which represents more likely input.
+// Same as TestFuzzyPoliciesIncDir() but using limited ASCII,
+// which represents more likely input.
 
 #if RUN_OFFLINE_TESTS
 
-TEST(CSPParser, FuzzyPoliciesIncDirLimASCII) {
+TEST(CSPParser, FuzzyPoliciesIncDirLimASCII)
+{
   char input[] =
       "1234567890"
       "abcdefghijklmnopqrstuvwxyz"
