@@ -10,16 +10,12 @@
 #include "nsContentUtils.h"
 #include "nsDeviceSensors.h"
 
-#include "nsIDOMWindow.h"
 #include "nsPIDOMWindow.h"
-#include "nsIDOMDocument.h"
 #include "nsIScriptObjectPrincipal.h"
-#include "nsIServiceManager.h"
-#include "nsIServiceManager.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/StaticPrefs_device.h"
 #include "mozilla/Attributes.h"
-#include "mozilla/Services.h"
-#include "nsIPermissionManager.h"
+#include "mozilla/dom/BrowsingContext.h"
 #include "mozilla/dom/DeviceLightEvent.h"
 #include "mozilla/dom/DeviceOrientationEvent.h"
 #include "mozilla/dom/DeviceProximityEvent.h"
@@ -33,16 +29,12 @@ using namespace mozilla;
 using namespace mozilla::dom;
 using namespace hal;
 
+class nsIDOMWindow;
+
 #undef near
 
 #define DEFAULT_SENSOR_POLL 100
 
-static bool gPrefSensorsEnabled = false;
-static bool gPrefMotionSensorEnabled = false;
-static bool gPrefOrientationSensorEnabled = false;
-static bool gPrefProximitySensorEnabled = false;
-static bool gPrefAmbientLightSensorEnabled = false;
-
 static const nsTArray<nsIDOMWindow*>::index_type NoIndex =
     nsTArray<nsIDOMWindow*>::NoIndex;
 
@@ -72,7 +64,7 @@ NS_INTERFACE_MAP_END
 NS_IMPL_ADDREF(nsDeviceSensorData)
 NS_IMPL_RELEASE(nsDeviceSensorData)
 
-nsDeviceSensorData::~nsDeviceSensorData() {}
+nsDeviceSensorData::~nsDeviceSensorData() = default;
 
 NS_IMETHODIMP nsDeviceSensorData::GetType(uint32_t* aType) {
   NS_ENSURE_ARG_POINTER(aType);
@@ -103,16 +95,6 @@ NS_IMPL_ISUPPORTS(nsDeviceSensors, nsIDeviceSensors)
 nsDeviceSensors::nsDeviceSensors() {
   mIsUserProximityNear = false;
   mLastDOMMotionEventTime = TimeStamp::Now();
-  Preferences::AddBoolVarCache(&gPrefSensorsEnabled, "device.sensors.enabled",
-                               true);
-  Preferences::AddBoolVarCache(&gPrefMotionSensorEnabled,
-                               "device.sensors.motion.enabled", true);
-  Preferences::AddBoolVarCache(&gPrefOrientationSensorEnabled,
-                               "device.sensors.orientation.enabled", true);
-  Preferences::AddBoolVarCache(&gPrefProximitySensorEnabled,
-                               "device.sensors.proximity.enabled", false);
-  Preferences::AddBoolVarCache(&gPrefAmbientLightSensorEnabled,
-                               "device.sensors.ambientLight.enabled", false);
 
   for (int i = 0; i < NUM_SENSOR_TYPE; i++) {
     nsTArray<nsIDOMWindow*>* windows = new nsTArray<nsIDOMWindow*>();
@@ -158,7 +140,6 @@ class DeviceSensorTestEvent : public Runnable {
     sensorData.values().AppendElement(0.5f);
     sensorData.values().AppendElement(0.5f);
     sensorData.values().AppendElement(0.5f);
-    sensorData.accuracy() = SENSOR_ACCURACY_UNRELIABLE;
     mTarget->Notify(sensorData);
     return NS_OK;
   }
@@ -168,8 +149,6 @@ class DeviceSensorTestEvent : public Runnable {
   uint32_t mType;
 };
 
-static bool sTestSensorEvents = false;
-
 NS_IMETHODIMP nsDeviceSensors::AddWindowListener(uint32_t aType,
                                                  nsIDOMWindow* aWindow) {
   if (!IsSensorAllowedByPref(aType, aWindow)) return NS_OK;
@@ -182,14 +161,7 @@ NS_IMETHODIMP nsDeviceSensors::AddWindowListener(uint32_t aType,
 
   mWindowListeners[aType]->AppendElement(aWindow);
 
-  static bool sPrefCacheInitialized = false;
-  if (!sPrefCacheInitialized) {
-    sPrefCacheInitialized = true;
-    Preferences::AddBoolVarCache(&sTestSensorEvents,
-                                 "device.sensors.test.events", false);
-  }
-
-  if (sTestSensorEvents) {
+  if (StaticPrefs::device_sensors_test_events()) {
     nsCOMPtr<nsIRunnable> event = new DeviceSensorTestEvent(this, aType);
     NS_DispatchToCurrentThread(event);
   }
@@ -217,42 +189,39 @@ NS_IMETHODIMP nsDeviceSensors::RemoveWindowAsListener(nsIDOMWindow* aWindow) {
 }
 
 static bool WindowCannotReceiveSensorEvent(nsPIDOMWindowInner* aWindow) {
-  // Check to see if this window is in the background.  If
-  // it is and it does not have the "background-sensors" permission,
-  // don't send any device motion events to it.
+  // Check to see if this window is in the background.
   if (!aWindow || !aWindow->IsCurrentInnerWindow()) {
     return true;
   }
 
-  bool disabled = aWindow->GetOuterWindow()->IsBackground() ||
-                  !aWindow->IsTopLevelWindowActive();
-  if (!disabled) {
-    nsCOMPtr<nsPIDOMWindowOuter> top = aWindow->GetScriptableTop();
-    nsCOMPtr<nsIScriptObjectPrincipal> sop = do_QueryInterface(aWindow);
-    nsCOMPtr<nsIScriptObjectPrincipal> topSop = do_QueryInterface(top);
-    if (!sop || !topSop) {
-      return true;
-    }
-
-    nsIPrincipal* principal = sop->GetPrincipal();
-    nsIPrincipal* topPrincipal = topSop->GetPrincipal();
-    if (!principal || !topPrincipal) {
-      return true;
-    }
-
-    disabled = !principal->Subsumes(topPrincipal);
-  }
-
+  nsPIDOMWindowOuter* windowOuter = aWindow->GetOuterWindow();
+  bool disabled =
+      windowOuter->IsBackground() || !windowOuter->IsTopLevelWindowActive();
   if (disabled) {
-    nsCOMPtr<nsIPermissionManager> permMgr = services::GetPermissionManager();
-    NS_ENSURE_TRUE(permMgr, true);
-    uint32_t permission = nsIPermissionManager::DENY_ACTION;
-    permMgr->TestPermissionFromWindow(aWindow, "background-sensors",
-                                      &permission);
-    return permission != nsIPermissionManager::ALLOW_ACTION;
+    return true;
   }
 
-  return false;
+  // Check to see if this window is a cross-origin iframe:
+
+  auto topBC = aWindow->GetBrowsingContext()->Top();
+  if (!topBC->IsInProcess()) {
+    return true;
+  }
+
+  nsCOMPtr<nsIScriptObjectPrincipal> sop = do_QueryInterface(aWindow);
+  nsCOMPtr<nsIScriptObjectPrincipal> topSop =
+      do_QueryInterface(topBC->GetDOMWindow());
+  if (!sop || !topSop) {
+    return true;
+  }
+
+  nsIPrincipal* principal = sop->GetPrincipal();
+  nsIPrincipal* topPrincipal = topSop->GetPrincipal();
+  if (!principal || !topPrincipal) {
+    return true;
+  }
+
+  return !principal->Subsumes(topPrincipal);
 }
 
 // Holds the device orientation in Euler angle degrees (azimuth, pitch, roll).
@@ -324,7 +293,7 @@ static Orientation RotationVectorToOrientation(double aX, double aY, double aZ,
 void nsDeviceSensors::Notify(const mozilla::hal::SensorData& aSensorData) {
   uint32_t type = aSensorData.sensor();
 
-  const InfallibleTArray<float>& values = aSensorData.values();
+  const nsTArray<float>& values = aSensorData.values();
   size_t len = values.Length();
   double x = len > 0 ? values[0] : 0.0;
   double y = len > 1 ? values[1] : 0.0;
@@ -346,7 +315,7 @@ void nsDeviceSensors::Notify(const mozilla::hal::SensorData& aSensorData) {
       continue;
     }
 
-    if (nsCOMPtr<nsIDocument> doc = pwindow->GetDoc()) {
+    if (nsCOMPtr<Document> doc = pwindow->GetDoc()) {
       nsCOMPtr<mozilla::dom::EventTarget> target =
           do_QueryInterface(windowListeners[i]);
       if (type == nsIDeviceSensorData::TYPE_ACCELERATION ||
@@ -383,8 +352,7 @@ void nsDeviceSensors::FireDOMLightEvent(mozilla::dom::EventTarget* aTarget,
 
   event->SetTrusted(true);
 
-  bool defaultActionEnabled;
-  aTarget->DispatchEvent(event, &defaultActionEnabled);
+  aTarget->DispatchEvent(*event);
 }
 
 void nsDeviceSensors::FireDOMProximityEvent(mozilla::dom::EventTarget* aTarget,
@@ -400,8 +368,7 @@ void nsDeviceSensors::FireDOMProximityEvent(mozilla::dom::EventTarget* aTarget,
       aTarget, NS_LITERAL_STRING("deviceproximity"), init);
   event->SetTrusted(true);
 
-  bool defaultActionEnabled;
-  aTarget->DispatchEvent(event, &defaultActionEnabled);
+  aTarget->DispatchEvent(*event);
 
   // Some proximity sensors only support a binary near or
   // far measurement. In this case, the sensor should report
@@ -426,8 +393,7 @@ void nsDeviceSensors::FireDOMUserProximityEvent(
 
   event->SetTrusted(true);
 
-  bool defaultActionEnabled;
-  aTarget->DispatchEvent(event, &defaultActionEnabled);
+  aTarget->DispatchEvent(*event);
 }
 
 void nsDeviceSensors::FireDOMOrientationEvent(EventTarget* aTarget,
@@ -445,8 +411,7 @@ void nsDeviceSensors::FireDOMOrientationEvent(EventTarget* aTarget,
     RefPtr<DeviceOrientationEvent> event =
         DeviceOrientationEvent::Constructor(aEventTarget, aType, init);
     event->SetTrusted(true);
-    bool dummy;
-    aEventTarget->DispatchEvent(event, &dummy);
+    aEventTarget->DispatchEvent(*event);
   };
 
   Dispatch(aTarget, aIsAbsolute ? NS_LITERAL_STRING("absolutedeviceorientation")
@@ -469,7 +434,7 @@ void nsDeviceSensors::FireDOMOrientationEvent(EventTarget* aTarget,
   }
 }
 
-void nsDeviceSensors::FireDOMMotionEvent(nsIDocument* doc, EventTarget* target,
+void nsDeviceSensors::FireDOMMotionEvent(Document* doc, EventTarget* target,
                                          uint32_t type, PRTime timestamp,
                                          double x, double y, double z) {
   // Attempt to coalesce events
@@ -477,7 +442,7 @@ void nsDeviceSensors::FireDOMMotionEvent(nsIDocument* doc, EventTarget* target,
       TimeDuration::FromMilliseconds(DEFAULT_SENSOR_POLL);
   bool fireEvent =
       (TimeStamp::Now() > mLastDOMMotionEventTime + sensorPollDuration) ||
-      sTestSensorEvents;
+      StaticPrefs::device_sensors_test_events();
 
   switch (type) {
     case nsIDeviceSensorData::TYPE_LINEAR_ACCELERATION:
@@ -537,8 +502,7 @@ void nsDeviceSensors::FireDOMMotionEvent(nsIDocument* doc, EventTarget* target,
 
   event->SetTrusted(true);
 
-  bool defaultActionEnabled = true;
-  target->DispatchEvent(event, &defaultActionEnabled);
+  target->DispatchEvent(*event);
 
   mLastRotationRate.reset();
   mLastAccelerationIncludingGravity.reset();
@@ -549,12 +513,12 @@ void nsDeviceSensors::FireDOMMotionEvent(nsIDocument* doc, EventTarget* target,
 bool nsDeviceSensors::IsSensorAllowedByPref(uint32_t aType,
                                             nsIDOMWindow* aWindow) {
   // checks "device.sensors.enabled" master pref
-  if (!gPrefSensorsEnabled) {
+  if (!StaticPrefs::device_sensors_enabled()) {
     return false;
   }
 
   nsCOMPtr<nsPIDOMWindowInner> window = do_QueryInterface(aWindow);
-  nsCOMPtr<nsIDocument> doc;
+  nsCOMPtr<Document> doc;
   if (window) {
     doc = window->GetExtantDoc();
   }
@@ -564,36 +528,36 @@ bool nsDeviceSensors::IsSensorAllowedByPref(uint32_t aType,
     case nsIDeviceSensorData::TYPE_ACCELERATION:
     case nsIDeviceSensorData::TYPE_GYROSCOPE:
       // checks "device.sensors.motion.enabled" pref
-      if (!gPrefMotionSensorEnabled) {
+      if (!StaticPrefs::device_sensors_motion_enabled()) {
         return false;
       } else if (doc) {
-        doc->WarnOnceAbout(nsIDocument::eMotionEvent);
+        doc->WarnOnceAbout(Document::eMotionEvent);
       }
       break;
     case nsIDeviceSensorData::TYPE_GAME_ROTATION_VECTOR:
     case nsIDeviceSensorData::TYPE_ORIENTATION:
     case nsIDeviceSensorData::TYPE_ROTATION_VECTOR:
       // checks "device.sensors.orientation.enabled" pref
-      if (!gPrefOrientationSensorEnabled) {
+      if (!StaticPrefs::device_sensors_orientation_enabled()) {
         return false;
       } else if (doc) {
-        doc->WarnOnceAbout(nsIDocument::eOrientationEvent);
+        doc->WarnOnceAbout(Document::eOrientationEvent);
       }
       break;
     case nsIDeviceSensorData::TYPE_PROXIMITY:
       // checks "device.sensors.proximity.enabled" pref
-      if (!gPrefProximitySensorEnabled) {
+      if (!StaticPrefs::device_sensors_proximity_enabled()) {
         return false;
       } else if (doc) {
-        doc->WarnOnceAbout(nsIDocument::eProximityEvent, true);
+        doc->WarnOnceAbout(Document::eProximityEvent, true);
       }
       break;
     case nsIDeviceSensorData::TYPE_LIGHT:
       // checks "device.sensors.ambientLight.enabled" pref
-      if (!gPrefAmbientLightSensorEnabled) {
+      if (!StaticPrefs::device_sensors_ambientLight_enabled()) {
         return false;
       } else if (doc) {
-        doc->WarnOnceAbout(nsIDocument::eAmbientLightEvent, true);
+        doc->WarnOnceAbout(Document::eAmbientLightEvent, true);
       }
       break;
     default:
@@ -605,5 +569,7 @@ bool nsDeviceSensors::IsSensorAllowedByPref(uint32_t aType,
     return true;
   }
 
-  return !nsContentUtils::ShouldResistFingerprinting(window->GetDocShell());
+  nsCOMPtr<nsIScriptObjectPrincipal> soPrincipal = do_QueryInterface(window);
+  return !nsContentUtils::ShouldResistFingerprinting(
+      soPrincipal->GetPrincipal());
 }
