@@ -15,9 +15,13 @@
 #include "mozilla/dom/workerinternals/RuntimeService.h"
 
 namespace mozilla {
+namespace webgpu {
+class Instance;
+}  // namespace webgpu
 namespace dom {
 class Promise;
 class StorageManager;
+class MediaCapabilities;
 
 namespace network {
 class Connection;
@@ -30,6 +34,8 @@ class WorkerNavigator final : public nsWrapperCache {
   NavigatorProperties mProperties;
   RefPtr<StorageManager> mStorageManager;
   RefPtr<network::Connection> mConnection;
+  RefPtr<dom::MediaCapabilities> mMediaCapabilities;
+  RefPtr<webgpu::Instance> mWebGpu;
   bool mOnline;
 
   WorkerNavigator(const NavigatorProperties& aProperties, bool aOnline);
@@ -62,15 +68,12 @@ class WorkerNavigator final : public nsWrapperCache {
   bool TaintEnabled() const { return false; }
 
   void GetLanguage(nsString& aLanguage) const {
-    if (mProperties.mLanguages.Length() >= 1) {
-      aLanguage.Assign(mProperties.mLanguages[0]);
-    } else {
-      aLanguage.Truncate();
-    }
+    MOZ_ASSERT(mProperties.mLanguages.Length() >= 1);
+    aLanguage.Assign(mProperties.mLanguages[0]);
   }
 
   void GetLanguages(nsTArray<nsString>& aLanguages) const {
-    aLanguages = mProperties.mLanguages;
+    aLanguages = mProperties.mLanguages.Clone();
   }
 
   void GetUserAgent(nsString& aUserAgent, CallerType aCallerType,
@@ -88,6 +91,10 @@ class WorkerNavigator final : public nsWrapperCache {
   StorageManager* Storage();
 
   network::Connection* GetConnection(ErrorResult& aRv);
+
+  dom::MediaCapabilities* MediaCapabilities();
+
+  webgpu::Instance* Gpu();
 };
 
 }  // namespace dom
