@@ -1,30 +1,36 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/ArrayUtils.h"
+#include "mozilla/EnumeratedArray.h"
+#include "mozilla/EnumeratedRange.h"
 #include "mozilla/FloatingPoint.h"
+#include "mozilla/MacroArgs.h"
+#include "mozilla/MacroForEach.h"
 
 #include "nsAtom.h"
 #include "nsGkAtoms.h"
 #include "txExecutionState.h"
 #include "txExpr.h"
 #include "txIXPathContext.h"
+#include "txIEXSLTFunctions.h"
 #include "txNodeSet.h"
 #include "txOutputFormat.h"
 #include "txRtfHandler.h"
 #include "txXPathTreeWalker.h"
+#include "nsImportModule.h"
 #include "nsPrintfCString.h"
 #include "nsComponentManagerUtils.h"
 #include "nsContentCID.h"
 #include "nsContentCreatorFunctions.h"
 #include "nsIContent.h"
-#include "nsIDOMDocumentFragment.h"
 #include "txMozillaXMLOutput.h"
 #include "nsTextNode.h"
-#include "mozilla/dom/DocumentFragment.h"
+#include "mozilla/dom/DocumentFragmentBinding.h"
 #include "prtime.h"
+#include "xpcprivate.h"
 
 using namespace mozilla;
 using namespace mozilla::dom;
@@ -35,21 +41,28 @@ class txStylesheetCompilerState;
 // Utility functions
 // ------------------------------------------------------------------
 
-static nsresult convertRtfToNode(txIEvalContext* aContext,
-                                 txResultTreeFragment* aRtf) {
+static Document* getSourceDocument(txIEvalContext* aContext) {
   txExecutionState* es =
       static_cast<txExecutionState*>(aContext->getPrivateContext());
   if (!es) {
     NS_ERROR("Need txExecutionState!");
 
-    return NS_ERROR_UNEXPECTED;
+    return nullptr;
   }
 
   const txXPathNode& document = es->getSourceDocument();
+  return txXPathNativeNode::getDocument(document);
+}
 
-  nsIDocument* doc = txXPathNativeNode::getDocument(document);
-  nsCOMPtr<nsIDOMDocumentFragment> domFragment =
-      new DocumentFragment(doc->NodeInfoManager());
+static nsresult convertRtfToNode(txIEvalContext* aContext,
+                                 txResultTreeFragment* aRtf) {
+  Document* doc = getSourceDocument(aContext);
+  if (!doc) {
+    return NS_ERROR_UNEXPECTED;
+  }
+
+  RefPtr<DocumentFragment> domFragment =
+      new (doc->NodeInfoManager()) DocumentFragment(doc->NodeInfoManager());
 
   txOutputFormat format;
   txMozillaXMLOutput mozHandler(&format, domFragment, true);
@@ -72,18 +85,13 @@ static nsresult convertRtfToNode(txIEvalContext* aContext,
 
 static nsresult createTextNode(txIEvalContext* aContext, nsString& aValue,
                                txXPathNode** aResult) {
-  txExecutionState* es =
-      static_cast<txExecutionState*>(aContext->getPrivateContext());
-  if (!es) {
-    NS_ERROR("Need txExecutionState!");
-
+  Document* doc = getSourceDocument(aContext);
+  if (!doc) {
     return NS_ERROR_UNEXPECTED;
   }
 
-  const txXPathNode& document = es->getSourceDocument();
-
-  nsIDocument* doc = txXPathNativeNode::getDocument(document);
-  nsCOMPtr<nsIContent> text = new nsTextNode(doc->NodeInfoManager());
+  RefPtr<nsTextNode> text =
+      new (doc->NodeInfoManager()) nsTextNode(doc->NodeInfoManager());
 
   nsresult rv = text->SetText(aValue, false);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -94,37 +102,16 @@ static nsresult createTextNode(txIEvalContext* aContext, nsString& aValue,
   return NS_OK;
 }
 
-static already_AddRefed<DocumentFragment> createDocFragment(
-    txIEvalContext* aContext) {
-  txExecutionState* es =
-      static_cast<txExecutionState*>(aContext->getPrivateContext());
-  if (!es) {
-    NS_ERROR("Need txExecutionState!");
-
-    return nullptr;
-  }
-
-  const txXPathNode& document = es->getSourceDocument();
-  nsIDocument* doc = txXPathNativeNode::getDocument(document);
-  RefPtr<DocumentFragment> fragment =
-      new DocumentFragment(doc->NodeInfoManager());
-
-  return fragment.forget();
-}
-
 static nsresult createAndAddToResult(nsAtom* aName, const nsAString& aValue,
                                      txNodeSet* aResultSet,
-                                     nsIContent* aResultHolder) {
-  NS_ASSERTION(aResultHolder->IsNodeOfType(nsINode::eDOCUMENT_FRAGMENT) &&
-                   aResultHolder->OwnerDoc(),
-               "invalid result-holder");
-
-  nsIDocument* doc = aResultHolder->OwnerDoc();
+                                     DocumentFragment* aResultHolder) {
+  Document* doc = aResultHolder->OwnerDoc();
   nsCOMPtr<Element> elem =
       doc->CreateElem(nsDependentAtomString(aName), nullptr, kNameSpaceID_None);
   NS_ENSURE_TRUE(elem, NS_ERROR_NULL_POINTER);
 
-  RefPtr<nsTextNode> text = new nsTextNode(doc->NodeInfoManager());
+  RefPtr<nsTextNode> text =
+      new (doc->NodeInfoManager()) nsTextNode(doc->NodeInfoManager());
 
   nsresult rv = text->SetText(aValue, false);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -135,7 +122,7 @@ static nsresult createAndAddToResult(nsAtom* aName, const nsAString& aValue,
   rv = aResultHolder->AppendChildTo(elem, false);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsAutoPtr<txXPathNode> xpathNode(
+  UniquePtr<txXPathNode> xpathNode(
       txXPathNativeNode::createXPathNode(elem, true));
   NS_ENSURE_TRUE(xpathNode, NS_ERROR_OUT_OF_MEMORY);
 
@@ -153,87 +140,82 @@ static const char* const sTypes[] = {"node-set", "boolean", "number", "string",
 // Function implementations
 // ------------------------------------------------------------------
 
+enum class txEXSLTType {
+  // http://exslt.org/common
+  NODE_SET,
+  OBJECT_TYPE,
+
+  // http://exslt.org/dates-and-times
+  DATE_TIME,
+
+  // http://exslt.org/math
+  MAX,
+  MIN,
+  HIGHEST,
+  LOWEST,
+
+  // http://exslt.org/regular-expressions
+  MATCH,
+  REPLACE,
+  TEST,
+
+  // http://exslt.org/sets
+  DIFFERENCE_,  // not DIFFERENCE to avoid a conflict with a winuser.h macro
+  DISTINCT,
+  HAS_SAME_NODE,
+  INTERSECTION,
+  LEADING,
+  TRAILING,
+
+  // http://exslt.org/strings
+  CONCAT,
+  SPLIT,
+  TOKENIZE,
+
+  _LIMIT,
+};
+
 struct txEXSLTFunctionDescriptor {
   int8_t mMinParams;
   int8_t mMaxParams;
   Expr::ResultType mReturnType;
+  nsStaticAtom* mName;
+  bool (*mCreator)(txEXSLTType, FunctionCall**);
   int32_t mNamespaceID;
-  nsStaticAtom** mName;
-  const char* mNamespaceURI;
 };
 
-static const char kEXSLTCommonNS[] = "http://exslt.org/common";
-static const char kEXSLTSetsNS[] = "http://exslt.org/sets";
-static const char kEXSLTStringsNS[] = "http://exslt.org/strings";
-static const char kEXSLTMathNS[] = "http://exslt.org/math";
-static const char kEXSLTDatesAndTimesNS[] = "http://exslt.org/dates-and-times";
-
-// The order of this table must be the same as the
-// txEXSLTFunctionCall::eType enum
-static txEXSLTFunctionDescriptor descriptTable[] = {
-    {1, 1, Expr::NODESET_RESULT, 0, &nsGkAtoms::nodeSet,
-     kEXSLTCommonNS},  // NODE_SET
-    {1, 1, Expr::STRING_RESULT, 0, &nsGkAtoms::objectType,
-     kEXSLTCommonNS},  // OBJECT_TYPE
-    {2, 2, Expr::NODESET_RESULT, 0, &nsGkAtoms::difference,
-     kEXSLTSetsNS},  // DIFFERENCE
-    {1, 1, Expr::NODESET_RESULT, 0, &nsGkAtoms::distinct,
-     kEXSLTSetsNS},  // DISTINCT
-    {2, 2, Expr::BOOLEAN_RESULT, 0, &nsGkAtoms::hasSameNode,
-     kEXSLTSetsNS},  // HAS_SAME_NODE
-    {2, 2, Expr::NODESET_RESULT, 0, &nsGkAtoms::intersection,
-     kEXSLTSetsNS},  // INTERSECTION
-    {2, 2, Expr::NODESET_RESULT, 0, &nsGkAtoms::leading,
-     kEXSLTSetsNS},  // LEADING
-    {2, 2, Expr::NODESET_RESULT, 0, &nsGkAtoms::trailing,
-     kEXSLTSetsNS},  // TRAILING
-    {1, 1, Expr::STRING_RESULT, 0, &nsGkAtoms::concat,
-     kEXSLTStringsNS},  // CONCAT
-    {1, 2, Expr::STRING_RESULT, 0, &nsGkAtoms::split,
-     kEXSLTStringsNS},  // SPLIT
-    {1, 2, Expr::STRING_RESULT, 0, &nsGkAtoms::tokenize,
-     kEXSLTStringsNS},                                              // TOKENIZE
-    {1, 1, Expr::NUMBER_RESULT, 0, &nsGkAtoms::max, kEXSLTMathNS},  // MAX
-    {1, 1, Expr::NUMBER_RESULT, 0, &nsGkAtoms::min, kEXSLTMathNS},  // MIN
-    {1, 1, Expr::NODESET_RESULT, 0, &nsGkAtoms::highest,
-     kEXSLTMathNS},  // HIGHEST
-    {1, 1, Expr::NODESET_RESULT, 0, &nsGkAtoms::lowest,
-     kEXSLTMathNS},  // LOWEST
-    {0, 0, Expr::STRING_RESULT, 0, &nsGkAtoms::dateTime,
-     kEXSLTDatesAndTimesNS},  // DATE_TIME
-
-};
+static EnumeratedArray<txEXSLTType, txEXSLTType::_LIMIT,
+                       txEXSLTFunctionDescriptor>
+    descriptTable;
 
 class txEXSLTFunctionCall : public FunctionCall {
  public:
-  // The order of this enum must be the same as the descriptTable
-  // table above
-  enum eType {
-    // Set functions
-    NODE_SET,
-    OBJECT_TYPE,
-    DIFFERENCE,
-    DISTINCT,
-    HAS_SAME_NODE,
-    INTERSECTION,
-    LEADING,
-    TRAILING,
-    CONCAT,
-    SPLIT,
-    TOKENIZE,
-    MAX,
-    MIN,
-    HIGHEST,
-    LOWEST,
-    DATE_TIME
-  };
+  explicit txEXSLTFunctionCall(txEXSLTType aType) : mType(aType) {}
 
-  explicit txEXSLTFunctionCall(eType aType) : mType(aType) {}
+  static bool Create(txEXSLTType aType, FunctionCall** aFunction) {
+    *aFunction = new txEXSLTFunctionCall(aType);
+    return true;
+  }
 
   TX_DECL_FUNCTION
 
  private:
-  eType mType;
+  txEXSLTType mType;
+};
+
+class txEXSLTRegExFunctionCall : public FunctionCall {
+ public:
+  explicit txEXSLTRegExFunctionCall(txEXSLTType aType) : mType(aType) {}
+
+  static bool Create(txEXSLTType aType, FunctionCall** aFunction) {
+    *aFunction = new txEXSLTRegExFunctionCall(aType);
+    return true;
+  }
+
+  TX_DECL_FUNCTION
+
+ private:
+  txEXSLTType mType;
 };
 
 nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
@@ -246,7 +228,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
   nsresult rv = NS_OK;
   switch (mType) {
-    case NODE_SET: {
+    case txEXSLTType::NODE_SET: {
       RefPtr<txAExprResult> exprResult;
       rv = mParams[0]->evaluate(aContext, getter_AddRefs(exprResult));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -276,7 +258,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
           nsAutoString value;
           exprResult->stringValue(value);
 
-          nsAutoPtr<txXPathNode> node;
+          UniquePtr<txXPathNode> node;
           rv = createTextNode(aContext, value, getter_Transfers(node));
           NS_ENSURE_SUCCESS(rv, rv);
 
@@ -288,7 +270,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return NS_OK;
     }
-    case OBJECT_TYPE: {
+    case txEXSLTType::OBJECT_TYPE: {
       RefPtr<txAExprResult> exprResult;
       nsresult rv = mParams[0]->evaluate(aContext, getter_AddRefs(exprResult));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -297,14 +279,15 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
       rv = aContext->recycler()->getStringResult(getter_AddRefs(strRes));
       NS_ENSURE_SUCCESS(rv, rv);
 
-      AppendASCIItoUTF16(sTypes[exprResult->getResultType()], strRes->mValue);
+      AppendASCIItoUTF16(MakeStringSpan(sTypes[exprResult->getResultType()]),
+                         strRes->mValue);
 
       NS_ADDREF(*aResult = strRes);
 
       return NS_OK;
     }
-    case DIFFERENCE:
-    case INTERSECTION: {
+    case txEXSLTType::DIFFERENCE_:
+    case txEXSLTType::INTERSECTION: {
       RefPtr<txNodeSet> nodes1;
       rv = evaluateToNodeSet(mParams[0], aContext, getter_AddRefs(nodes1));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -317,7 +300,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
       rv = aContext->recycler()->getNodeSet(getter_AddRefs(resultSet));
       NS_ENSURE_SUCCESS(rv, rv);
 
-      bool insertOnFound = mType == INTERSECTION;
+      bool insertOnFound = mType == txEXSLTType::INTERSECTION;
 
       int32_t searchPos = 0;
       int32_t i, len = nodes1->size();
@@ -338,7 +321,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return NS_OK;
     }
-    case DISTINCT: {
+    case txEXSLTType::DISTINCT: {
       RefPtr<txNodeSet> nodes;
       rv = evaluateToNodeSet(mParams[0], aContext, getter_AddRefs(nodes));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -365,7 +348,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return NS_OK;
     }
-    case HAS_SAME_NODE: {
+    case txEXSLTType::HAS_SAME_NODE: {
       RefPtr<txNodeSet> nodes1;
       rv = evaluateToNodeSet(mParams[0], aContext, getter_AddRefs(nodes1));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -387,8 +370,8 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return NS_OK;
     }
-    case LEADING:
-    case TRAILING: {
+    case txEXSLTType::LEADING:
+    case txEXSLTType::TRAILING: {
       RefPtr<txNodeSet> nodes1;
       rv = evaluateToNodeSet(mParams[0], aContext, getter_AddRefs(nodes1));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -411,7 +394,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
       int32_t end = nodes1->indexOf(nodes2->get(0));
       if (end >= 0) {
         int32_t i = 0;
-        if (mType == TRAILING) {
+        if (mType == txEXSLTType::TRAILING) {
           i = end + 1;
           end = nodes1->size();
         }
@@ -425,7 +408,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return NS_OK;
     }
-    case CONCAT: {
+    case txEXSLTType::CONCAT: {
       RefPtr<txNodeSet> nodes;
       rv = evaluateToNodeSet(mParams[0], aContext, getter_AddRefs(nodes));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -438,8 +421,8 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return aContext->recycler()->getStringResult(str, aResult);
     }
-    case SPLIT:
-    case TOKENIZE: {
+    case txEXSLTType::SPLIT:
+    case txEXSLTType::TOKENIZE: {
       // Evaluate parameters
       nsAutoString string;
       rv = mParams[0]->evaluateToString(aContext, string);
@@ -449,15 +432,18 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
       if (mParams.Length() == 2) {
         rv = mParams[1]->evaluateToString(aContext, pattern);
         NS_ENSURE_SUCCESS(rv, rv);
-      } else if (mType == SPLIT) {
+      } else if (mType == txEXSLTType::SPLIT) {
         pattern.Assign(' ');
       } else {
         pattern.AssignLiteral("\t\r\n ");
       }
 
       // Set up holders for the result
-      RefPtr<DocumentFragment> docFrag = createDocFragment(aContext);
-      NS_ENSURE_STATE(docFrag);
+      Document* sourceDoc = getSourceDocument(aContext);
+      NS_ENSURE_STATE(sourceDoc);
+
+      RefPtr<DocumentFragment> docFrag = new (sourceDoc->NodeInfoManager())
+          DocumentFragment(sourceDoc->NodeInfoManager());
 
       RefPtr<txNodeSet> resultSet;
       rv = aContext->recycler()->getNodeSet(getter_AddRefs(resultSet));
@@ -477,7 +463,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
         }
 
         tailIndex = string.Length();
-      } else if (mType == SPLIT) {
+      } else if (mType == txEXSLTType::SPLIT) {
         nsAString::const_iterator strStart, strEnd;
         string.BeginReading(strStart);
         string.EndReading(strEnd);
@@ -521,8 +507,8 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return NS_OK;
     }
-    case MAX:
-    case MIN: {
+    case txEXSLTType::MAX:
+    case txEXSLTType::MIN: {
       RefPtr<txNodeSet> nodes;
       rv = evaluateToNodeSet(mParams[0], aContext, getter_AddRefs(nodes));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -532,7 +518,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
                                                      aResult);
       }
 
-      bool findMax = mType == MAX;
+      bool findMax = mType == txEXSLTType::MAX;
 
       double res = findMax ? mozilla::NegativeInfinity<double>()
                            : mozilla::PositiveInfinity<double>();
@@ -553,8 +539,8 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return aContext->recycler()->getNumberResult(res, aResult);
     }
-    case HIGHEST:
-    case LOWEST: {
+    case txEXSLTType::HIGHEST:
+    case txEXSLTType::LOWEST: {
       RefPtr<txNodeSet> nodes;
       rv = evaluateToNodeSet(mParams[0], aContext, getter_AddRefs(nodes));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -569,7 +555,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
       rv = aContext->recycler()->getNodeSet(getter_AddRefs(resultSet));
       NS_ENSURE_SUCCESS(rv, rv);
 
-      bool findMax = mType == HIGHEST;
+      bool findMax = mType == txEXSLTType::HIGHEST;
       double res = findMax ? mozilla::NegativeInfinity<double>()
                            : mozilla::PositiveInfinity<double>();
       int32_t i, len = nodes->size();
@@ -597,7 +583,7 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return NS_OK;
     }
-    case DATE_TIME: {
+    case txEXSLTType::DATE_TIME: {
       // http://exslt.org/date/functions/date-time/
 
       PRExplodedTime prtime;
@@ -629,10 +615,14 @@ nsresult txEXSLTFunctionCall::evaluate(txIEvalContext* aContext,
 
       return NS_OK;
     }
+    default: {
+      aContext->receiveError(NS_LITERAL_STRING("Internal error"),
+                             NS_ERROR_UNEXPECTED);
+      return NS_ERROR_UNEXPECTED;
+    }
   }
 
-  aContext->receiveError(NS_LITERAL_STRING("Internal error"),
-                         NS_ERROR_UNEXPECTED);
+  MOZ_ASSERT_UNREACHABLE("Missing return?");
   return NS_ERROR_UNEXPECTED;
 }
 
@@ -641,29 +631,130 @@ Expr::ResultType txEXSLTFunctionCall::getReturnType() {
 }
 
 bool txEXSLTFunctionCall::isSensitiveTo(ContextSensitivity aContext) {
-  if (mType == NODE_SET || mType == SPLIT || mType == TOKENIZE) {
+  if (mType == txEXSLTType::NODE_SET || mType == txEXSLTType::SPLIT ||
+      mType == txEXSLTType::TOKENIZE) {
     return (aContext & PRIVATE_CONTEXT) || argsSensitiveTo(aContext);
   }
   return argsSensitiveTo(aContext);
 }
 
 #ifdef TX_TO_STRING
-nsresult txEXSLTFunctionCall::getNameAtom(nsAtom** aAtom) {
-  NS_ADDREF(*aAtom = *descriptTable[mType].mName);
-  return NS_OK;
+void txEXSLTFunctionCall::appendName(nsAString& aDest) {
+  aDest.Append(descriptTable[mType].mName->GetUTF16String());
+}
+#endif
+
+nsresult txEXSLTRegExFunctionCall::evaluate(txIEvalContext* aContext,
+                                            txAExprResult** aResult) {
+  *aResult = nullptr;
+  if (!requireParams(descriptTable[mType].mMinParams,
+                     descriptTable[mType].mMaxParams, aContext)) {
+    return NS_ERROR_XPATH_BAD_ARGUMENT_COUNT;
+  }
+
+  nsAutoString string;
+  nsresult rv = mParams[0]->evaluateToString(aContext, string);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsAutoString regex;
+  rv = mParams[1]->evaluateToString(aContext, regex);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsAutoString flags;
+  if (mParams.Length() >= 3) {
+    rv = mParams[2]->evaluateToString(aContext, flags);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
+  nsCOMPtr<txIEXSLTFunctions> funcs =
+      do_ImportModule("resource://gre/modules/txEXSLTRegExFunctions.jsm");
+  MOZ_ALWAYS_TRUE(funcs);
+
+  switch (mType) {
+    case txEXSLTType::MATCH: {
+      nsCOMPtr<Document> sourceDoc = getSourceDocument(aContext);
+      NS_ENSURE_STATE(sourceDoc);
+
+      RefPtr<DocumentFragment> docFrag;
+      rv = funcs->Match(string, regex, flags, sourceDoc,
+                        getter_AddRefs(docFrag));
+      NS_ENSURE_SUCCESS(rv, rv);
+      NS_ENSURE_STATE(docFrag);
+
+      RefPtr<txNodeSet> resultSet;
+      rv = aContext->recycler()->getNodeSet(getter_AddRefs(resultSet));
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      UniquePtr<txXPathNode> node;
+      for (nsIContent* result = docFrag->GetFirstChild(); result;
+           result = result->GetNextSibling()) {
+        node = WrapUnique(txXPathNativeNode::createXPathNode(result, true));
+        rv = resultSet->add(*node);
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+
+      resultSet.forget(aResult);
+
+      return NS_OK;
+    }
+    case txEXSLTType::REPLACE: {
+      nsAutoString replace;
+      rv = mParams[3]->evaluateToString(aContext, replace);
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      nsAutoString result;
+      rv = funcs->Replace(string, regex, flags, replace, result);
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      rv = aContext->recycler()->getStringResult(result, aResult);
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      return NS_OK;
+    }
+    case txEXSLTType::TEST: {
+      bool result;
+      rv = funcs->Test(string, regex, flags, &result);
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      aContext->recycler()->getBoolResult(result, aResult);
+
+      return NS_OK;
+    }
+    default: {
+      aContext->receiveError(NS_LITERAL_STRING("Internal error"),
+                             NS_ERROR_UNEXPECTED);
+      return NS_ERROR_UNEXPECTED;
+    }
+  }
+
+  MOZ_ASSERT_UNREACHABLE("Missing return?");
+  return NS_ERROR_UNEXPECTED;
+}
+
+Expr::ResultType txEXSLTRegExFunctionCall::getReturnType() {
+  return descriptTable[mType].mReturnType;
+}
+
+bool txEXSLTRegExFunctionCall::isSensitiveTo(ContextSensitivity aContext) {
+  if (mType == txEXSLTType::MATCH) {
+    return (aContext & PRIVATE_CONTEXT) || argsSensitiveTo(aContext);
+  }
+  return argsSensitiveTo(aContext);
+}
+
+#ifdef TX_TO_STRING
+void txEXSLTRegExFunctionCall::appendName(nsAString& aDest) {
+  aDest.Append(descriptTable[mType].mName->GetUTF16String());
 }
 #endif
 
 extern nsresult TX_ConstructEXSLTFunction(nsAtom* aName, int32_t aNamespaceID,
                                           txStylesheetCompilerState* aState,
                                           FunctionCall** aResult) {
-  uint32_t i;
-  for (i = 0; i < ArrayLength(descriptTable); ++i) {
-    txEXSLTFunctionDescriptor& desc = descriptTable[i];
-    if (aName == *desc.mName && aNamespaceID == desc.mNamespaceID) {
-      *aResult =
-          new txEXSLTFunctionCall(static_cast<txEXSLTFunctionCall::eType>(i));
-      return NS_OK;
+  for (auto i : MakeEnumeratedRange(txEXSLTType::_LIMIT)) {
+    const txEXSLTFunctionDescriptor& desc = descriptTable[i];
+    if (aName == desc.mName && aNamespaceID == desc.mNamespaceID) {
+      return desc.mCreator(i, aResult) ? NS_OK : NS_ERROR_FAILURE;
     }
   }
 
@@ -671,16 +762,55 @@ extern nsresult TX_ConstructEXSLTFunction(nsAtom* aName, int32_t aNamespaceID,
 }
 
 extern bool TX_InitEXSLTFunction() {
-  uint32_t i;
-  for (i = 0; i < ArrayLength(descriptTable); ++i) {
-    txEXSLTFunctionDescriptor& desc = descriptTable[i];
-    NS_ConvertASCIItoUTF16 namespaceURI(desc.mNamespaceURI);
-    desc.mNamespaceID = txNamespaceManager::getNamespaceID(namespaceURI);
-
-    if (desc.mNamespaceID == kNameSpaceID_Unknown) {
-      return false;
-    }
+#define EXSLT_FUNCS(NS, CLASS, ...)                                           \
+  {                                                                           \
+    int32_t nsid = txNamespaceManager::getNamespaceID(NS_LITERAL_STRING(NS)); \
+    if (nsid == kNameSpaceID_Unknown) {                                       \
+      return false;                                                           \
+    }                                                                         \
+    MOZ_FOR_EACH(EXSLT_FUNC, (nsid, CLASS, ), (__VA_ARGS__))                  \
   }
 
+#define EXSLT_FUNC(NS, CLASS, ...)                      \
+  descriptTable[txEXSLTType::MOZ_ARG_1 __VA_ARGS__] = { \
+      MOZ_ARGS_AFTER_1 __VA_ARGS__, CLASS::Create, NS};
+
+  EXSLT_FUNCS("http://exslt.org/common", txEXSLTFunctionCall,
+              (NODE_SET, 1, 1, Expr::NODESET_RESULT, nsGkAtoms::nodeSet),
+              (OBJECT_TYPE, 1, 1, Expr::STRING_RESULT, nsGkAtoms::objectType))
+
+  EXSLT_FUNCS("http://exslt.org/dates-and-times", txEXSLTFunctionCall,
+              (DATE_TIME, 0, 0, Expr::STRING_RESULT, nsGkAtoms::dateTime))
+
+  EXSLT_FUNCS("http://exslt.org/math", txEXSLTFunctionCall,
+              (MAX, 1, 1, Expr::NUMBER_RESULT, nsGkAtoms::max),
+              (MIN, 1, 1, Expr::NUMBER_RESULT, nsGkAtoms::min),
+              (HIGHEST, 1, 1, Expr::NODESET_RESULT, nsGkAtoms::highest),
+              (LOWEST, 1, 1, Expr::NODESET_RESULT, nsGkAtoms::lowest))
+
+  EXSLT_FUNCS("http://exslt.org/regular-expressions", txEXSLTRegExFunctionCall,
+              (MATCH, 2, 3, Expr::NODESET_RESULT, nsGkAtoms::match),
+              (REPLACE, 4, 4, Expr::STRING_RESULT, nsGkAtoms::replace),
+              (TEST, 2, 3, Expr::BOOLEAN_RESULT, nsGkAtoms::test))
+
+  EXSLT_FUNCS(
+      "http://exslt.org/sets", txEXSLTFunctionCall,
+      (DIFFERENCE_, 2, 2, Expr::NODESET_RESULT, nsGkAtoms::difference),
+      (DISTINCT, 1, 1, Expr::NODESET_RESULT, nsGkAtoms::distinct),
+      (HAS_SAME_NODE, 2, 2, Expr::BOOLEAN_RESULT, nsGkAtoms::hasSameNode),
+      (INTERSECTION, 2, 2, Expr::NODESET_RESULT, nsGkAtoms::intersection),
+      (LEADING, 2, 2, Expr::NODESET_RESULT, nsGkAtoms::leading),
+      (TRAILING, 2, 2, Expr::NODESET_RESULT, nsGkAtoms::trailing))
+
+  EXSLT_FUNCS("http://exslt.org/strings", txEXSLTFunctionCall,
+              (CONCAT, 1, 1, Expr::STRING_RESULT, nsGkAtoms::concat),
+              (SPLIT, 1, 2, Expr::STRING_RESULT, nsGkAtoms::split),
+              (TOKENIZE, 1, 2, Expr::STRING_RESULT, nsGkAtoms::tokenize))
+
+#undef EXSLT_FUNCS
+#undef EXSLT_FUNC
+#undef EXSLT_FUNC_HELPER
+#undef EXSLT_FUNC_HELPER2
+
   return true;
 }
